# can't access active_support until bundler or gemcache are required, so path to BUNDLE_GEMFILE  needs to be calculated
# from __FILE__
metasploit_pro_directory = File.dirname(__FILE__)
pro_directory = File.join(metasploit_pro_directory, '..', '..', '..', '..')
pro_directory = File.expand_path(pro_directory)

gemfile_path = File.join(pro_directory, 'ui', 'Gemfile')
ENV['BUNDLE_GEMFILE'] = gemfile_path

ENV['BUNDLE_PATH'] = ENV['GEM_HOME']

# bundler/setup just sets up the $LOAD_PATHs, the gems aren't automatically required...
require 'bundler/setup'

# rails/all must be required explicitly to get the railties that pro/ui/config/application.rb uses
require 'rails/all'
# require all the gems in the current environment
Bundler.require(*Rails.groups(:assets => %w(development test cucumber)))

#
# Core
#
require 'logger'

#
# Project
#
require 'metasploit/framework'
require 'metasploit/pro/engine/arguments'
require 'metasploit/pro/engine/configuration'
require 'metasploit/pro/engine/database'
require 'pro/background_daemon'
require 'pro/client'
require 'pro/filters'
require 'pro/rpc/v10/rpc_pro'

module Metasploit::Pro::Engine
  include Metasploit::Configured
  include Metasploit::Pro::Engine::Arguments
  include Metasploit::Pro::Engine::Database

  #
  # CONSTANTS
  #

  DEFAULT_ENVIRONMENT = 'production'

  class << self
    def configure_database_logging(opts={})

      # msfpro uses the normal msfconsole configuration directory
      return if opts[:msfpro]

      logs_pathname = Metasploit::Pro::Engine.root.join('config', 'logs')
      logs_path = logs_pathname.to_s
      FileUtils.mkdir_p logs_path

      db_log_pathname = logs_pathname.join('db.log')
      db_log_path = db_log_pathname.to_s
      ActiveRecord::Base.logger = Logger.new(db_log_path)

      #
      # Configure the DB log verbosity
      #
      unless configuration.database_debug?
        ActiveRecord::Base.logger.level = Logger::WARN
      end
    end

    #
    # Configure the process environment to work with java and other tools
    #
    def configure_environment(opts={})
      if Metasploit::Pro::Engine.configuration.platform.win32?
        # TODO determine whether this is a workaround for mingw32 or if File::PATH_SEPARATOR will work
        path_separator = ';'
      else
        path_separator = File::PATH_SEPARATOR
      end

      bins = %W{ common/bin ruby/bin nmap/bin postgresql/bin apache2/bin java/bin jruby/bin }
      pathparts = ENV['PATH'].split(path_separator)
      bin_root_pathname = Metasploit::Pro::Engine.root.join('..', '..', '..').expand_path

      bins.each do |bin_path|
        bin_pathname = bin_root_pathname.join(bin_path)
        bin_path = bin_pathname.to_s

        pathparts.unshift bin_path
      end

      ENV['PATH'] = pathparts.uniq.join(path_separator)

      java_home_path = bin_root_pathname.join('java').to_s
      ENV['JAVA_HOME'] = java_home_path

      terminfo_path = bin_root_pathname.join('common', 'share', 'terminfo').to_s
      ENV['TERMINFO'] = terminfo_path

      ENV['DISPLAY'] = nil

      # Convert path delimiters for Windows
      if Metasploit::Pro::Engine.configuration.platform.win32?
        ENV['PATH']      = ENV['PATH'].gsub("/", "\\")
        ENV['JAVA_HOME'] = ENV['JAVA_HOME'].gsub("/", "\\")
      end
    end

    def configure_file_permissions(opts={})

      return if opts[:msfpro]

      # Set the file permissions when not running on Windows
      if (not configuration.platform.win32? and
          # TODO figure out why apps is 2 directories above Metasploit Pro's root (HDM: because Pro.root is set to: @root = Metasploit::Pro.root.join('engine') [apps/pro/engine/])
          Metasploit::Pro.root.join('..', '..', 'apps').directory?)
        $stdout.puts '[*] Configuring file permissions...'

        root_entities = ['license', 'license.log', 'certs', 'config']
        root_entities.each do |entity|
          entity_pathname = Metasploit::Pro::Engine.root.join(entity)

          if entity_pathname.exist?
            entity_pathname.chmod(0700)
          end
        end

        daemon_root_engine_entities = ['tmp']
        daemon_root_engine_entities.each do |entity|
          entity_pathname = Metasploit::Pro::Engine.root.join(entity)

          # Pathname#chown doesn't support name, only uid and gid, so use FileUtils#chown, which does support names
          FileUtils.chown 'daemon', 'root', entity_pathname.to_s

          entity_pathname.chmod(0750)
        end

        daemon_root_pro_entities = ['campaign_files', 'cred_files', 'loot', 'reports', 'export', 'tasks', ::File.join('ui', 'config', 'database.yml') ]
        daemon_root_pro_entities.each do |entity|
          entity_pathname = Metasploit::Pro.root.join(entity)

          # Ensure that the directory exists so we can set the permissions properly
          unless entity_pathname.exist?
            FileUtils.mkdir_p(entity_pathname)
          end

          entity_path = entity_pathname.to_path

          # Pathname#chown doesn't support name, only uid and gid, so use FileUtils#chown, which does support names
          FileUtils.chown_R 'daemon', 'root', entity_path
          FileUtils.chmod_R 0750, entity_path
        end

        script_glob = Metasploit::Pro::UI.root.join('script', '*')
        Dir.glob(script_glob) do |script_path|
          FileUtils.chmod(0755, script_path)
        end

        framework_data_pathname = Metasploit::Framework.root.join('data')
        framework_data_path = framework_data_pathname.to_s
        FileUtils.chmod_R 0755, framework_data_path

        framework_scripts_glob = Metasploit::Framework.root.join('msf*')
        Dir.glob(framework_scripts_glob) do |script_path|
          FileUtils.chmod(0755, script_path)
        end

        Metasploit::Pro.root.chmod(0755)
      end
    end

    def ensure_root_user!
      # Check process id, bail out if not root in production
      if configuration.environment.production?
        if Process.uid != 0
          $stderr.puts "Process not running as root!"

          exit(1)
        else
          puts "[-] Warning: This process runs as root (in production).  " \
                   "It listens at #{configuration.server.host}:#{configuration.server.port}/tcp.  " \
                   "Protect it accordingly!"
        end
      end
    end

    def expand_file_limit
      if configuration.platform.linux32? or configuration.platform.linux64?
        # current max
        number_of_files_limit = Process.getrlimit(Process::RLIMIT_NOFILE)[1]

        loop do
          number_of_files_limit *= 2
          number_of_files_soft_limit = number_of_files_limit
          number_of_files_hard_limit = number_of_files_limit

          begin
            Process.setrlimit(Process::RLIMIT_NOFILE, number_of_files_soft_limit, number_of_files_hard_limit)
          rescue Errno::EPERM
            # hit system maximum, NR_OPEN, so stop
            break
          end
        end
      end
    end

    # Initialize the default profile if necessary
    # TODO: This should be happening inside the Profile model.
    def find_or_create_default_profile!
      if Mdm::Profile.find_by_name("default").blank?
        dp = Mdm::Profile.new
        dp.active = true
        # TODO: This should be happening inside of an after_initialize callback.
        dp.settings = {"automatically_check_updates" => true}
        dp.name = "default"
        dp.owner = "<system>"
        dp.save!
      end
    end

    def framework
      unless instance_variable_defined? :@framework
        $stdout.puts '[*] Initializing the framework instance...'

        config_pathname  = Metasploit::Pro::Engine.root.join('config')
        config_path = config_pathname.to_s
        framework = Msf::Simple::Framework.create('ConfigDirectory' => config_path)

        # Pro will resolve to Metasploit::Pro in this lexical scope, so prefix with Pro with :: to get top-level Pro
        # namespace
        framework.extend(::Pro::License::Product)

        license_pathname = Metasploit::Pro::Engine.root.join('license')
        license_path = license_pathname.to_s
        framework.esnecil_init(license_path)
        @framework = framework
      end

      @framework
    end

    # Uses the initializers from pro/ui to add Pro functionality to Mdm models.
    #
    # @todo Load prosvc in Rails environment for Pro so this just happens automatically.
    def initialize_metasploit_data_models(opts)
      metasploit_data_models_initializer_pathname = Metasploit::Pro::UI.root.join(
          'config',
          'initializers',
          'metasploit_data_models.rb'
      )
      metasploit_data_models_initializer_path = metasploit_data_models_initializer_pathname.to_path
      # must pass a path, and not a pathname as Fastlib's require uses =~, which doesn't work on Pathnames.
      require metasploit_data_models_initializer_path
    end

    def merge_meterpreter_extensions
      $stdout.puts '[*] Merging Metasploit Pro data files...'

      # Copy any additional Meterpreter extension files
      extension_glob = Metasploit::Pro.root.join('data', 'meterpreter', 'ext_*')
      meterpreter_pathname = Metasploit::Pro.root.join('msf3', 'data', 'meterpreter')
      # Pathname doesn't support '/' suffix, so change to string path
      meterpreter_directory = "#{meterpreter_pathname}/"

      Dir.glob(extension_glob) do |extension_path|
        FileUtils.copy(extension_path, meterpreter_directory)
      end
    end

    def run_flash_policy_server
      if configuration.flash_policy_server.run
        flash_policy_server = Metasploit::Pro::Engine::FlashPolicyServer.new(configuration.flash_policy_server)

        flash_policy_server.start
      end
    end

    def start_listeners
      client = Pro::Client.new

      enabled_listeners = Mdm::Listener.where(:enabled => true)

      # Start any persistent listeners that are enabled
      enabled_listeners.each do |listener|
        task = client.start_listener(
            'workspace'      => listener.workspace.name,
            'username'       => listener.owner,
            'DS_LISTENER_ID' => listener[:id]
        )
        task_id = task['task_id']

        if task_id
          listener.task_id = task_id
        end

        if listener.changed?
          listener.save
        end
      end
    end

    def start_nada_proxy
  	  $stdout.puts '[*] Loading Nada Proxy..'
      framework.threads.spawn("nada_proxy", false, "auxiliary/pro/web/nada_proxy") do |module_name|
        Thread.current.priority = -5
      	begin
      		mod = framework.modules.create(module_name)
      		mod.run_simple
      	rescue ::Exception => e
      	  $stdout.puts "[-] could not load Nada Proxy: #{e} #{e.backtrace}"
      	end
      end
    end


    def start_rpc_engine
      $stdout.puts '[*] Starting the RPC engine...'

      timestamp = Time.now.utc.to_i
      server = ::Msf::RPC::Service.new(
          framework,
          {
              :host   => configuration.server.host,
              :port   => configuration.server.port,
              :uri    => configuration.uri,
              :ssl    => configuration.ssl?,
              :tokens => {
                  configuration.token => [
                      'prosvc',
                      timestamp,
                      timestamp,
                      true
                  ]
              },
              :tasks  => Pro::TaskContainer.new
          }
      )

      rpc_pro = Pro::RPC::RPC_Pro.new(server)
      server.add_handler('pro', rpc_pro)

      # Register our main worker thread
      framework.threads.register(Thread.current, 'ProSvc', true)

      # Start the actual service
      server.start

      # Give the background services time to initialize
      until (server.service and
          server.service.listener and
          server.service.listener.listener_thread and
          server.service.listener.listener_thread.status == 'sleep')
        select(nil, nil, nil, 0.25)
      end

      # Update the background threads and mark as critical
      framework.threads.update(server.service.listener.listener_thread, 'ProSrv-Listener', true)
      framework.threads.update(server.service.listener.clients_thread, 'ProSrv-ClientMonitor', true)

      # Bump their priorities to the highest level
      server.service.listener.listener_thread.priority = 30
      server.service.listener.clients_thread.priority = 30

      # @note this starts a background thread.  IT IS NOT DEAD CODE.
      Pro::BackgroundDaemon.new(framework)

      #---- BEGIN things that require ActiveRecord to be started----------
      social_engineering_email_pathname = Metasploit::Pro::UI.root.join(
          'app',
          'models',
          'social_engineering',
          'email.rb'
      )
      social_engineering_email_path = social_engineering_email_pathname.to_s
      require social_engineering_email_path
      #---- END things that require ActiveRecord to be started----------

      # Spawn a console or simply wait for the service to exit
      if configuration.console_mode
        # Flush arguments off ARGV so irb will load properly
        while ARGV.length > 0
          ARGV.shift
        end

        console_driver = Msf::Ui::Console::Driver.new(
            "%undmsfpro%clr",
            Msf::Ui::Console::Driver::DefaultPromptChar,
            {
                'Framework' => framework
            }
        )
        console_driver.run
      else
        $stdout.puts '[*] Ready'
        server.wait
      end
    end

    def synchronize_token
      unless configuration.token
        pathname = root.join('tmp', 'servicekey.txt')

        unless pathname.size?
          token = Rex::Text.rand_text_alphanumeric(32)
          configuration.token = token

          pathname.open('wb') do |f|
            f.write(token)
          end
        else
          configuration.token = pathname.read
        end

        # Directory permissions are used to control access
        pathname.chmod(0644)

        $stdout.puts "[*] Service token stored in #{pathname}..."
      end
    end

    # Clean up any old / defunct tasks
    def update_defunct_tasks!
      Mdm::Task.find(:all, :conditions => { :completed_at => nil }).each do |t|
        t.completed_at = ::Time.now.utc
        t.save! rescue nil
      end
    end

    # Save our PID to the pid file so other apps can verify we're running or not
    def write_pid_file
      pid_pathname = root.join('tmp', 'prosvc.pid')

      pid_pathname.open('wb') do |f|
        f.puts Process.pid
      end
    end
  end
end
