$:.push(File.join(File.dirname(__FILE__), '..', '..', '..', 'msf3', 'lib'))

require File.join(File.dirname(__FILE__), '..', '..', '..', 'ui', 'app', 'models', 'update_check')
require File.join(File.dirname(__FILE__), '..', '..', '..', 'ui', 'lib', 'pro', 'client_connection_manager')

require 'msf/core'
require 'rex'

require 'rapid7/nexpose'

require 'timeout'
require 'ice_cube'

module Pro
class BackgroundDaemon

	attr_accessor :framework, :monitor, :state, :profile, :update_last_checked

	def initialize(framework)
		self.state = {}
		self.framework = framework

		#
		# Spawn thread to kickoff sanity/maintenance loop
		#
		self.monitor = framework.threads.spawn("BackgroundDaemon", false) do
			#
			# This is a sanity loop that will launch and re-launch
			# the maintenance loop if it dies for any reason
			#
			loop do
				maintenance_tasks rescue nil
				::IO.select(nil, nil, nil, 30.0)
			end
		end
	end # initialize

	#
	# Execute sub-tasks defined below
	#
	def maintenance_tasks
		loop do
			#
			# Execute nexpose console state update logic
			#
			begin
				update_nexpose_console_states
 			rescue ::Exception => e
			end

			#
			# Execute product update alerting logic
			#
			begin
				check_for_updates
			rescue ::Exception => e
				elog("[BG Daemon] Product Updates: #{e.class} #{e} #{e.backtrace}")
			end

			#
			# Execute scheduled task chain logic
			#
			begin
				exec_scheduled_chains
			rescue ::Exception => e
				elog("[BG Daemon] Scheduled Chains: #{e.class} #{e} #{e.backtrace}")
			end

			::IO.select(nil, nil, nil, 10.0)
		end
	end # maintenance_tasks

	def update_nexpose_console_states
		::ActiveRecord::Base.connection_pool.with_connection {
			Mdm::NexposeConsole.find(:all).each do |con|
				self.state[con.id] ||= 0
				next if not con.enabled
	
				# Look for recently modified consoles
				if con.updated_at.utc.to_i > self.state[con.id] or self.state[con.id] + (60 * 10) < ::Time.now.utc.to_i
					update_nexpose_console_state(con)
				end
			end
		}
	end

	def check_for_updates
		::ActiveRecord::Base.connection_pool.with_connection {
			profile = Mdm::Profile.find_by_active(true)
			return if not profile
		
			#
			# Run through the software update check if:
			#	1. Global options are set to automatically check for updates
			#	and
			#	2. The check has not been run before
			#	or
			#	3. The check has not happened in 6 hours
			#
			if profile.settings["automatically_check_updates"] &&
				(@update_last_checked.nil? ||
				 @update_last_checked < 6.hours.ago.utc)

				#
				# Redmine: #6898
				# Update last checked time here to ensure any errors below will
				# not cause this logic to be executed every 10 seconds.
				#
				@update_last_checked = Time.now.utc

				#
				# Use an HTTP proxy for the update request if one is configured
				#
				if profile.settings["use_http_proxy"]
					proxy_settings =  {
						:proxy_host => profile.settings["http_proxy_host"],
						:proxy_port => profile.settings["http_proxy_port"],
						:proxy_user => profile.settings["http_proxy_user"],
						:proxy_pass => profile.settings["http_proxy_pass"],
					}
				end

				#
				# Make actual call to check for available update package
				#
				update = ::UpdateCheck.new(proxy_settings || {})

				#
				# Update profile record based on status of update check
				#
				if update.error.nil?
					profile.settings["update_proxy_error"] = nil
					profile.settings["update_available"]   = !update.up_to_date?
				else
					profile.settings["update_proxy_error"] = true
					profile.settings["update_available"]   = nil
				end
			
				#
				# If the update check takes five minutes to time out, then we don't want
				# to save the profile object we loaded five minutes ago. Reload it, overwrite
				# the settings we changed, and save it again to avoid ActiveRecord::StaleObjectError.
				#
				fresh_profile = Mdm::Profile.find(profile[:id])
				["update_proxy_error","update_available"].each do |field|
					fresh_profile.settings[field] = profile.settings[field]
				end
				fresh_profile.save
			end
		}
	end

	#
	# Handle execution of scheduled task chains
	#
	def exec_scheduled_chains
		::ActiveRecord::Base.connection_pool.with_connection {
			#
			# Query scheduled task chains and extract records
			# whose next_run_at value is <= the current time
			#
			task_chains = TaskChain.where(
				"next_run_at <= ? AND state = 'ready'", Time.now.utc
			)

			#
			# Iterate over any chains returned above and execute task list
			#
			chain_cnt = 1
			task_chains.each do |chain|
				#
				# De-serialize the ice cube object, update the chain runtimes
				# and start the chain to set the state to 'running'
				#
				chain.last_run_at = Time.now.utc
	
				# Run it just this once, or set its next run time
				if chain.schedule.present?
					chain.next_run_at = chain.schedule.next_occurrence
				else
					chain.next_run_at = nil
				end
	
				chain.start
				
				# Skip over any chain that fails validation
				next unless chain.save

				#
				# Spawn a thread on a per-chain basis so we do not block
				# the overall event loop querying for scheduled chains
				#
				framework.threads.spawn("TaskChain#{chain_cnt}", false, chain) do |xchain|
					begin
						#
						# Execute all tasks associated with each chain
						#
						exec_scheduled_tasks(xchain)
					rescue ::Exception
						elog("Error processing Chain #{xchain.id}: #{$!.class} #{$!} #{$!.backtrace}")
						#
						# Re-set the chain's state before exiting
						#
						xchain.finish! rescue nil
					end
				end # thread spawn

				chain_cnt += 1
			end # task chain iterator
		}
	end # exec_scheduled_chains

	def exec_scheduled_tasks(chain)
		::ActiveRecord::Base.connection_pool.with_connection {
			#
			# Grab associated tasks for this chain
			#
			task_list = ScheduledTask.where(
				"task_chain_id = ?", chain.id
			).order("position ASC")
	
			#
			# Stop all sessions and delete all hosts associated with the project
			# for which this chain was scheduled.
			#
			if chain.clear_workspace_before_run
				chain.workspace.sessions.each { |s| s.stop rescue nil }
				chain.workspace.hosts.destroy_all
			end
		
			#
			# Instantiate RPC client object
			#
			rpc = ::Pro::Client.get
		
			#
			# Iterate over task list and execute
			#
			task_list.each do |task|
				#
				# Update last_run_at value to be the start time of task
				#
				task.last_run_at = Time.now.utc
				task.save!
				
				#
				# Pass task's config_hash off to rpc method based on task type
				#
				case task.kind
				when /^scan$/i # avoid clash w/ webscan
					tid = rpc.call("pro.start_discover", task.config_hash)['task_id']
				when /exploit/i
					tid = rpc.call("pro.start_exploit", task.config_hash)['task_id']
				when /bruteforce/i
					tid = rpc.call("pro.start_bruteforce", task.config_hash)['task_id']
				when /report/i
					tid = rpc.call("pro.start_report", task.config_hash)['task_id']
				when /import/i				
					this_config = task.config_hash.dup
					this_config['DS_REMOVE_FILE'] = false
					tid = rpc.call("pro.start_import", this_config)['task_id']
				when /webscan/i
					tid = rpc.call("pro.start_webscan", task.config_hash)['task_id']
				when /nexpose/i
					tid = rpc.call("pro.start_nexpose", task.config_hash)['task_id']
				when /collect/i
					tid = rpc.call("pro.start_collect", task.config_hash)['task_id']
				when /cleanup/i
					tid = rpc.call("pro.start_cleanup", task.config_hash)['task_id']
				when /module_run/i
					tid = rpc.call("pro.start_single", task.config_hash)['task_id']
				end
				
				#
				# Block on task execution waiting until complete (or error)
				#
				task_status = true
				while true
					#
					# If RPC call did not return a task ID,
					# notify of error condition and break
					#
					unless tid
						task.state  = 'Error: RPC call did not return task id'
						task_status = false
						break
					end

					res = rpc.call("pro.task_status", tid)
					
					#
					# If error condition detected, set status to false and break
					#
					if res['error'] or res[tid]['error'].length > 0
						task.state  = "Error: #{res['error'] || res[tid]['error']}"
						task_status = false
						break
					end
			
					#
					# Update the current state of running task
					#
					unless task.state == res[tid]['info']
						task.state = res[tid]['info']
						task.save!
					end
			
					#
					# Break from while loop when task is complete
					#
					break if res[tid]['status'] != 'running'
					::IO.select(nil, nil, nil, 10.0)
				end
			
				#
				# Update task status and save
				#
				task.last_run_status = task_status ? 'success' : 'failure'
				unless task.save
					elog("Error processing Task #{task.id}: failed to save updated status")
					task_status = false
				end

				#
				# Stop task chain execution if any task fails
				#
				break unless task_status
			end # task_list iterator

 				#
 				# Update the chain's state to 'ready' and save
				#
 				chain.finish!

			#
			# Close out db handles before returning
			#
			chain.connection.close
			task_list.connection.close
		}
	end # exec_scheduled_tasks

	def update_nexpose_console_state(con)
		::ActiveRecord::Base.connection_pool.with_connection {
			status = "Unknown"
			sites  = []
			vers   = ''
			nsc    = nil

			begin
				::Timeout.timeout(10) do
					nsc = ::Nexpose::Connection.new( con.address, con.username, con.password, con.port )
					nsc.login
					status = "Available"
				end

				::Timeout.timeout(10) do
					if nsc
						res = nsc.console_command("ver") rescue nil
						if res and res =~ /^(NSC|Console) Version ID:\s*(\d+)\s*$/m
							vers = $2
						else
							vers = "Unknown"
						end
					end
				end

				::Timeout.timeout(10) do
					sites = ( nsc.site_listing || []).map{|x| x[:name] } if nsc
				end
	
			rescue ::Timeout::Error => e
				status = "Timeout"
			rescue ::Nexpose::AuthenticationFailed => e
				status = "No Access"
			rescue ::Nexpose::APIError => e
				status = "Error: #{e.req.error}"
			rescue ::SocketError
				status = "Unreachable"
			rescue ::Exception => e
				status = "Error(#{e.class})"
			ensure
				if nsc
					nsc.logout rescue nil
				end
			end

			update_info = { :status => status, :updated_at => ::Time.now.utc }
			update_info[:cached_sites] = sites if (sites.length > 0)
			update_info[:version]      = vers  if (vers and not vers.empty?)
	
			Mdm::NexposeConsole.update(con.id, update_info)
			self.state[con.id] = ::Time.now.utc.to_i
		}
	end


end
end

