=begin
--Notes--

The bruteforce options that govern cred generation in this Factory are:
		ds["MaxGuessesPerUser"]
		ds["MaxGuessesPerUser"]
		ds["Preserve_Domains"]
		ds["Bruteforce_Skip_Blank_Passwords"]
		ds["Bruteforce_Skip_Machine_Names"]
		ds["Bruteforce_Skip_Builtin_Windows_Accounts"]
		ds["Bruteforce_Skip_Builtin_Unix_Accounts"]
		ds["Bruteforce_QuickMode_Creds"]
		ds["CRED_FILE_IDS"]
=end

require 'msf/base/config'
require 'rex/text'
require 'pro/mixins/locations'

module Msf
module Pro
module Credentials

	def self.camelcase_symbol(sym)
		sym.to_s.downcase.split("_").map {|w| w.capitalize}.join
	end

class Factory
	@@valid_classes = []
	attr_accessor :workspace, :ds, :builders, :sections, :type, :flavor
	attr_accessor	:max_creds_reached, :max_creds_per_user_reached, :max_creds
	attr_accessor :seed_users, :seed_passes
	attr_accessor :all_creds_count
	attr_accessor :rhosts, :rport
	# Only really used for SSH-flavored factories. Refactor this some day.
	attr_accessor :ssh_keys, :ssh_key_md5s, :ssh_pubkeys

	DEFAULT_MAX_CREDS = 1000

	def initialize(workspace,ds)
		@all_creds_count = 0
		self.builders = []
		self.workspace = workspace
		self.ds = validate_datastore(ds)
		self.sections = {}
		self.max_creds = DEFAULT_MAX_CREDS
		self.max_creds_reached = false
		self.max_creds_per_user_reached = {}
		self.seed_users = []
		self.seed_passes = []
		self.rhosts = []
		self.rport = 1
		self.ssh_keys = []
		self.ssh_key_md5s = []
		self.ssh_pubkeys = []
	end

	def ==(other)
		return false unless other.respond_to? :factory_id
		return true if other.factory_id == self.factory_id
		return false unless other.all_creds_count == self.all_creds_count
		Rex::Text.md5(other.all_creds.to_s) == Rex::Text.md5(self.all_creds.to_s)
	end

	# This is a shorthand method to see if this object is,
	# in fact, a factory, and if so, returns its object ID.
	# It's a little bit of a contrivance.
	def factory_id
		self.object_id
	end

	# Required to pass the OptPath validation and to communicate that
	# this is not a regular path, but a Ruby object to the aux module that
	# ultimiately consumes this object.
	def acts_as_file?
		true
	end

	def inspect
		"<#%s:%d @type=%s @flavor=%s | RHOSTS: %s | Creds:%d/%d | Seeds: %d/%d | Sections:[%s] >" % [
			self.class.name,
			self.object_id,
			self.type.inspect,
			self.flavor.inspect,
			self.rhosts.inspect,
			self.all_creds.size,
			self.max_creds,
			self.seed_users.size,
			self.seed_passes.size,
			self.builders.map {|b| [b.this_section,b.creds.size].join(":")}.join(", ")
		]
	end

	def register(obj)
		self.builders = self.builders | [obj]
	end

	# Returns all the creds so far as one array. Note that because
	# of ordered hashes, these will be in the correct credlist order.
	def all_creds
		self.sections.values.flatten
	end

	# Since this doesn't order creds by section, it shouldn't be used for
	# writing out final credentials lists. This is used to check for dupes
	# already written to prior sections -- at least, it would if
	# checking against the dehexed list wouldn't take forever. Currently.
	# unused.
	def all_creds_dehexed
		self.all_creds.map {|c| Rex::Text.dehex(c)}
	end

	# Returns all the users so far (dehexed).
	def all_users
		users_mapped = self.all_creds.map {|cred| u,p = cred.split(/\s+/,2); u}
		users_mapped.compact.uniq
	end

	# Returns all ssh keys
	def all_ssh_keys
		self.ssh_keys.flatten.compact.uniq
	end

	# Returns all ssh pubkeys
	def all_ssh_pubkeys
		self.ssh_pubkeys.flatten.compact.uniq
	end

	# This is used mainly to calculate the number of creds per user in the
	# event we have datastore["MaxGuessesPerUser"] set. Note that it's okay
	# if a password is nil here, since a blank pw counts as a pw.
	#
	# I suspect this is quite slow for large lists. Live with it for now.
	def all_creds_by_user
		user_pass_hash = {}
		self.all_creds.each do |cred|
			u,p = cred.split(/\s+/,2).map {|x| Rex::Text.dehex(x)}
			user_pass_hash[u] ||= []
			user_pass_hash[u] << p
		end
		user_pass_hash
	end

	# Adds a credential set to a :section. 
	def add_section(section,creds)
		key = section.to_s.downcase.to_sym
		self.sections[key] = []
		if creds.kind_of? Array
			self.sections[key] = creds
		end
		self.sections[key]
	end

	def remove_section(section)
		key = section.to_s.downcase.to_sym
		if self.sections[key] && !self.sections[key].empty?
			self.max_creds_reached = false
		end
		self.sections.delete key
		self.sections
	end

	def required_datastore_elements
		%w{
			Bruteforce_Scope
			Bruteforce_Services
			Bruteforce_QuickMode_Creds
		}
	end

	def validate_datastore(ds)
		unless ds.kind_of? Msf::DataStore
			raise ArgumentError, "argument to validate_datastore is not a kind of Msf::DataStore"
		end
		required_datastore_elements.each do |key|
			unless ds[key]
				raise ArgumentError, "Missing DataStore option: `#{key}'"
			end
		end
		ds["MaxGuessesOverall"]    ||= 0
		ds["MaxGuessesPerService"] ||= 0
		ds["MaxGuessesPerUser"]    ||= 0
		ds
	end

	# class.parent does the pretty much the same thing but it's an 
	# ActiveRecord thing that I don't quite trust or understand yet, 
	# and I need the string version anyway.
	def namespace
		ns_array = self.class.to_s.split("::")
		ns_array[0,ns_array.size-1].join("::")
	end

	# From http://www.rubyquiz.com/quiz113.html , seems there's no
	# intuitive method for this kind of thing.
	def resolve_namespace(ns)
		ns.split(/::/).inject(Object) {|s,c| s.const_get(c.to_sym)}
	end

	def camelcase_symbol(sym)
		Msf::Pro::Credentials.camelcase_symbol(sym)
	end

	# When the first builder is created, it casts this factory as a factory for
	# just that kind of builder, so subsequent builders must all be of the same
	# time. This is because Factory instances keep track of max creds and creds
	# created so far.
	#
	# If the builder for the named section is already made, just return it. 
	def create_builder(args)
		proto = args[:proto].to_s.to_sym
		flavor = args[:flavor].to_s.to_sym if args[:flavor]
		section = args[:section].to_s.to_sym
		subclass = "%s::%s%sCreds" % [namespace,proto.to_s.downcase.capitalize, camelcase_symbol(section)]
		subclass_resolved = resolve_namespace(subclass)
		unless valid_classes.include?(subclass_resolved)
			raise ArgumentError, "Cannot locate builder class `#{subclass}'"
		end
		if (idx = self.builders.map {|b| b.this_section}.index section.downcase.to_sym) # Assignment!
			cred_builder = self.builders[idx]
		else
			cred_builder = subclass_resolved.new(self.ds)
			self.type ||= cred_builder.this_service
			self.max_creds = cred_builder.max_creds
			if self.type == cred_builder.this_service
				cred_builder.workspace = self.workspace
				cred_builder.factory = self
				self.flavor = flavor
				cred_builder.flavor = self.flavor
				cred_builder.register
				return cred_builder
			else
				raise ArgumentError, "Cannot create a builder class not of type `#{self.type.inspect}'"
			end
		end
	end

	def has_section?(sym)
		raise ArgumentError unless sym.respond_to? :to_sym
		self.sections.keys.include? sym.to_sym
	end

	def builder(sym)
		return nil unless self.has_section? sym
		self.builders.select {|b| b.this_section == sym.to_sym}.first
	end

	def section(sym)
		return nil unless self.builder(sym)
		self.builder(sym).creds
	end

	alias :seed_passwords :seed_passes
	alias :seed_usernames :seed_users

	# Class methods -- manages @@valid_classes
	class << self

		# Enforces the naming convention for subclasses <Proto><SectionName>Creds,
		# and skips those subclasses that lacks a section name. Eg, SshCreds will
		# not pass, but SshKnownCreds and SshSeededUserCreds will.
		def validate_classname(subclass)
			words = subclass.to_s.split("::").last.gsub(/[A-Z]/) {|x| "-#{x}"}.split("-")
			(words.size >= 4 && words.last == "Creds")
		end

		def add_cred_class(subclass)
			if validate_classname(subclass)
				@@valid_classes = @@valid_classes | [subclass]
			end
		end

		def remove_cred_class(subclass)
			validate_classname(subclass)
			@@valid_classes.delete subclass
			@@valid_classes
		end

		def valid_classes
			@@valid_classes
		end
	end

	def valid_classes
		@@valid_classes
	end

end

class Builder

	attr_accessor :max_creds, :flavor, :factory, :workspace, :creds
	attr_accessor :imported_ssh_keys, :imported_ssh_pubkeys
	DEEP_FACTOR = 3
	ONE_OFF_CREDENTIALS = []

	def initialize(ds)
		set_default_max_creds(ds)
		@creds = []
	end

	def self.inherited(subclass)
		Factory.add_cred_class(subclass)
	end

	def register
		self.factory.register(self)
	end

	# Returns the service name of this Builder as a downcased symbol.
	# Requires the naming be consistent, of course -- start with an
	# upper (impossible not to), and the protocol name is all downcased
	# alphanum.
	def this_service
		self.class.name.split("::").last[/([A-Z][a-z0-9]+)/].downcase.to_sym
	end

	alias :service :this_service

	def this_section
		words = self.class.name.split("::").last.gsub(/[A-Z]/) {|x| "-#{x}"}.split("-")
		words[2,words.size-3].join("_").downcase.to_sym
	end

	alias :section :this_section

	def inspect
		"<#%s:%d @flavor=%s @workspace=<#%s:%d> @factory=<#%s:%d> | Creds: %d/%d >" % [
			self.class.name,
			self.object_id,
			self.flavor.inspect,
			self.workspace.name,
			self.workspace.id,
			self.factory.class.name,
			self.factory.object_id,
			self.creds.size,
			self.max_creds
		]
	end

	# Sets the initial max credits for this Builder, which incidentally
	# gets picked up by the factory. This gets calculated with every new
	# builder which is a little bit of a waste, so fix that sometime.
	def set_default_max_creds(ds)
		if self.class.const_get(:SERVICE_LIMIT)
			default_limit = self.class.const_get(:SERVICE_LIMIT) 
		else
			default_limit = 1000
		end
		if ds["Bruteforce_Scope"] == "50k"
			self.max_creds = 50_000
		elsif ds["MaxGuessesPerService"] > 0
			self.max_creds = ds["MaxGuessesPerService"]
		elsif ds["Bruteforce_Scope"] == "deep"
			if self.class.const_get(:DEEP_FACTOR)
				self.max_creds = default_limit * self.class.const_get(:DEEP_FACTOR)
			else
				self.max_creds = default_limit
			end
		else
			self.max_creds = default_limit
		end
		self.max_creds
	end

	# Adds creds to self.cred after making the checks for uniqueness and the
	# various limits. Also discards nil or empty creds. If seed is set to
	# true, this username and password will also be added to the factory.seed_user
	# and factory.seed_pass list.
	#
	# Performance metrics, retaining this for future work:
	# No checks for uniqeness: 116 seconds for 50_000 creds
	# Checks for uniqueness: 303 seconds for 50_000 creds
	# Precalculate all_creds.size, uniq check, and reordering failthrough: 191 secs
	# No checks for uniq with precalc all_creds.size: 1.7 seconds
	# Checking uniq only on section complete and limit reached: 79 seconds
	#
	# It's probably worth an option to skip the uniq check since that takes
	# a ton of time and may be worthless if it's a large import and 
	# they're already unique
	def add_cred(elem="")
		return self.creds unless elem
		return self.creds if self.factory.max_creds_reached
		stripped_elem = elem.strip.lstrip
		return self.creds if max_guesses_per_user_reached?(stripped_elem)
		return self.creds if stripped_elem.empty?
		formatted_cred = format_cred(stripped_elem)
		return self.creds unless formatted_cred
		return self.creds if formatted_cred.empty?
		self.creds.push formatted_cred 
		# Spot check here # if self.creds.size % 1000 == 0
		check_limit()
		self.creds
	end

	# Check for the uniqueness of all creds in self.creds. This only gets called when we hit
	# our self.max_creds to avoid checking every time. 
	def check_limit
		if self.creds.size + self.factory.all_creds_count >= self.max_creds
			check_uniqueness(:limit)
			if self.creds.size + self.factory.all_creds_count >= self.max_creds
				self.factory.max_creds_reached = true
				self.factory.all_creds_count += self.creds.size 
			end
		end
		return self.factory.max_creds_reached
	end

	def add_seed(elem)
		return nil unless elem
		return nil if self.factory.max_creds_reached
		stripped_elem = elem.strip.lstrip
		user,pass = stripped_elem.split(/\s+/,2)
		if user && !user.empty? && !self.factory.seed_users.include?(user)
			self.factory.seed_users.push user
		end
		if pass && !pass.empty? && !self.factory.seed_passes.include?(pass)
			self.factory.seed_passes.push pass
		end
		[self.factory.seed_users, self.factory.seed_passes]
	end

	def max_guesses_per_user_reached?(elem)
		ret = false
		unless self.factory.ds["MaxGuessesPerUser"].zero?
			limit = self.factory.ds["MaxGuessesPerUser"]
			user,pass = elem.split(/\s+/,2).map { |x| Rex::Text.dehex(x) }
			ret ||= self.factory.max_creds_per_user_reached[user]
			if not ret
				factory_passwords = self.factory.all_creds_by_user[user]
				if factory_passwords
					ret ||= factory_passwords.size >= limit
				end
				unless ret
					self_passwords = creds_by_user[user]
					if self_passwords
						factory_passwords ||= []
						ret ||= (self_passwords + factory_passwords).size >= limit
					end
				end
			end
			self.factory.max_creds_per_user_reached[user] = ret
		end
		return ret
	end

	def creds_by_user
		user_pass_hash = {}
		self.creds.each do |cred|
			u,p = cred.split(/\s+/,2).map {|x| Rex::Text.dehex(x)}
			user_pass_hash[u] ||= []
			user_pass_hash[u] << p
		end
		user_pass_hash
	end

	def seed_with_these_creds?
		false
	end

	# Controlled by seed_with_these_creds? (default false),
	# once self.creds is built up with credentials, strip out
	# the username and passwords and add them to the seed tables.
	def seed_creds
		if seed_with_these_creds? 
			if self.class.const_get(:ONE_OFF_CREDENTIALS)
				skippable = self.class.const_get(:ONE_OFF_CREDENTIALS)
			else
				skippable = []
			end
			self.creds.each {|cred| add_seed(cred) unless skippable.include? cred}
		end
	end

	# This can produce nils -- the caller should discard them or compact
	# the result. Can take a Cred object or a String.
	def format_cred(cred)
		return nil unless cred
		if cred.respond_to? :split
			if just_passwords?
				username = "root"
				password = cred.split(/\s/,2).last
			else
				username,password = cred.split(/\s/,2)
				if username =~ /[^\x2f]+\x2f(.+)/
					short_username = $1
					if this_service == :smb
						unless self.factory.ds["Preserve_Domains"]
							username = short_username
						end
					else
						username = short_username
					end
				end
			end
		elsif cred.respond_to? :user
			return nil if should_skip? cred
			if just_passwords?
				username = "root"
			else
				username = should_strip_domain?(cred) ? short_username(cred) : cred.user
			end
			password = cred.pass
		else
			raise ArgumentError, "Unsupported cred type: `#{cred.class.name}'"
		end
		unless just_passwords?
			return nil if(username.to_s.empty? and not(blank_usernames_allowed?))
			username = username.to_s.downcase if should_downcase_user?
		end
		return nil if(password.to_s.empty? and not(blank_passwords_allowed?))
		return nil if(password.to_s.empty? and self.factory.ds["Bruteforce_Skip_Blank_Passwords"])
		password = should_downcase_pass? ? password.to_s.downcase : password.to_s
		if just_usernames?
			username
		else
			[username,password].join(" ")
		end
	end

	# Take a Cred object or a String. Note that this
	# can return a nil if it's not first vetted via
	# should_strip_domain()
	def short_username(cred)
		if cred.respond_to? :user
			u = cred.user.split("/",2)[1]
		else
			u = cred.split("/",2)[1]
		end
	end

	def should_skip?(cred)
		# Always skip ftp anonymous
		if cred.user.to_s.downcase == "anonymous" and cred.service.name =~ /^ftp/
			return true
		end
		# Always skip ssh pubkeys
		if cred.ptype == "ssh_pubkey"
			return true
		end
		# Skip ssh_keys unless we're ssh -- those will be handled specially
		# by SshKnownUserpassCreds#find_workspace_creds().
		if cred.ptype == "ssh_key"
			return !(this_service == :ssh)
		end
		# Skip smb_hashes unless we are specifically Microsoft SMB.
		if cred.ptype == "smb_hash"
			return !microsoft_smb?
		end
		# If you've gotten here, assume it's okay.
		return false
	end

	def hexwise_reverse(str)
		if str.index("\\x")
			regex = /\x5cx[0-9a-f]{2}/mi
			reverse_array = []
			pos = 0
			while pos < str.size
				if str[pos,4] =~ regex
					reverse_array.unshift str[pos,4]
					pos += 4
				else
					reverse_array.unshift str[pos,1]
					pos += 1
				end
			end
			reverse_array.join
		else
			str.reverse
		end
	end

	# Includes blanks for users unless that's not allowed.
	def create_username_based_credentials(users=nil)
		users ||= self.creds.map {|x| x.split(/[\s+]/)[0]}.uniq
		users.each do |u|
			if blank_passwords_allowed? && !self.factory.ds["Bruteforce_Skip_Blank_Passwords"]
				add_cred "#{u}" 
			end
			add_cred "#{u} #{u}" 
			add_cred "#{u} #{u * 2}" if u.size < 9 # Just 16 chars or less
			add_cred "#{u} #{u}123"
			add_cred "#{u} #{hexwise_reverse(u)}"
			add_cred "#{u} #{u}!"
		end
		self.creds
	end

	# Note allowed: [:smb, :mysql, :mssql, :db2, :postgres, :oracle, :login, :shell, :exec]
	def blank_usernames_allowed?
		true
	end

	# TODO: Add notes
	def blank_passwords_allowed?
		true
	end

	# Used by :shell
	def just_usernames?
		false
	end

	# TODO: Add notes
	def just_passwords?
		false
	end

	# Seems like this is going to usually be true
	def should_downcase_user?
		true
	end

	# Seems like this is going to usually be false
	def should_downcase_pass?
		false
	end

	# This only matters for smb_hashes.
	def microsoft_smb?
		this_service == :smb and self.flavor.to_s[/microsoft/i]
	end

	def create_creds
		self.creds if self.factory.max_creds_reached
		if self.creds && !self.creds.empty?
			self.factory.remove_section(this_section)
			self.creds = []
		end
		self.factory.all_creds_count = self.factory.all_creds.size
		section_cred_creator
		check_uniqueness(:section) unless self.factory.max_creds_reached
		seed_creds() # Toggled by seed_with_these_creds? , default false
		self.factory.add_section(this_section,self.creds)
		self.factory.all_creds_count = self.factory.all_creds.size
	end

	def check_uniqueness(reason=nil)
		self.creds = self.creds.uniq
		repeat_creds = []
		self.factory.all_creds.each {|cred| repeat_creds << cred if self.creds.include? cred}
		self.creds = self.creds.reject {|x| repeat_creds.include? x}
	end

	def check_uniqueness(reason=nil)
		self.creds = self.creds.uniq
		repeat_creds = []
		self.factory.all_creds.each {|cred| repeat_creds << cred if self.creds.include? cred}
		self.creds = self.creds.reject {|x| repeat_creds.include? x}
	end

	def section_cred_creator
		raise RuntimeError, "section_cred_creator undefined for subclass `#{self.class}'"
	end

	# Used by both QuickUtils and ImportUtils. Returns an array of credentials to
	# be iterated over by add_creds.
	def parse_userpass_pair(pair)
		these_creds = []
		# Test for a pwdump-style of SMB hashes.
		if pair.strip =~ /^([^\s:]+):[0-9]+:([A-Fa-f0-9]+:[A-Fa-f0-9]+):[^\s]*$/
			if this_service == :smb
				user,hash = $1,$2
				these_creds << "#{user} #{hash}"
			end
		# Test for a pwdump-style of SMB hashes when you don't have a password.
		elsif pair.strip =~ /^([^\s:]+):([0-9]+):NO PASSWORD\*+:NO PASSWORD\*+[^\s]*$/
			if this_service == :smb
				user,sid = $1,$2.to_i
				if sid != 501 # Skip the Guest account (by SID)
					these_creds << user.to_s
				end
			end
		# Test for an authorized_keys file line
		# Regular user, userpass, or multiple passwords
		else
			u,passes = pair.split(/\s+/,2).map {|x| x.strip}
			if(u.kind_of?(String) and !u.empty?)
				if(passes and !passes.empty?)
					passes.split(/\s+/).each {|p| these_creds << "#{u} #{p}" }
				else
					these_creds << "#{u}"
				end
			end
		end
		return these_creds
	end

	# The preference determines the order by which we add_creds.
	# If prefer is :user_pass, each user will get all passwords.
	# If perfer is :pass_user, each password will be assigned to all users.
	# This matters in case you run out of room in the end, and it sets up
	# the order. :pass_user style feels more democratic, as it fills users
	# evenly.
	def create_creds_from_users_and_passes(users=[],passes=[],prefer=:pass_user)
		if prefer == :pass_user
			passes.each do |p| 
				users.each do |u| 
					this_cred = [u.to_s,p].join(" ").strip
					add_cred this_cred
				end
			end
		else
			users.each do |u| 
				passes.each do |p| 
					this_cred = [u.to_s,p].join(" ").strip
					add_cred this_cred
				end
			end
		end
		self.creds
	end

	def create_creds_from_protocol_wordlists(proto, is_private = false)
		if is_private
			msf_dir = ::File.expand_path(::File.join(Msf::Config.install_root, ".."))
		else
			msf_dir = ::File.expand_path(Msf::Config.install_root)
		end
		wordlist_path = ::File.join(msf_dir,"data","wordlists")
		upfile = ::File.join(wordlist_path,"#{proto}_default_userpass.txt")
		if ::File.readable?(upfile)
			these_pairs = ::File.open(upfile) {|f| f.read(f.stat.size)}.split(/\n/)
			these_pairs.each do |pair|
				these_creds = parse_userpass_pair(pair)
				these_creds.each {|c| add_cred c}
			end
		end
		ufile = ::File.join(wordlist_path,"#{proto}_default_users.txt")
		pfile = ::File.join(wordlist_path,"#{proto}_default_pass.txt")
		users = []
		passes = []
		if ::File.readable?(ufile)
			users = ::File.open(ufile) {|f| f.read(f.stat.size)}.split(/\n/)
		end
		if ::File.readable?(pfile)
			passes = ::File.open(pfile) {|f| f.read(f.stat.size)}.split(/\n/)
		end
		create_creds_from_users_and_passes(users,passes)
		return self.creds
	end

	# Used by QuickUtils and ImportUtils. Returns an explicit 
	# symbol to determine if this line is the
	# start or end of an ssh key, or false. Note that it's not
	# useful to eval the return as a mere boolean!
	def detect_incoming_ssh_key_cred(line)
		return :invalid unless line.kind_of? String
		return :empty if line.empty?
		case line
		when /--BEGIN [DR]SA PRIVATE KEY--/
			:start_ssh_key
		when /--END [DR]SA PRIVATE KEY--/
			:end_ssh_key
		when /^\s*ssh-(rsa|dss)\s+[A-Za-z0-9\x2b\x2f\x3d]{64,}/
			:is_ssh_pubkey
		else
			false
		end
	end

	# Used by QuickUtils and Import Utils. 
	# If we're in an SSH key, add the line to the last in the 
	# imported_ssh_keys array. If it's an SSH pubkey, add it
	# to the imported_ssh_pubkeys array, and add any comments
	# to the users list.
	def import_ssh_key_line(line)
		key_state = detect_incoming_ssh_key_cred(line)
		case key_state
		when :start_ssh_key
			self.imported_ssh_keys << line.strip.lstrip + "\n"
		when :end_ssh_key, FalseClass
			self.imported_ssh_keys.last << line.strip.lstrip << "\n"
		when :is_ssh_pubkey
			key_line = line.strip.lstrip
			key_type, key_data, key_comments = key_line.split(/\s+/,3)
			self.imported_ssh_pubkeys << [key_type, key_data].join(" ")
			if key_comments
				key_comments.split(/\s+/).each do |comment|
					user,host = comment.split(/@/)
					add_cred(user)
				end
			end
		end
	end

	# Used by QuickUtils and ImportUtils.
	# This does some pretty basic and silent validation -- if you're malformed,
	# you get dropped. Otherwise, the ssh_pubkey module will be happy to let
	# you know that you're bad news.
	def validate_imported_ssh_keys
		return unless self.imported_ssh_keys.respond_to? :uniq!
		self.imported_ssh_keys.uniq!
		bad_indeces = []
		self.imported_ssh_keys.each_with_index do |key,idx|
			is_bad = false
			# Require one and only one SSH Key start
			is_bad ||= key.scan(/--BEGIN [DR]SA PRIVATE KEY--/).size != 1
			# Require one and only one SSH Key end
			is_bad ||= key.scan(/--END [DR]SA PRIVATE KEY--/).size != 1
			# No horizontal whitespace in any line
			key.each_line do |line|
				next if line =~ /PRIVATE KEY--/
				is_bad ||= line[/[\x20\x09]/]
			end
			bad_indeces << idx if is_bad
		end
		bad_indeces.each { |i| self.imported_ssh_keys[i] = nil }
		self.imported_ssh_keys.compact!
		add_uniq_ssh_keys()
	end

	# Add any imported keys to the factory, depending on uniqueness,
	# so we can get a hold of them later.
	def add_uniq_ssh_keys
		self.factory.ssh_key_md5s ||= [] 
		self.factory.ssh_keys.flatten.compact.each { |key|
			self.factory.ssh_key_md5s << ssh_key_md5(key)
		}
		self.imported_ssh_keys.flatten.each { |key|
			next if self.factory.ssh_key_md5s.include? ssh_key_md5(key)
			self.factory.ssh_keys << key
			self.factory.ssh_key_md5s << ssh_key_md5(key)
		}
		self.factory.ssh_pubkeys ||= []
		self.factory.ssh_keys
	end

	# Used by QuickUtils and ImportUtils.
	# This does some pretty basic and silent validation -- if you're malformed,
	# you get dropped. Otherwise, the ssh_pubkey module will be happy to let
	# you know that you're bad news.
	def validate_imported_ssh_pubkeys
		return unless self.imported_ssh_pubkeys.respond_to? :uniq!
		self.imported_ssh_pubkeys.uniq!
		#  TODO: do validation here
		#  Now actually add the pubkey
		add_uniq_ssh_pubkeys()
	end

	def add_uniq_ssh_pubkeys
		self.imported_ssh_pubkeys.flatten.each { |key|
			next if self.factory.ssh_pubkeys.include? key
			# Fix up any whitespace, drop extra elements
			fixed_key = key.split[0,3].join("\s") 
			self.factory.ssh_pubkeys << key
		}
		self.factory.ssh_pubkeys
	end

	# MD5sums the important parts of an ssh key, ignoring whitespace
	# and hyphens. We do care about the begin and end lines insomuch that
	# we care that it's RSA or DSA.
	def ssh_key_md5(str)
		return unless str.respond_to? :gsub
		Rex::Text.md5_raw(str.gsub(/[\s-]/,""))
	end

end

# Handles all the functions having to do with finding known credentials in
# the database, making the decisions to include them in the known list or
# not, uniquing them, and pretty much everything else having to do with
# knowns. 
module KnownUtils

	# This boolean depends on the Cred and not the builder. SMB is
	# going to be a special case with the preserve domains business.
	def should_strip_domain?(cred)
		if cred.service.name.to_s[/^smb/]
			if this_service == :smb
				return !!self.factory.ds["Preserve_Domains"]
			end
		end
		!!(cred.service.name.to_s[/^(smb|postgres|db2|oracle)/i] &&
			cred.user &&
			cred.user[/^[^\x2f]+\x2f./]
		)
	end

	# This boolean controls which usernames and passwords get
	# picked up by those sections further down the chain:
	# Recombined, GuessedPassword, and MutantPassword.
	#
	# By default, quick, imported, and known are subject to seeding,
	# while common isn't. Common is probably the most variable
	# per protocol. For example, the shell utils have a lot
	# of specific usernames and passwords, and it doesn't make
	# a lot of sense to test those non-defaults, while something
	# like db2 has a very few number of users, with highly variable
	# passwords that tend to get changed away from the defaults.
	#
	# Note that credentials mentioned in the class constant of
	# ONE_OFF_CREDENTIALS will never be subject to seeding.
	def seed_with_these_creds?
		true
	end

	def section_cred_creator
		find_workspace_creds
		users = find_known_users
		# Skip users we already have known passwords for
		users = users.reject {|x| creds_by_user.keys.include? x}
		unless self.factory.ds["Bruteforce_Skip_Machine_Names"]
			users = users | find_machine_names
		end
		users.each {|u| add_seed u}
	end

	def find_workspace_creds
		self.workspace.creds.each do |cred| 
			next if should_skip? cred
			if cred.ptype == "smb_hash"
				next unless this_service == :smb
			end
			next if cred.ptype == "ssh_key"
			this_cred = cred.pass ? [cred.user,cred.pass].join(" ") : cred.user
			add_cred this_cred
		end
		self.creds
	end

	# Takes a String (username) Fixnum (uid) or Cred (or anything that
	# responds to #user()
	#
	# TODO: Are there more? I'm thinking that the user might want to skip
	# accounts like TsInternetUser, krbtgt, NetShowServices, IUSR_*, IWAM_*
	def is_user_useless?(user)
		this_user = user.respond_to?(:user) ? user.user : user
		case this_user
		when Fixnum
			return true if user == 501 # UID for a guest account, always disabled
			return false
		when String
			return true if user[0,1] == "_" # Mac OS dummy account
			if self.factory.ds["Bruteforce_Skip_Builtin_Windows_Accounts"]
				builtins = %w{ TsInternetUser krbtgt NetShowServices }
				return true if builtins.include? user
				%w{IUSR_ IWAM_ WMUS_USER- }.each do |builtin|
					return true if user =~ /^#{builtin}/ 
				end
			end
			if self.factory.ds["Bruteforce_Skip_Builtin_Unix_Accounts"]
				builtins = %w{
					daemon bin sys sync games man lp mail news uucp proxy www-data
					backup list irc gnats nobody libuuid syslog messagebus haldaemon hplip
					avahi couchdb kernoops saned pulse gdm sshd telnetd dhcp
					avahi-autoipd speech-dispatcher 
				}
				return true if builtins.include? user
			end
			return true if user[/^SUPPORT_[0-9a-f]{8}$/i] # Built-in support, always disabled
			return false
		else
			return true # Botched up input, assume it's bad
		end
	end

	def find_known_users
		enum_users = []
		lookupsid_users = []
		finger_users = []

		self.workspace.notes.find_all_by_ntype('smb.domain.enumusers').each do |u|
			if u.data and u.data[:users].respond_to? :each_key
				u.data[:users].each_key do |uid|
					next if is_user_useless?(uid)
					next if is_user_useless?(u.data[:users][uid])
					enum_users << u.data[:users][uid].to_s.downcase
				end
			end
		end

		self.workspace.notes.find_all_by_ntype('smb.domain.lookupsid').each do |u|
			if u.data and u.data[:users].respond_to? :each_key
				u.data[:users].each_key do |uid|
					next if is_user_useless?(uid)
					next if is_user_useless?(u.data[:users][uid])
					lookupsid_users << u.data[:users][uid].to_s.downcase
				end
			end
		end

		# Finger users
		self.workspace.notes.find_all_by_ntype('finger.users').each do |u|
			if u.data and u.data[:users].respond_to? :each
				u.data[:users].each do |username|
					next if is_user_useless?(username)
					finger_users << username
				end
			end
		end

		# TODO: This should be controllable more directly by datastore
		(enum_users | lookupsid_users | finger_users).uniq
	end

	def find_machine_names
		self.workspace.hosts.map {|x| x.name.to_s.split(/\x2e/).first}.compact.reject {|x| x.empty?}
	end

end

module QuickUtils

	# Don't seed with our gen'ed default set, just with creds the user
	# provides. Otherwise, recombined will pick them up.
	def seed_with_these_creds?
		!factory.ds["BRUTEFORCE_QUICKMODE_CREDS"].to_s.strip.empty?
	end

	def section_cred_creator
		create_quick_creds()
	end

	# Reads in the "Additional Credentials" feild that gets passed to
	# the datastore, skipping any ssh key chunks and handling those elsewhere.
	def create_quick_creds
		self.creds = []
		self.imported_ssh_keys ||= []
		self.imported_ssh_pubkeys ||= []
		in_ssh_key = nil
		quick_creds = self.factory.ds["Bruteforce_QuickMode_Creds"].to_s.strip.split("\n")
		quick_creds.each do |pair|
			is_ssh_pubkey = false
			key_status = detect_incoming_ssh_key_cred(pair)
			if key_status == :start_ssh_key
				in_ssh_key = true
			elsif key_status == :end_ssh_key
				in_ssh_key = false
			elsif key_status == :is_ssh_pubkey
				is_ssh_pubkey = true
			end
			if in_ssh_key || key_status.kind_of?(Symbol) || is_ssh_pubkey
				import_ssh_key_line(pair)
				next
			end
			these_creds = parse_userpass_pair(pair)
			these_creds.each { |c| add_cred c }
		end
		if self.creds.empty?
			default_quick_creds.each {|c| add_cred c }
		end
		# If this is non-nil, we got at least one key, so validate them
		validate_imported_ssh_keys unless in_ssh_key.nil?
		if self.imported_ssh_pubkeys.respond_to?(:size) and self.imported_ssh_pubkeys.size > 0
			validate_imported_ssh_pubkeys
		end
		self.creds
	end

	def default_quick_creds
		creds = %W{
			admin\ admin
			admin\ admin1
			admin\ admin!
			test\ test
			test\ test1234
			test123\ test123
			cisco\ cisco
			user\ user
			administrator\ administrator
			root\ root
			root\ toor
			admin
			test
			test123
			cisco
			user
			administrator
			root
		}
		if blank_usernames_allowed?
			creds << " admin"
		end
		return creds
	end

end

# Most classes that include this are going to want
# to redefine common_users and common_passwords
module CommonUtils

	def section_cred_creator
		create_common_creds()
	end

	def create_common_creds
		users = create_common_users
		passes = create_common_passwords
		create_creds_from_users_and_passes(users,passes)
		create_username_based_credentials()
		create_additional_creds
		if self.flavor && !self.flavor.empty?
			wordlist_proto = [this_service,self.flavor].join("_")
		else
			wordlist_proto = this_service
		end
		create_creds_from_protocol_wordlists(wordlist_proto)
	end

	def common_users
		%w{admin}
	end

	def common_passwords
		%w{!! 123456 12345678 111111 12345 
			qwerty abc123 letmein monkey link182
			changeme test1234}
	end

	# If there are specific credentials to add, put them here.
	def create_additional_creds
		if self.class.const_get(:ONE_OFF_CREDENTIALS)
			self.class.const_get(:ONE_OFF_CREDENTIALS).each do |cred|
				add_cred cred
			end
		end
	end

end

# Our really long list of default credentials lives here.
module CommonShellUtils

	# TODO: Merge in: http://www.vulnerabilityassessment.co.uk/fgdump_passwords.txt
	def common_shell_creds
		# Blank username, admin password. Linksys default, see #3022
		add_cred " admin"

		# HPUX as of 2010-06-07 apparently
		add_cred "catadm catadm"
		add_cred "security transfer"

		add_cred "pix cisco"      # Cisco PIX
		add_cred "root alpine"    # iPhone
		add_cred "root dottie"    # iPhone
		add_cred "hscroot abc123" # IBM HSC
		add_cred "admin admin123" # NAI Intrushield IPS

		# DRAC5
		add_cred "root calvin"

		# Various (BIG-IP, Meridian)
		add_cred "service smile"
		add_cred "service sm1l3"

		# Various DVR gear
		add_cred "root admin"

		# Tons of equipment, including Snap servers, Viking VPNs, etc
		add_cred "admin admin"
		add_cred "guest guest"
		add_cred "root default"

		# SimpleNET
		add_cred "sshadmin simplenet"

		# BlueCoat
		add_cred "admin articon"

		# Radware
		add_cred "radware radware"
		add_cred "lp lp"

		# AX-100P TrixBox / AtCom IP-PBX
		add_cred "root 12xerXes06"

		# MN4L
		add_cred "root 0000"

		# IP04
		add_cred "root uClinux"

		# InReach MRV
		add_cred "InReach access"
		add_cred "Superuser system"

		# AdTran System Controller Unit
		add_cred "ADMIN PASSWORD"

		# Baytech DS
		add_cred "root baytech"

		# APC
		add_cred "apc apc"
		add_cred "device apc"

		# Avaya
		add_cred "avniuadm r8nd0m"

		# WANScaler: http://support.citrix.com/article/CTX112404 (ssh is the same as web)
		add_cred "admin orbital"
		add_cred "admin wanscaler"

		# VirtualIQ
		add_cred "IQMANAGER IQ"

		# Cisco ANA
		add_cred "sheer sheer"

		# ISPAdmin
		add_cred "root ispadmin"

		# WD ShareSpace
		add_cred "downloader downloader"

		# Plesk
		add_cred "admin setup"

		# Sentry
		add_cred "admn admn"

		# SonicWall: https://www.fuzeqna.com/sonicwallkb/consumer/kbdetail.asp?kbid=3641
		add_cred "snwlcli password"
		add_cred "root password"
		add_cred "admin password"

		# PASTA Microsensor
		add_cred "root rootme"

		# Raritan
		add_cred "admin raritan"
		add_cred "dominion "

		# Various
		add_cred "default "
		add_cred "nobody " unless self.flavor == :apple 

		# Trend Micro - Threat Discovery Appliance
		add_cred "tda "
		add_cred "tda admin"

		# AutumnTECH Spam Fighting Appliance
		add_cred "root password"
		add_cred "spam password"

		# Cymphonix
		add_cred "menu cymphonix"

		# Vonage
		add_cred "user user"

		# VicidialNOW (mysql as well)
		add_cred "root vicidialnow"

		# LANTRONIX SCS100/200/400
		add_cred "login access"
		add_cred "root system"

		# Porttracker
		add_cred "admin appliance"

		# Wireless LAN Device Series ZWA-G120 User's Manual
		add_cred "root qwert"

		# Allied Telesis
		add_cred "manager friend"

		# OpenSLUG
		add_cred "root opeNSLUg"

		# TANDBERG
		add_cred "admin TANDBERG"

		# Various
		add_cred "root cisco123"

		# Digi Passport
		add_cred "root dbps"

		# Various
		add_cred "admin guest"
		add_cred "admin default"

		# Cyclades ACS
		add_cred "root tslinux"

		# Yoggie
		add_cred "root yoggie"

		# NetOptics
		add_cred "customer netoptics"

		# JetNEXUS
		add_cred "admin jetNEXUS"

		# Vyatta
		add_cred "root vyatta"

		# Various dev kits
		add_cred "root pass"

		# SPA500 Firewall
		add_cred "admin pfsense"

		# SilverStorm 9024
		add_cred "admin adminpass"

		# BrightMail Appliance
		add_cred "admin symantec"

		# FreeNAS
		add_cred "admin freenas"

		# FreeWRT
		add_cred "admin FreeWRT"

		# NBox
		add_cred "root nbox"

		# Wyse
		add_cred "root wyse"

		# Vertex
		add_cred "root vertex25"

		# AlienTechnology ALR-9900 default root password and backdoor
		# http://seclists.org/fulldisclosure/2010/May/63
		add_cred "root alien"

		# PwnPlug, see:
		# http://pwnieexpress.com/pdfs/PwnplugManual_0.3.pdf
		add_cred "root pwnplug8000"

		### Telnet Credentials ###

		# http://www.phenoelit-us.org/dpl/dpl.html
		# Looks like a fine list; lots of root passwords though which will slow
		# things down for Ubuntu machines quite a bit. Need to fix up some kind
		# of parallel model in the telnet module proper to do this right, and
		# probably need to wise up after the fifth root timeout that hey, root
		# is never going to work here.
		add_cred 'admin admin'
		add_cred 'admin default'
		add_cred 'admin hello'
		add_cred 'Administrator admin'
		add_cred 'Administrator ggdaseuaimhrke'
		add_cred 'Administrator '
		add_cred 'admin '
		add_cred 'admin P@55w0rd!'
		add_cred 'admin password'
		add_cred 'admin switch'
		add_cred 'ADMN admn'
		add_cred 'adm '
		add_cred 'Alphanetworks wrgg15_di524'
		add_cred 'bbsd-client changeme2'
		add_cred 'bbsd-client NULL'
		add_cred 'cablecom router'
		add_cred 'cisco '
		add_cred 'corecess corecess'
		add_cred 'debug synnet'
		add_cred 'd.e.b.u.g User'
		add_cred 'diag switch'
		add_cred 'D-Link D-Link'
		add_cred 'draytek 1234'
		add_cred 'echo User'
		add_cred 'GEN1 gen1'
		add_cred 'GEN2 gen2'
		add_cred 'guest User'
		add_cred 'iclock timely'
		add_cred 'installer installer'
		add_cred 'login admin'
		add_cred 'login password'
		add_cred 'manage !manage'
		add_cred 'Manager Admin'
		add_cred 'Manager Manager'
		add_cred 'Manager '
		add_cred 'MD110 help'
		add_cred 'netopia netopia'
		add_cred 'operator '
		add_cred 'recovery recovery'
		add_cred 'root 1234'
		add_cred 'root admin'
		add_cred 'root admin_1'
		add_cred 'root default'
		add_cred 'root test'
		add_cred 'root ggdaseuaimhrke'
		add_cred 'root iDirect'
		add_cred '!root '
		add_cred 'root '
		add_cred 'root pass'
		add_cred 'root root'
		add_cred 'root tini'
		add_cred 'security security'
		add_cred 'service smile'
		add_cred 'super super'
		add_cred 'super surt'
		add_cred 'superuser 123456'
		add_cred 'superuser admin'
		add_cred 'superuser '
		add_cred 'support support'
		add_cred 'sysadm Admin'
		add_cred 'sysadm anicust'
		add_cred 'sysadm sysadm'
		add_cred 'target password'
		add_cred 'tech '
		add_cred 'tech tech'
		add_cred 'User '
		add_cred 'write private'
	end
end

module ImportedUtils

	attr_accessor :imported_users, :imported_passes

	def files_and_types
		ft = {}
		self.factory.ds["CRED_FILE_IDS"].split(/\s+/).each do |id|
			fpath = self.workspace.cred_files.find(id).path rescue nil
			ftype = self.workspace.cred_files.find(id).ftype rescue nil
			ft[fpath] = ftype if fpath
		end
		return ft
	end

	def seed_with_these_creds?
		true
	end

	def section_cred_creator
		collect_imported_credentials
	end

	def collect_imported_credentials
		self.imported_users  ||= []
		self.imported_passes ||= []
		self.imported_ssh_keys ||= []
		self.imported_ssh_pubkeys ||= []
		files_and_types.each do |file,type|
			next unless ::File.readable? file
			case type
			when "userpass"
				parse_imported_userpass_file(file)
			when "user"
				parse_imported_user_file(file)
			when "pass"
				parse_imported_pass_file(file)
			when "pwdump"
				parse_imported_pwdump_file(file)
			when "ssh_keys"
				parse_imported_ssh_key_file(file)
			end
		end
		create_creds_from_users_and_passes(self.imported_users,self.imported_passes)
	end

	def parse_imported_ssh_key_file(file)
		fh = ::File.open(file, "rb")
		in_ssh_key = nil
		begin
			fh.each_line do |line|
				is_ssh_pubkey = false
				next unless line || !line.empty?
				key_status = detect_incoming_ssh_key_cred(line.strip)
				case key_status
				when :start_ssh_key
					in_ssh_key = true
				when :end_ssh_key
					in_ssh_key = false
				when :is_ssh_pubkey
					is_ssh_pubkey = true
				end
				if in_ssh_key || key_status.kind_of?(Symbol)
					import_ssh_key_line(line.strip)
					next
				end
			end
		ensure
			fh.close
		end
		# If this is non-nil, we got at least one key, so validate them
		validate_imported_ssh_keys unless in_ssh_key.nil?
		if self.imported_ssh_pubkeys.respond_to?(:size) and self.imported_ssh_pubkeys.size > 0
			validate_imported_ssh_pubkeys 
		end
		return [self.imported_ssh_keys, self.imported_ssh_pubkeys]
	end

	def parse_imported_pass_file(file)
		fh = ::File.open(file, "rb")
		begin
			fh.each_line do |line|
				i = 0
				next if line.to_s.strip.empty?
				this_pass = parse_just_passes(line.chomp)
				next if this_pass.nil? || this_pass.empty?
				passes_dehexed = self.imported_passes.map {|x| Rex::Text.dehex(x)}
				this_pass_dehexed = Rex::Text.dehex(this_pass)
				self.imported_passes << this_pass unless passes_dehexed.include? this_pass_dehexed
			end
		ensure
			fh.close
		end
		return self.imported_passes
	end

	def parse_imported_user_file(file)
		fh = ::File.open(file, "rb")
		begin
			fh.each_line do |line|
				this_user = parse_just_users(line)
				next unless this_user
				next if this_user.empty?
				users_dehexed = self.imported_users.map {|x| Rex::Text.dehex(x)}
				this_user_dehexed = Rex::Text.dehex(this_user)
				self.imported_users << this_user unless users_dehexed.include? this_user_dehexed
			end
		ensure
			fh.close
		end
		return self.imported_users
	end

	# Similar to parse_userpass_pair, parse_just_users takes a string,
	# strips it, and returns it. It also
	# deals with PWDump lines by chopping at the first colon. Unlike
	# parse_userpass_pair, we'll suck the usernames out regardless if
	# the target host is actually smb.
	#
	# Note that if someone accidentally calls a userpass file a user file,
	# we should be able to handle it sensibly (ie, only returning the 
	# userpart)
	def parse_just_users(user)
		this_user = nil
		# Test for a pwdump-style of SMB hashes.
		if user.to_s.strip =~ /^([^\s:]+):[0-9]+:([A-Fa-f0-9]+:[A-Fa-f0-9]+):[^\s]*$/
			this_user = $1
		else
			userpart = user.to_s.split(/\s+/).first
			this_user = userpart 
		end
		return this_user.to_s.strip.lstrip
	end

	# Unlike usernames, we can't be sure what the user meant if they mislabel their
	# file. Is it user pass, user pass1 pass2, or pass1 pass2? It's best to just take
	# them at their word, and just take the whole line as a password. This means that
	# passwords can easily have spaces, which is often complained about.
	#
	# That said, we can still tell if it's a PWDump file, and return the hash part
	# if the target can consume that.
	def parse_just_passes(pass)
		this_pass = nil
		# Test for a pwdump-style of SMB hashes.
		if pass.to_s.strip =~ /^([^\s:]+):[0-9]+:([A-Fa-f0-9]+:[A-Fa-f0-9]+):[^\s]*$/
			if this_service == :smb
				hash = $2
				this_pass = hash 
			end
		else
			this_pass = pass.to_s
		end
		if this_pass && !this_pass.strip.empty?
			this_pass.gsub(/\s/) {|x| "\\x%02x" % x.ord}
		end
	end

	# The most straightforward file parser we have.
	def parse_imported_userpass_file(file)
		fh = ::File.open(file, "rb")
		begin
			fh.each_line do |pair|
				parse_userpass_pair(pair).each { |c| add_cred c }
			end
		ensure
			fh.close
		end
	end

	# Mostly reanimated from the old credential importer, now with
	# SSH key smarts. TODO: This detect_incoming_ssh_key_cred really
	# needs to be a proc that just gets reused. Refactor this since
	# it's in like 3 different places now.
	def parse_imported_pwdump_file(file)
		return self.creds unless ::File.readable? file
		in_ssh_key = nil
		begin
			fh = ::File.open(file, "rb")
			fh.seek 0
			fh.each_line do |line|
				next if line.to_s.strip.empty?
				is_ssh_pubkey = false
				key_status = detect_incoming_ssh_key_cred(line)
				if key_status == :start_ssh_key
					in_ssh_key = true
				elsif key_status == :end_ssh_key
					in_ssh_key = false
				elsif key_status == :is_ssh_pubkey
					is_ssh_pubkey = true
				end
				if in_ssh_key || key_status.kind_of?(Symbol) | is_ssh_pubkey
					import_ssh_key_line(line)
					next
				end
				# We only get here if it's /not/ an ssh key
				user = nil; pass = nil; ptype = nil
				case line.to_s.strip
				when /^[^\s]+\s'[0-9a-f:]{47}'/ # SSH key fingerprint.
					next
				when /^[\s]*#/ # Comment lines
					next
				when /^[\s]*Warning:/
					next # Discard warning messages.
				when /^[\s]*([^\s:]+):[0-9]+:([A-Fa-f0-9]+:[A-Fa-f0-9]+):[^\s]*$/ # SMB Hash
					user = ([nil, "<BLANK>"].include?($1)) ? "" : $1
					pass = ([nil, "<BLANK>"].include?($2)) ? "" : $2
					ptype = "smb_hash"
				when /^[\s]*([^\s:]+):([0-9]+):NO PASSWORD\*+:NO PASSWORD\*+[^\s]*$/ # SMB Hash
					user = ([nil, "<BLANK>"].include?($1)) ? "" : $1
					pass = ""
					ptype = "smb_hash"
				when /^[\s]*([\x21-\x7f]+)[\s]+([\x21-\x7f]+)?/ # Must be a space-delimited user pass entry
					user = ([nil, "<BLANK>"].include?($1)) ? "" : $1
					pass = ([nil, "<BLANK>"].include?($2)) ? "" : $2
					ptype = "userpass"
				else # No spaces, so it's either an SSH key or a password.
					next if in_ssh_key
					user = ""
					pass = line.to_s.strip
					ptype = "password"
				end # case
				if [user,pass,ptype].compact.size == 3
					case ptype
					when "smb_hash"
						if this_service == :smb
							add_cred("%s %s" % [user,pass])
						end
					when "userpass","password"
						add_cred("%s %s" % [user,pass])
					end
				end
			end
		ensure
			fh.close
		end
		validate_imported_ssh_keys unless in_ssh_key.nil?
		if self.imported_ssh_pubkeys.respond_to?(:size) and self.imported_ssh_pubkeys.size > 0
			validate_imported_ssh_pubkeys
		end
		return self.creds
	end
end

module RecombinedUtils

	def section_cred_creator
		create_recombined_credentials
	end

	def create_recombined_credentials
		users = self.factory.seed_users
		if just_usernames?
			users.each { |cred| add_cred cred}
		else
			passes = self.factory.seed_passes
			create_creds_from_users_and_passes(users,passes)
			create_username_based_credentials(users)
		end
	end
end

module UnixWordlistUtils

	def wordlist_common_passwords
		passes = []
		pass_file = File.join(Msf::Config.install_root, "data", "wordlists", "unix_passwords.txt")
		if File.exist?(pass_file)
			passes = File.open(pass_file) {|f| f.read(f.stat.size)}.split(/\n/)
		end
		return passes.uniq
	end

	def wordlist_common_users
		users = []
		user_file = File.join(Msf::Config.install_root, "data", "wordlists", "unix_users.txt")
		if File.exist?(user_file)
			users = File.open(user_file) {|f| f.read(f.stat.size)}.split(/\n/)
		end
		return users.uniq
	end

end

module GuessedPasswordUtils

	def section_cred_creator
		create_password_guesses
	end

	def create_password_guesses
		users = self.factory.seed_users
		if just_usernames?
			users.each {|cred| add_cred cred}
		else
			passes = wordlist_common_passwords
			create_creds_from_users_and_passes(users,passes,:pass_user)
		end
	end

end	

module GuessedUserpassUtils

	def section_cred_creator
		create_userpass_guesses
	end

	def create_userpass_guesses
		users = wordlist_common_users
		if just_usernames?
			users.each {|cred| add_cred cred}
		else
			passes = wordlist_common_passwords
			create_creds_from_users_and_passes(users,passes)
		end
	end

end	


module MutationMethods

	PUNCTUATION_ARRAY = ((33..47).to_a | (58..64).to_a | (91..96).to_a | (123..126).to_a).map {|i| i.chr}

	# Takes a password, chops all the trailing digit off,
	# and adds 0..9
	def append_single_digit
		self.seed_creds.each do |user,pass|
			base_pass = pass.gsub(/[0-9]$/,"")
			(0..9).each {|i| 
				this_cred = "#{user} #{base_pass}#{i}"
				add_cred this_cred
			}
		end
		return self.creds
	end

	# Takes a password, chops all the leading digit off,
	# and adds 0..9
	def prepend_single_digit
		self.seed_creds.each do |user,pass|
			base_pass = pass.gsub(/^[0-9]/,"")
			(0..9).each {|i| 
				this_cred = "#{user} #{i}#{base_pass}"
				add_cred this_cred
			}
		end
		return self.creds
	end

	# Takes a password, and if it has numerals in it, replace them with all
	# other numbers. Currently ignores any token longer than 2 digits,
	# or passwords lacking any alphas. Also limits the combinations to
	# a max of two to avoid exploding both the password list.
	def substitute_single_digits
		self.seed_creds.each do |user,pass|
			next unless(pass =~ /[0-9]/)
			next unless(pass =~ /[A-Z]/i)
			pass_tokens = tokenize(pass)
			indeces = []
			pass_tokens.each_with_index do |token,idx|
				next unless token.kind_of? Fixnum
				next if token > 9
				indeces << idx
			end
			# Fake combinatorial design, only two cases.
			# If you either have 0 single digit tokens, or more than 2, skip.
			# (Presuming we don't want to blow 1000 slots on one password that
			# might just be leetspeak anyway)
			case indeces.size
			when 1
				this_pass = pass_tokens.dup
				10.times do |i| 
					this_pass[indeces.first] = i
					add_cred "#{user} #{this_pass.join}"
				end
			when 2
				this_pass = pass_tokens.dup
				10.times do |i| 
					10.times do |j|
						this_pass[indeces.first] = i
						this_pass[indeces.last] = j
						add_cred "#{user} #{this_pass.join}"
					end
				end
			end
		end
	end

	def l33t_passwords
		self.seed_creds.each do |user,pass|
			leet_pws = []
			next unless(pass =~ /[EATOIL]/i) # anagram lol
			leet_pws << pass.tr("Oo","0") if pass =~ /O/i
			leet_pws << pass.tr("Ii","1") if pass =~ /I/i
			leet_pws << pass.tr("Ii","!") if pass =~ /I/i
			leet_pws << pass.tr("Ll","1") if pass =~ /L/i
			leet_pws << pass.tr("Ll","!") if pass =~ /L/i
			leet_pws << pass.tr("Ee","3") if pass =~ /E/i
			leet_pws << pass.tr("Aa","4") if pass =~ /A/i
			leet_pws << pass.tr("Aa","@") if pass =~ /A/i
			leet_pws << pass.tr("Tt","7") if pass =~ /T/i
			leet_pws << pass.tr("OoIiEeAaTt","0011334477") if pass =~ /OIEAT/i
			leet_pws << pass.tr("OoIiEeAaTt","00!!334477") if pass =~ /OIEAT/i
			leet_pws << pass.tr("OoLlEeAaTt","0011334477") if pass =~ /OLEAT/i
			leet_pws << pass.tr("OoLlEeAaTt","00!!334477") if pass =~ /OLEAT/i
			leet_pws << pass.tr("OoIiEeAaTt","001133@@77") if pass =~ /OIEAT/i
			leet_pws << pass.tr("OoIiEeAaTt","00!!33@@77") if pass =~ /OIEAT/i
			leet_pws << pass.tr("OoLlEeAaTt","001133@@77") if pass =~ /OLEAT/i
			leet_pws << pass.tr("OoLlEeAaTt","00!!33@@77") if pass =~ /OLEAT/i
			leet_pws.each { |pw| add_cred "#{user} #{pw}"}
		end
	end

	# Takes a password, chops all the trailing punc off,
	# and adds all the punc characters. 
	def append_single_punc
		self.seed_creds.each do |user,pass|
			if PUNCTUATION_ARRAY.include? pass[-1,1]
				base_pass = pass[0,pass.size-1]
			else
				base_pass = pass
			end
			PUNCTUATION_ARRAY.each {|i| 
				this_cred = "#{user} #{base_pass}#{i}"
				add_cred this_cred
			}
		end
		return self.creds
	end

	# Takes a password, chops all the leading digit off,
	# and adds 0..9
	def prepend_single_punc
		self.seed_creds.each do |user,pass|
			if PUNCTUATION_ARRAY.include? pass[0,1]
				base_pass = pass[1,pass.size]
			else
				base_pass = pass
			end
			PUNCTUATION_ARRAY.each {|i| 
				this_cred = "#{user} #{i}#{base_pass}"
				add_cred this_cred
			}
		end
		return self.creds
	end

	# Takes a string (a password), and returns a tokenized
	# array split by alpha, numeric, and other. For example:
	# tokenize("foobar") => ["foobar"]
	# tokenize("foo3bar") => ["foo", 3, "bar"]
	# tokenize("32foo1!bar") => [32, "foo", 1, "!", "bar"]
	def tokenize(str)
		last_token = nil
		tokens = []
		str.each_char do |x|
			token = case x
							when /[0-9]/
								:num
							when /[A-Z]/i
								:alph
							else
								:other
							end
			if token == last_token
				tokens.last << x
			else
				tokens << x
			end
			last_token = token
		end
		tokens.map {|tok| tok.to_i.to_s == tok ? tok.to_i : tok}
	end


end

# Originally, it was conceived that seeding usernames and passwords would be useful
# for mutations, but due to the geometric effects of recombining users and passwords,
# the lists rapidly become infeasable to handle.
#
# Instead, mutations should be applied to only those credentials the user is explicit
# about. This still allows for recombining, but also allows the user to limit their
# results if care is taken in importing credential files.
#
# So, mutations can be applied to :quick , :imported , and :known credentials, but
# will never be applied to :common, :recombined, :guessed_password, or :guessed_userpass
# 
# If there is a request for it, we can add mutations for seeded creds, but in 
# all cases, the actual, desired effect will be better achieved by the user simply
# importing a credfiles of type user and pass, and recombining credentials that way,
# since, by default, we tend to pick up a bunch of junk known usernames, and currently
# have no way to exclude them from either bruteforcing or the mutation process.
#
# Note that several of these methods take seed_pws as an argument. This has nothing
# to do with self.factory.seed_passes . Also note that since this builder depends
# on other builders, it should be generated after at least one of the source builders.
module MutantPasswordUtils

	# Unlike most creds, this starts off as [user,pass] -- it's pre-split
	attr_accessor :seed_creds

	# All the actual mutations 
	include MutationMethods

	def section_cred_creator
		return if just_usernames?
		generate_seed_creds()
		create_mutant_creds()
	end

	# Selects all the MUTATION_OPTS datastore options that are true-ish.
	def create_mutant_creds
		return if self.seed_creds.empty?
		# Kind of a funky selector
		mutations = factory.ds.select {|k,v| k =~ /^MUTATION_OPT/i && (v.to_s.downcase != "false" || !v.to_s.empty?)}.keys
		return if mutations.empty?
		mutation_methods = []
		mutations.each do |opt|
			meth = opt.match(/MUTATION_OPT_(.*)/i)[1].to_s.downcase
			mutation_methods << meth.to_sym unless meth.empty?
		end
		mutation_methods.uniq.each do |meth| 
			if MutationMethods.instance_methods.include? meth
				self.send(meth)
			else
				raise ArgumentError, "Unknown mutation `#{meth}'"
			end
		end
	end

	def generate_seed_creds
		self.seed_creds = []
		cred_sections = []
		if factory.ds['MUTATE_KNOWN']
			cred_sections << :known_userpass 
		end
		if factory.ds['MUTATE_QUICK']
			unless factory.ds['BRUTEFORCE_QUICKMODE_CREDS'].to_s.strip.empty?
				cred_sections << :quick 
			end
		end
		if factory.ds['MUTATE_IMPORTED']
			cred_sections << :imported
		end
		return self.seed_creds if cred_sections.empty?
		cred_sections.each do |sec|
			next unless factory.section(sec)
			factory.section(sec).each do |cred|
				next if(cred =~ /\\x[0-9A-Fa-f]{2}/) # Ignore hex anything, too complicated right now.
				user,pass = cred.split(/\s/,2)
				next if pass.to_s.strip.empty? # Ignore nil or whitespace-only pw's
				next if pass =~ /[A-Fa-f0-9]{32}:[A-Fa-f0-9]{32}/i # Ignore SMB hashes
				self.seed_creds << [user,pass] unless self.seed_creds.include? [user,pass]
			end
		end
		return self.seed_creds
	end

	# Converts passwords to use common number for letter
	# substitutions. 'L' or 'I' becomes '1', 'E' becomes '3',
	# etc. The first set is single substitutions, the next
	# is substituting all characters.
	def mutate_convert_characters(seed_pws)
		passwords = []
		seed_pws.each do |pass|
			next unless(pass =~ /[A-Za-z]/)
			next if(pass =~ /\\x[0-9A-Fa-f]{2}/) # Ignore hex pws
			passwords << pass.tr("Oo","0") if pass =~ /O/i
			passwords << pass.tr("Ii","1") if pass =~ /I/i
			passwords << pass.tr("Ii","!") if pass =~ /I/i
			passwords << pass.tr("Ll","1") if pass =~ /L/i
			passwords << pass.tr("Ll","!") if pass =~ /L/i
			passwords << pass.tr("Ee","3") if pass =~ /E/i
			passwords << pass.tr("Aa","4") if pass =~ /A/i
			passwords << pass.tr("Aa","@") if pass =~ /A/i
			passwords << pass.tr("Tt","7") if pass =~ /T/i
			passwords << pass.tr("OoIiEeAaTt","0011334477") if pass =~ /OIEAT/i
			passwords << pass.tr("OoIiEeAaTt","00!!334477") if pass =~ /OIEAT/i
			passwords << pass.tr("OoLlEeAaTt","0011334477") if pass =~ /OLEAT/i
			passwords << pass.tr("OoLlEeAaTt","00!!334477") if pass =~ /OLEAT/i
			passwords << pass.tr("OoIiEeAaTt","001133@@77") if pass =~ /OIEAT/i
			passwords << pass.tr("OoIiEeAaTt","00!!33@@77") if pass =~ /OIEAT/i
			passwords << pass.tr("OoLlEeAaTt","001133@@77") if pass =~ /OLEAT/i
			passwords << pass.tr("OoLlEeAaTt","00!!33@@77") if pass =~ /OLEAT/i
		end
		return passwords
	end

	def mutate_add_digits(seed_pws)
		passwords = []
		seed_pws.each do |pass|
			next unless(pass =~ /[A-Za-z]/)
			next if(pass =~ /\\x[0-9A-Fa-f]{2}/) # Ignore hex pws
			strip_pass = pass.gsub(/[0-9]+$/,"")
			("1".."9").each { |i| passwords.push strip_pass+i }
		end
		return passwords
	end

end	

	# This ought to be automatic but w/e
	def self.implemented_services
		[
			:telnet,
			:ssh,
			:db2,
			:mysql,
			:postgres,
			:oracle,
			:mssql,
			:ftp,
			:exec,
			:shell,
			:login,
			:vnc,
			:snmp,
			:smb,
			:http,
			:https,
			:pop3,
			:vmauthd,
			:afp,
			:pcanywhere_data,
			:winrm
		]
	end

end
end
end

Msf::Pro::Credentials.implemented_services.each do |svc|
	require File.join("pro","credfactory",svc.to_s)
end

