class TaskChainsController < ApplicationController
  include TasksSharedControllerMethods
  attr_reader :task_chain_data, :ice_cube_schedule

  before_filter :load_workspace

  def index
    @task_chains = TaskChain.for_workspace(@workspace).visible
  end

  def new
    @import_creds_task = ImportCredsTask.new(
      :workspace => @workspace,
      :username => current_user.username
    )
    @task_chain = TaskChain.new
  end

  def show
    @task_chain = TaskChain.find(params[:id]) 
  end

  def create
    @task_chain                            = TaskChain.new
    @task_chain.user                       = current_user
    @task_chain.name                       = params[:task_chain][:name]
    @task_chain.workspace                  = @workspace
    @task_chain.schedule                   = ice_cube_schedule
    @task_chain.clear_workspace_before_run = params[:task_chain][:clear_workspace_before_run]

    params['scheduled_task_types'].each_key do |k|
      unless params['inputs_for_task'][k]
        empty_error = "Task ##{k} (#{params['scheduled_task_types'][k]}) had an empty config! Please try again."
        respond_to do |format|
          format.js do
            render :js => "jQuery('#popup_errors').html('#{j empty_error}');"
          end
          format.html do
            flash[:error] = empty_error
            redirect_to new_workspace_task_chain_path
          end
        end
        return
      end
    end

    schedule_for_run
    config_hash_array.each_with_index do |config, index|
      position = index + 1

      new_task = @task_chain.scheduled_tasks.build
      new_task.kind        = config[0]
      new_task.config_hash = config[1]
      new_task.position    = position
    end
   
    if @task_chain.save
      flash.now[:notice] = "Chain '#{@task_chain.name}' has been created"
      render :action => "show"
    else
      @import_creds_task = ImportCredsTask.new(
        :workspace => @workspace,
        :username => current_user.username
      )
      render :action => :new
    end
  end


  def edit
    
  end

  def update
    
  end

  def destroy_multiple
    ids_to_delete = params[:task_chain_ids] || []
    if ids_to_delete.empty?
      flash[:error] = 'No task chains selected to delete.'
    else
      result = TaskChain.destroy ids_to_delete
      flash[:notice] = "#{view_context.pluralize(result.size, 'task chain')} deleted."
    end
    render_js_redirect workspace_task_chains_path(@workspace)
  end

  def run_now
    task_chain = TaskChain.find(params[:id])
    task_chain.schedule_to_run_at(Time.now.utc)
    task_chain.save
    flash[:notice] = "'#{task_chain.name}' has been scheduled to start in less than a minute."
    redirect_to :action => "index"
  end

private
  # NB: the order created in task_chain_data needs to be preserved for all of this to
  # work as designed.  Ordered hashes -- yet another reason to run Ruby 1.9.2+!

  # Create a nice fat hash that can be used to instantiate TaskConfig descendant models
  # and therefore create a bunch of ScheduledTasks
  def task_chain_data
    @task_chain_data ||= params[:scheduled_task_order].split(',').inject({}) do |hash, task_order_key|
      hash[task_order_key] = {}
      potential_type = params[:scheduled_task_types][task_order_key]
      if potential_type.include?("collect")
        potential_type = "collect_evidence"
      end
      hash[task_order_key][:type] = potential_type
      hash[task_order_key][:inputs_for_task] = params[:inputs_for_task][task_order_key]
      
      hash 
    end
  end

  # The hash we use to build the ScheduledTask objects in the TaskChain.
  def config_hash_array
    array = []
    task_chain_data.each_value do |input_hash|
      type = ""
      if input_hash.include? "collect" # just had to be different, didn't you?
        type = "collect_evidence"
      else
        type = input_hash[:type]
      end
      input_hash[:inputs_for_task][:workspace] = @workspace
      array << [type, process_task_config(type, input_hash[:inputs_for_task]).config_to_hash]
    end
    array
  end

  # Sets up the IceCube::Schedule object that's used to hold a recurrence schedule
  def ice_cube_schedule
    @ice_cube_schedule ||= make_schedule
  end

  def make_schedule
    # Just let #create manipulate next_run_at instead of setting up a Schedule
    return nil if (params[:schedule_type] == "now" || params[:schedule_type] == 'manual')

    type     = params[:schedule_recurrence][:frequency]
    config   = params[:schedule_recurrence][type]
    schedule = nil

    if type == "once"
      return nil
    elsif type == "hourly"
      minute_of_hour = config['minute'].to_i
      t = Time.now.utc

      #
      # If current time is past scheduled minutes
      # start in next hour else start this hour
      #
      start_hour = minute_of_hour < t.min ? t.hour + 1 : t.hour
      start_time = Time.local(t.year, t.month, t.mday, start_hour, minute_of_hour, 0).utc

      #
      # Create schedule object given start time and
      # add hourly recurrence rule
      #
      schedule   = IceCube::Schedule.new(start_time)
      schedule.add_recurrence_rule IceCube::Rule.hourly
    else
      # Common to any non-hourly recurrence schedule
      start_time = Time.parse(config[:time]).utc
      schedule   = IceCube::Schedule.new(start_time)

      case type

      when "daily"
        schedule.add_recurrence_rule IceCube::Rule.daily
      when "weekly"
        active_days = config[:days].map{|d| d.to_i }.flatten
        interval    = config[:interval].to_i
        schedule.add_recurrence_rule IceCube::Rule.weekly(interval).day(*active_days)
      when "monthly"
        # TODO: set up monthly recurrence for days and relative
        #   * day: on the 5th
        #   * relative: on the 2nd Tuesday
      end
    end
    schedule 
  end

  def schedule_for_run
    if should_schedule?
      run_time = determine_time_to_run
      schedule_for(run_time)
    end
  end

  def should_schedule?
    not (params[:schedule_type] == 'manual')
  end

  def determine_time_to_run
    if params[:schedule_type] == 'now'
      Time.now.utc
    elsif params[:schedule_type] == 'future'
      if params[:schedule_recurrence]['once_datetime'].present?
        request_time = params[:schedule_recurrence]['once_datetime']
        Time.strptime(request_time, "%m/%d/%Y %I:%M %P").utc
      else
        @task_chain.schedule.next_occurrence
      end
    end
  end

  def schedule_for(time)
    @task_chain.schedule_to_run_at(time)
  end
end
