# Holds code that enables a common interface for processing task configuration
# forms in both the TasksController and TaskChainsController views.

# -- RETURN CONFIG VS RENDERING VIEW --
# The launch_now boolean is used to determine whether to start the task now
# (and render an HTTP response)  or to simply return the finished config
# object for use in creating a ScheduledTask object for persistence.
#
# It's assumed that in the case of the latter, a wrapper method is
# handling the work of rendering an HTTP response.

module TasksSharedControllerMethods

private

  def process_task_config(task_type, task_params, launch_now=false)
    send("process_#{task_type}", task_params, launch_now)
  end

  # Used in a couple places for ReportTasks
  def process_report_params
    if params and params[:conf]
      @config = ReportConfig.new(params[:conf])
    else
      @config = ReportConfig.new(
        :addresses => @workspace.addresses
      )
    end
    # Logic for displaying errors shamelessly stolen from
    # models/scan_task
    unless @config.valid?
      respond_to do |format|
        format.js do
          render :js => "jQuery('#popup_errors').html('#{j @config.error}');"
        end
        format.html do
          flash[:error] = @config.error
          redirect_to reports_path
        end
      end
    end
  end


  def process_bruteforce(task_params, launch_now)
    @bruteforce_task = BruteforceTask.new(task_params.merge({:username => current_user.username}))
    if launch_now
      if (@task = @bruteforce_task.start)
        render_new_task(@task)
      else
        render_task_errors(@bruteforce_task)
      end 
    else
      return @bruteforce_task
    end
  end

  def process_nexpose_asset_group_push(task_params, launch_now)
    @nx_task = NexposeAssetGroupPushTask.new(task_params.merge({:username => current_user.username}))
    if launch_now
      if (@task = @nx_task.start)
        render_new_task(@task)
      else
        render_task_errors(@nx_task)
      end 
    else
      return @nx_task
    end
  end

  def process_nexpose_exception_push(task_params, launch_now)
    @nx_task = NexposeExceptionPushTask.new(task_params.merge({:username => current_user.username}))
    if launch_now
      if (@task = @nx_task.start)
        render_new_task(@task)
      else
        render_task_errors(@nx_task)
      end 
    else
      return @nx_task
    end
  end

  def process_cleanup(task_params, launch_now)
    @cleanup_task = CleanupTask.new(task_params.merge({:username => current_user.username}))
    if launch_now
      if (@task = @cleanup_task.start)
        render_new_task(@task)
      else
        render_task_errors(@cleanup_task)
      end
    else
      return @cleanup_task
    end
  end
  
  def process_upgrade_sessions(task_params, launch_now)
    @upgrade_sessions_task = UpgradeSessionsTask.new(task_params.merge({:username => current_user.username, :workspace => @workspace}))
    if launch_now
      if (@task = @upgrade_sessions_task.start)
        render_new_task(@task)
      else
        render_task_errors(@upgrade_sessions_task)
      end
    else
      return @upgrade_sessions_task
    end
  end
  

  def process_collect_evidence(task_params, launch_now)
    if task_params[:workspace_id].nil?
      task_params[:workspace_id] = @workspace.id
    end
    @collect_task = CollectEvidenceTask.new(task_params.merge({:username => current_user.username}))
    if launch_now
      if (@task = @collect_task.start)
        render_new_task(@task)
      else
        render_task_errors(@collect_task)
      end
    else
      return @collect_task
    end
  end

  def process_exploit(task_params, launch_now)
    @exploit_task = ExploitTask.new(task_params.merge({:username => current_user.username}))
    if launch_now
      if params[:back]
        @task = @exploit_task
        render "new_exploit"
      elsif (@task = @exploit_task.start)
        redirect_to(task_detail_path(@task.workspace, @task.id))
      else
        @task = @exploit_task
        @show_new_exploit_error = true # workaround for #4096
        render "new_exploit"
      end
    else
      return @exploit_task
    end
  end

  def process_import(task_params, launch_now)
    temp = nil
    path = nil
    if task_params.present? and task_params[:file].present?
      # Write the uploaded file to a tempfile
      temp = ::Rex::Quickfile.new('import')
      uploaded_io = task_params[:file]

      begin
        while (buff = uploaded_io.read(1024*64))
          temp.write(buff)
        end
      rescue ::EOFError
      end

      temp.flush
    else
      temp = nil
    end

    @import_task = ImportTask.new(
      task_params.merge(
        :path => (temp ? temp.path : nil),
        :workspace => @workspace,
        :username => current_user.username)
    )

    temp = nil

    # Launching now
    if launch_now
      if (@task = @import_task.start)
        respond_to do |format|
          format.html { redirect_to task_detail_path(@task.workspace, @task) }
          format.js do
            render_new_task(@task)
          end
        end
      else
        respond_to do |format|
          format.html do
            flash[:error] = @import_task.error
            redirect_to new_import_path
          end
          format.js do
            render_task_errors(@import_task)
          end
        end
      end

    # Launching on a schedule
    else
      return @import_task
    end
  end

  def process_module_run(task_params, launch_now)
    task_params.merge!({
      :username  => current_user.username,
      :workspace => @workspace,
      # XXX - look into making everything use either :select_action or just :action
      :action    => task_params[:selected_action],
    })

    if launch_now
      task_params[:module] = find_module_by_path
      if task_params[:options] and task_params[:options][:DynamicExe]
        exeopt = Option.new(:advanced => true, :default => true, :desc => "Generate a dynamic EXE payload", :name => "DynamicExe", :required => true, :type => "bool")
        task_params[:module].options.push(exeopt)
      end
      @module_run_task = ModuleRunTask.new(task_params)
      if (@task = @module_run_task.start)
        flash[:notice] = "Task started"
        redirect_to task_detail_path(@workspace, @task)
      else
        flash[:error] = @module_run_task.error
        redirect_to new_module_run_path(@workspace, @module_run_task.module.fullname)
      end
    else
      task_params[:module] = find_module_by_path(task_params[:path])
      if task_params[:options] and task_params[:options][:DynamicExe]
        exeopt = Option.new(:advanced => true, :default => true, :desc => "Generate a dynamic EXE payload", :name => "DynamicExe", :required => true, :type => "bool")
        task_params[:module].options.push(exeopt)
      end
      @module_run_task = ModuleRunTask.new(task_params)
      return @module_run_task
    end
  end

  def process_nexpose(task_params, launch_now)
    @nexpose_task = NexposeTask.new(task_params.merge({:username => current_user.username}))
    if launch_now
      if (@task = @nexpose_task.start)
        # Save preferences if the user has no existing nexpose settings
        if not current_user.nexpose_host
          current_user.nexpose_creds_type = @nexpose_task.nexpose_creds_type
          current_user.nexpose_creds_user = @nexpose_task.nexpose_creds_user
          current_user.nexpose_creds_pass = @nexpose_task.nexpose_creds_pass
          current_user.save!
        end
        render_new_task(@task)
      else
        render_task_errors(@nexpose_task)
      end
    else
      return @nexpose_task
    end
  end

  def process_report(task_params, launch_now)
    merge_hash = {:username => current_user.username}
    merge_hash[:additional_params], jxml_file = load_additional_report_params(task_params[:report_type])
    merge_hash[:report_task] = {}
    if task_params[:display_sections] and task_params[:display_sections].respond_to?(:keys)
      merge_hash[:report_task][:display_sections_string] = task_params[:display_sections].keys.sort.join(",")
    end
    @report_task = ReportTask.new(task_params.merge(merge_hash))
    populate_module_reference_table() unless task_params[:dont_populate_mod_refs].present?

    if jxml_file and not jxml_file.empty?
      @report_task.report_template = jxml_file
    end
    
    if launch_now
      if (@task = @report_task.start)
        render_new_task(@task)
      else
        render_task_errors(@report_task)
      end
    else
      return @report_task
    end
  end

  def process_scan(task_params, launch_now)
    @scan_task = ScanTask.new(task_params.merge(:username => current_user.username))
    if launch_now
      if (@task = @scan_task.start)
        render_new_task(@task)
      else
        render_task_errors(@scan_task)
      end
    else
      return @scan_task
    end
  end

  def process_webscan(task_params, launch_now)
    @webscan_task = WebscanTask.new(task_params.merge({:username => current_user.username}))
    if launch_now
      if (@task = @webscan_task.start)
        render_new_task(@task)
      else
        render_task_errors(@webscan_task)
      end 
    else
      return @webscan_task
    end
  end

  # Probably a temporary thing, requires the 20110928101300_add_mod_ref_table migration
  def populate_module_reference_table
    Mdm::ModRef rescue raise "The MsfModule migration 20110928101300_add_mod_ref_table has not been committed yet!"
    modules_and_refs = []
    MsfModule.all.each do |m|
      next if m.references.nil? || m.references.empty?
      this_mod = [m.fullname,m.references.map {|r| r.join("-")}.join("\n"),m.type]
      modules_and_refs << this_mod
    end
    modules_and_refs.each do |mod,ref,type|
      m = Mdm::ModRef.find_or_create_by_module(mod)
      m.ref = ref
      m.mtype = type
      m.save if m.changed?
    end
  end

  #
  # NB: The parameter names must match the parameter
  # names in the jasper definition file.
  #
  def load_additional_report_params(report_type)
    # ivars loaded in filter methods when starting task immediately
    # so we have to call the same ones here if they're not available
    
    process_report_params
    gather_users if @users.blank?
    gather_workspace if @workspace_created_at.blank?

    additional_params = {}
    jxml_file = nil
    if report_type.present?
      additional_params['workspace_name'] = @workspace_name
      additional_params['workspace_created'] = @workspace_created_at.to_s
      additional_params['workspace_updated'] = @workspace_updated_at.to_s
      additional_params['user'] = @users.keys.join(", ")
      jxml_file = case report_type[0]
      when /^SERVICES/ 
        'msfx_services.jrxml'
      when /^COMPROMISED/
        'msfx_compromised_hosts.jrxml'
      when /^AUTH/
        'msfx_auth_tokens.jrxml'
      when /^WEBAPP_ASSESSMENT/
        'webapp_assessment/main.jrxml'
      when /^WEBAPP/
        'msfxv3_webapp.jrxml'
      when /^CAMPAIGNS/
        'msfx_campaigns.jrxml'
      when /^SE_CAMPAIGN_DETAIL/
        'social_engineering/ms_se_campaign_detail.jrxml'
      when /^(EVIDENCE|LOOT)/
        'msfx_loot.jrxml'
      end
    end

    return additional_params, jxml_file
  end

  # Load via URL path from route or from other named param
  def find_module_by_path(lookup_param=nil)
    if lookup_param
      MsfModule.find_by_fullname(lookup_param) 
    else
      MsfModule.find_by_fullname(params[:path]) 
    end
  end

end
