class VulnsController < ApplicationController
	include AnalysisSearchMethods
	before_filter :find_vuln, :only => [:edit, :destroy, :update, :show, :details, :attempts, :exploits]
	before_filter :find_host, :only => [:new, :edit, :create, :show, :details, :attempts, :exploits ]
	before_filter :find_workspace, :only => [:new, :edit, :show, :details, :attempts, :exploits ]
	before_filter :load_workspace, :only => [:index, :combined, :destroy_multiple]
	before_filter :set_pagination, :only => [:index, :combined]
	before_filter :fix_datatables_search_parameters, :only => [:index]


	helper RefHelper

	def index
		load_vulns if request.format.json?

		session[:return_to] = workspace_vulns_path(@workspace)

		respond_to do |format|
			format.html
			format.json { render :partial => 'vulns', :vulns => @vulns }
		end
	end

	def combined
		load_combined_vulns if request.format.json?

		respond_to do |format|
			format.html
			format.json { render :partial => 'vulns_combined', :vulns => @vulns }
		end
	end

	def new
		@vuln = @host.vulns.new
		build_ref
	end

	def edit
		build_ref
	end

	def show
		build_ref
		build_exploit_match
	end

	def details
		build_ref
		build_exploit_match
		respond_to do |format|
			format.html { render :partial => "vuln_details_tab" }
			format.js { render :partial => "vuln_details_tab" }
		end
	end

	def attempts
		build_ref
		build_exploit_match
		respond_to do |format|
			format.html { render :partial => "vuln_attempts_tab" }
			format.js { render :partial => "vuln_attempts_tab" }
		end
	end

	def exploits
		build_ref
		build_exploit_match
		respond_to do |format|
			format.html { render :partial => "vuln_exploits_tab" }
			format.js { render :partial => "vuln_exploits_tab" }
		end
	end

	def create
		@vuln = @host.vulns.new(params[:vuln])

		if @vuln.save
			flash[:notice] = "Vulnerability added"
			redirect_to_hosts_show
		else
			build_ref
			find_host
			find_workspace
			render :action => 'new'
		end
	end

	def update
		params[:vuln][:existing_ref_attributes] ||= {}

		if @vuln.update_attributes(params[:vuln])
			flash[:notice] = "Vulnerability added"
			redirect_to_hosts_show
		else
			build_ref
			find_host
			find_workspace
			render :action => 'edit'
		end
	end

	def destroy
		@vuln.destroy

		respond_to do |format|
			format.html do
				flash[:notice] = "Vulnerability removed"
				redirect_to_hosts_show
			end
			format.js
		end
	end

	def destroy_multiple
		vulns = Mdm::Vuln.destroy params[:vuln_ids]

		if vulns.length.zero?
			flash[:error] = "No vulnerabilities selected to delete"
		else
			flash[:notice] = view_context.pluralize(vulns.length, "vuln") + " deleted"
		end

		render_js_redirect(workspace_vulns_path(@workspace))
	end

	private

	def find_vuln
		@vuln = Mdm::Vuln.find(params[:id])
	end

	def find_host
		if params[:host_id]
			@host = Mdm::Host.find(params[:host_id])
		else
			@host = @vuln.host
		end
	end

	def find_workspace
		@workspace = @host.workspace
	end

	# Determine the correct pagination arguments based on the incoming DT parameters.
	#
	# Returns nothing.
	def set_pagination
		if params[:iDisplayLength] == '-1'
			@vulns_per_page = nil
		else
			@vulns_per_page = (params[:iDisplayLength] || 20).to_i
		end
		@vulns_offset = (params[:iDisplayStart] || 0).to_i
	end

	# Load the Vulns with correct sorting and pagination.
	#
	# Returns nothing.
	def load_vulns
		vuln_set = pro_search(:vulns, false)
		@vulns = vuln_set
			.limit(@vulns_per_page)
			.offset(@vulns_offset)
			.order(vulns_sort_option)
			.select("hosts.name AS host_name, hosts.address, hosts.id AS host_id, " +
				# okay, Mdm::Host has some gotchas to sorting:
				# 1. Generally, sort Hosts by #name (type:string)
				# 2. If Host#name is blank, default to sorting by #address (type:inet)
				# 3. If Host#name is EQUAL to Host#address, default to sorting by #address (type:inet)
				#       (otherwise the address will be alphabetically sorted, WRONG)
				'(CASE hosts.name WHEN HOST(hosts.address::inet)::text THEN NULL ELSE LOWER(hosts.name) END) ' +
				'AS host_name_sort')
			.group("hosts.id")
		@vulns_total_display_count = vuln_set.length
	end

	# Load the Vulns with correct sorting and pagination.
	#
	# Returns nothing.
	def load_combined_vulns
		combined_vulns_group_option  = "vulns.name, vulns.info"
		vuln_set = pro_search(:vulns, true)
		@vulns = vuln_set.
			limit(@vulns_per_page).
			offset(@vulns_offset).
			order(vulns_combined_sort_option)
 
	
		@vulns_total_display_count = @vulns.length
	end

	# Generate a SQL sort by option based on the incoming DataTables parameter.
	#
	# Returns the SQL String
	def vulns_sort_option
		sort_dir = params[:sSortDir_0] =~ /^A/i ? 'asc' : 'desc'
		reverse_sort_dir = sort_dir == 'asc' ? 'desc' : 'asc';
		case params[:iSortCol_0]
		when '1'
			"host_name_sort #{sort_dir}, hosts.address #{sort_dir}"
		when '2'
			"services.port #{sort_dir}"
		when '3'
			"vulns.name #{sort_dir}"
		else
			"vulns.exploited_at #{sort_dir}, vulns.vuln_attempt_count #{reverse_sort_dir}"
		end
	end

	# Generate a SQL sort by option based on the incoming DataTables parameter.
	#
	# Returns the SQL String
	def vulns_combined_sort_option
		column = case params[:iSortCol_0]
		when '0'
			'name'
		when '2'
			'host_count'
		end
		column + ' ' + (params[:sSortDir_0] =~ /^A/i ? 'asc' : 'desc') if column
	end		


	def redirect_to_hosts_show
		redirect_to host_path(@vuln.host) + "#vulns"
	end

	def build_ref
		@vuln.refs.build
	end

	# Generate an include option for the tag table when loading records
	# for the Analysis tab.
	#
	# Returns the Array condition.
    def tags_include_option
	    if License.get.supports_tags?
	    	[:refs, {:host => :tags}]
	    else
	    	[:refs, :host]
	    end
    end

	def build_exploit_match
		@exploits = ::Mdm::ModuleDetail.select("DISTINCT(module_details.*)").
			from("module_details, module_refs, vulns_refs, refs").
			where(
				"module_details.id = module_refs.module_detail_id AND vulns_refs.vuln_id = ? AND " +
				"vulns_refs.ref_id = refs.id AND refs.name = module_refs.name AND " +
				"module_details.mtype IN (?)", 
				@vuln.id, ["exploit", "auxiliary"]
			).
			group("module_details.id").
			order("module_details.mtype DESC, module_details.rank DESC").
			all

	end
end

