module HostsHelper

	def link_to_add_service(f)
		link_to_add_field(f, :service, :class => Mdm::Service).html_safe
	end

	# Finds exploit modules matching a vuln and host,
  # Returns all matches found linked to the module page.
	def map_exploit(vuln, host)
    found_exploits = vuln.vuln_attempts.map(&:module)

		if found_exploits.empty?
      # Pull in system note (applies to Nexpose imports)
			ret = map_vuln_key(vuln) if @host
		else
      linked_exploits = found_exploits.map { |mod| link_to(h(mod), new_module_run_path(@workspace, mod, :host_ids => [host.id])) }.uniq.join(", ")
			ret = linked_exploits
		end
		ret = "" if ret.nil?
		return ret
	end

	# So far, only nexpose imports populate the vuln key system note, so just
	# deal with that.
	def map_vuln_key(vuln)
		return unless(vuln.name =~ Mdm::Vuln::NEXPOSE_REF_ID_PATTERN)
		vid = vuln.name.gsub(Mdm::Vuln::NEXPOSE_REF_ID_PATTERN,"")
		has_vuln_key = false
		@host.notes.visible.each do |note|
			next unless note.ntype == "host.vuln.nexpose_keys"
			next unless note.data.include? vid
			has_vuln_key = true
			break
		end
		if has_vuln_key
			link_to("See host.vuln.nexpose_keys", "#notes")
		end
	end

    def host_tag_cloud(host)
      return "".html_safe unless host.tags.size > 0
      return "".html_safe unless License.get.supports_tags?
      hsh = host.tag_frequency
      tag_links = []
      total_hosts = host.workspace.hosts.size.to_f
      hsh.each_pair do |name,freq|
        if freq == total_hosts
          fsize = 3
        elsif (freq <= total_hosts * 0.25) or freq == 1
          fsize = 0
        elsif freq <= total_hosts * 0.5
          fsize = 1
        elsif freq > total_hosts * 0.5
          fsize = 2
        elsif freq >= total_hosts * 0.75
          fsize = 3
        end
        pc = "%.1f" % (freq.to_f/total_hosts * 100)

        tag_link = content_tag(
          :div,
          link_to(
            h(name),
            hosts_path(host.workspace) << ("?search=%23" << name), :title => "#{pc}% of hosts"),
          :class => "tag"
        )

        case fsize
        when 0,1
          tag_links.push(tag_link)
        when 2,3
          tag_links.unshift(tag_link)
        end

      end
      tag_links.join(" ").html_safe
    end

	def hosts_page_link(name, args, workspace=@workspace)
		hosts_per_page = ( args[:hosts_per_page] || @hosts_per_page ).to_i
		path_args = { :workspace_id => workspace.id, :page => args[:page], :n => hosts_per_page }
		path_args[:search] = params[:search] if params[:search]
		path_args[:sort_by] = params[:sort_by] if params[:sort_by]
		path_args[:sort_dir] = params[:sort_dir] if params[:sort_dir]
		html_options = args[:html_options] || {}
	 	link_to name, hosts_path(path_args), html_options
	end

	def host_status_text(host)
		@stats ||= host.workspace.stats
		case host.status(@stats)
		when 4
			"looted"
		when 3
			"shelled"
		when 2
			"cracked"
		else
			"scanned"
		end
	end

	# Generate the markup for the host's row checkbox.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_checkbox_tag(host)
		check_box_tag("host_ids[]", host.id, false, :id => nil).to_json.html_safe
	end

	# Generate the markup for the host IP address.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_address_html(host)
		link_to(host.address, host_path(host)).to_json.html_safe
	end

	# Generate the markup for the host name.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_name_html(host)
		json_data_scrub(h(host.name)).to_json.html_safe
	end

	# Generate the markup for the host OS icons.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_os_html(host)
		os_string = (host.os || "Unknown") + " " + host.os_sp.to_s
		"<img src='#{os_to_icon(host.os)}' class='os_icon'>#{h(json_data_scrub(os_string))}".to_json.html_safe
	end

	# Generate the markup for the host OS version.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_os_virtual_html(host)
	
		h_data = "&nbsp;"
		if host.is_vm_guest?
			hdata = "<img src='/assets/icons/os/vm_logo.png' title='#{h(host.virtual_host)}'>"
		end
		json_data_scrub(hdata).to_json.html_safe
	end

	# Generate the markup for the host purpose.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the markup html, escaped for json.
	def host_purpose_html(host)
		json_data_scrub(truncate(h(host.purpose.to_s))).to_json.html_safe
	end

	# Generate the markup for the host services.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_services_html(host)
		blank_if_zero(host.service_count.to_i).to_json.html_safe
	end

	# Generate the markup for the host vulns.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_vulns_html(host)
		blank_if_zero(host.vuln_count.to_i).to_json.html_safe
	end

	# Generate the markup for the host tags or notes, depending upon the license.
	#
	# host       - The Mdm::Host to fetch the attribute from.
	# tag_search - The Boolean indicating whether or not a tag search is being performed.
	#
	# Returns the String markup html, escaped for json.
	def host_tags_or_notes_html(host, tag_search)
		tag_count = tag_search ? host.tags.size : host.tags_count
		if License.get.supports_tags?
			if tag_count.to_i > 0
				markup = tag_html(host.tags.first)
				if tag_count.to_i > 1
					markup += icon('list', :class => 'tags-icon')
					tags = ""
					host.tags.each do |tag|
						tags += tag_html(tag)
					end
					markup += content_tag(:div, :class => 'tags-hover') do
						tags.html_safe + image_tag('triangle.png', :class => 'tags-hover-triangle').html_safe
					end
				end
				markup = content_tag :div, :class => 'tags-wrap' do # wrap in a div
					markup
				end
				markup.to_json.html_safe
			else
				return "".to_json.html_safe
			end
		else
			blank_if_zero(host.note_count.to_i).to_json.html_safe
		end
	end

	# Generate the markup for the host tag, including hidden fields needed
	# for the front-end JavaScript.
	#
	# tag - The Mdm::Tag to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def tag_html(tag, workspace=@workspace)
		content_tag(:div, :class => 'tag') do
			link_to(h(tag.name), hosts_path(workspace) << ("?search=%23" << tag.name)) +
			content_tag(:span, :class => 'tag-id invisible') { tag.id.to_s } +
			content_tag(:span, :class => 'tag-name invisible') { tag.name }
		end
	end

	# Generate the markup for the host's update_at attribute.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_updated_at_html(host)
		content_tag(:span, :class => (host.flagged_count.to_i > 0 && host.recently_updated?) ? "badge" : "") do
			host.updated_at ? "#{time_ago_in_words(host.updated_at)} ago" : "never"
		end.to_json.html_safe
	end

	# Generate the markup for the host's update_at attribute.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_status_html(host)
		text = host_status_text(host)
		content_tag(:div, text.capitalize, :class => "host_status_#{text}").to_json.html_safe
	end


	# Generate the markup for the host's exploit attempt count attribute.
	#
	# host - The Mdm::Host to fetch the attribute from.
	#
	# Returns the String markup html, escaped for json.
	def host_attempts_html(host)
		blank_if_zero(host.exploit_attempt_count.to_i).to_s.to_json.html_safe
	end

	# Display a properly formatted Mdm::Vuln name.
	#
	# vuln - The Mdm::Vuln to fetch the attribute from.
	#
	# Returns the String markup.
	def vuln_name(vuln)
		if vuln.from_nexpose?
			ref = vuln.name.sub(Mdm::Vuln::NEXPOSE_REF_ID_PATTERN, '')
			link_text = "Nexpose ( #{ref} )"
			"<a href='http://www.rapid7.com/vulndb/lookup/#{ref.downcase}' target='_blank'>#{link_text}</a>"
		elsif vuln.exploit?
			link_to_exploit(vuln.name)
		else
			h vuln.name
		end.html_safe
	end
end

def exploit_link(name=nil,host=nil,service=nil)
    	link = new_module_run_path(@workspace, h(name))
     	if host && host.address
      	link << "?target_host=#{host.address}"
     	end
     	if service && service.port
      	link << "&target_port=#{service.port}"
     	end
     	return link
  end
 
  def link_to_exploit(name=nil,host=nil,service=nil)
 		link_to h(name), exploit_link(name,host,service)
  end
