class BruteforceTask < TaskConfig
  include ActiveModel::Validations
  include BruteforceTask::Addresses
  include BruteforceTask::Credentials
  include BruteforceTask::Limits
  include BruteforceTask::Payload
  include BruteforceTask::Scope
  include BruteforceTask::Services
  include BruteforceTask::Speed
  include BruteforceTask::SSH

  #
  # Methods
  #

  def allows_replay?
    true
  end

  def config_to_hash
    conf = {
      'DS_BLACKLIST_HOSTS'  => blacklist.join(' '),
      'DS_BRUTEFORCE_GETSESSION' => brute_sessions?,
      'DS_BRUTEFORCE_QUICKMODE_CREDS' => quickmode_creds,
      'DS_BRUTEFORCE_RECOMBINE_CREDS' => recombine_creds?,
      'DS_BRUTEFORCE_SCOPE' => fix_scope(scope),
      'DS_BRUTEFORCE_SERVICES' => brute_services.join(' '),
      'DS_BRUTEFORCE_SKIP_BLANK_PASSWORDS' => skip_blank_passwords?,
      'DS_BRUTEFORCE_SKIP_BUILTIN_UNIX_ACCOUNTS' => skip_builtin_unix_accounts?,
      'DS_BRUTEFORCE_SKIP_BUILTIN_WINDOWS_ACCOUNTS' => skip_builtin_windows_accounts?,
      'DS_BRUTEFORCE_SKIP_MACHINE_NAMES' => skip_machine_names?,
      'DS_BRUTEFORCE_SPEED' => speed,
      'DS_CRED_FILE_IDS' => cred_file_ids.join(' '),
      'DS_DB_NAMES' => db_names,
      'DS_DRY_RUN' => dry_run?,
      'DS_DynamicExe' => dynamic_exe?,
      'DS_INCLUDE_KNOWN' => include_known?,
      'DS_MAXGUESSESOVERALL' => max_guesses_overall,
      'DS_MAXGUESSESPERSERVICE' => max_guesses_per_service,
      'DS_MAXGUESSESPERUSER' => max_guesses_per_user,
      'DS_MAXMINUTESOVERALL' => max_minutes_overall,
      'DS_MAXMINUTESPERSERVICE' => max_minutes_per_service,
      'DS_MSSQL_WINDOWS_AUTH' => mssql_windows_auth?,
      'DS_PAYLOAD_METHOD' => connection.downcase,
      'DS_PAYLOAD_PORTS'  => payload_ports,
      'DS_PAYLOAD_TYPE'   => payload_type.downcase,
      'DS_PRESERVE_DOMAINS'  => preserve_domains?,
      'DS_SMB_DOMAINS'  => smb_domains,
      'DS_SSH_AUTH_PASSWORD' => ssh_auth_password, # Not exposed yet
      'DS_SSH_AUTH_PRIVKEY' => ssh_auth_privkey,   # Not exposed yet
      'DS_SSH_AUTH_PUBKEY' => ssh_auth_pubkey,      # Not exposed yet
      'DS_STOP_ON_SUCCESS' => stop_on_success?,
      'DS_VERBOSE'  => verbose?,
      'DS_WHITELIST_HOSTS'  => whitelist.join(" "),
      'username'            => username,
      'workspace'           => workspace.name
    }

    if License.get.supports_credential_mutation?
      mutate_opts = {
        'DS_MUTATE_IMPORTED' => mutate_imported?,
        'DS_MUTATE_KNOWN' => mutate_known?,
        'DS_MUTATE_QUICK' => mutate_quick?,
        'DS_MUTATION_OPT_APPEND_SINGLE_DIGIT' => mutation_opt_append_single_digit?,
        'DS_MUTATION_OPT_APPEND_SINGLE_PUNC' => mutation_opt_append_single_punc?,
        'DS_MUTATION_OPT_L33T_PASSWORDS' => mutation_opt_l33t_passwords?,
        'DS_MUTATION_OPT_PREPEND_SINGLE_DIGIT' => mutation_opt_prepend_single_digit?,
        'DS_MUTATION_OPT_PREPEND_SINGLE_PUNC' => mutation_opt_prepend_single_punc?,
        'DS_MUTATION_OPT_SUBSTITUTE_SINGLE_DIGITS' => mutation_opt_substitute_single_digits?
      }

      conf = conf.merge! mutate_opts
    end

    if current_profile.settings['payload_prefer_https']
      conf['DS_EnableReverseHTTPS'] = true
    end

    if current_profile.settings['payload_prefer_http']
      conf['DS_EnableReverseHTTP'] = true
    end

    if macro_name.present?
      conf['DS_AutoRunScript'] = "post/multi/pro/macro MACRO_NAME='#{macro_name}'"
    end

    if payload_lhost
      conf['DS_PAYLOAD_LHOST'] = payload_lhost
    end

    conf
  end

  def cred_file_ids
    @cred_file_ids ||= []
  end

  def cred_file_ids=(cred_file_ids)
    @cred_file_ids = select_valid_cred_file_ids(cred_file_ids || [])
  end

  boolean_attr_accessor :dry_run, :default => false

  def initialize(attributes={})
    super(attributes)

    attribute_names = [
        :blacklist,
        :blacklist_string,
        :brute_sessions,
        :brute_services,
        :connection,
        :cred_file_ids,
        :db_names,
        :dry_run,
        :dynamic_exe,
        :include_known,
        :macro_name,
        :max_guesses_overall,
        :max_guesses_per_service,
        :max_guesses_per_user,
        :max_minutes_overall,
        :max_minutes_per_service,
        :mssql_windows_auth,
        :mutate_imported,
        :mutate_known,
        :mutate_quick,
        :mutation_opt_append_single_digit,
        :mutation_opt_append_single_punc,
        :mutation_opt_l33t_passwords,
        :mutation_opt_prepend_single_digit,
        :mutation_opt_prepend_single_punc,
        :mutation_opt_substitute_single_digits,
        :payload_ports,
        :payload_type,
        :preserve_domains,
        :quickmode_creds,
        :recombine_creds,
        :scope,
        :ssh_auth_password,
        :ssh_auth_privkey,
        :ssh_auth_pubkey,
        :skip_blank_passwords,
        :skip_builtin_unix_accounts,
        :skip_builtin_windows_accounts,
        :skip_machine_names,
        :smb_domains,
        :speed,
        :stop_on_success,
        :verbose,
        :whitelist,
        :whitelist_string
    ]

    attribute_names.each do |attribute_name|
      setter = "#{attribute_name}="
      value = attributes[attribute_name]

      send(setter, value)
    end
  end

  def rpc_call
    client.start_bruteforce(config_to_hash)
  end

  boolean_attr_accessor :verbose, :default => false

  private

  # Strip off the "only " tags for known and defaults. If this
  # gets any more complicated, just use an index mapping of
  # SCOPES to pick the right label -> scope.
  def fix_scope(str)
    if str =~ /([^\s]+) only$/
      $1
    elsif str.nil?
      DEFAULT_SCOPE
    else
      str
    end
  end

  def select_valid_cred_file_ids(cred_file_ids)
    valid_cred_file_ids = []

    CredFile.find_all_by_id(cred_file_ids).each do |cred_file|
      if File.readable?(cred_file.path)
        valid_cred_file_ids << cred_file.id
      end
    end

    valid_cred_file_ids
  end
end

