class Campaign < ActiveRecord::Base
  extend MetasploitDataModels::SerializedPrefs
  include Campaign::Status

  #
  # CONSTANTS
  #

  BOOLEAN_SERIALIZED_PREFS = [
      :do_email,
      :do_exe_gen,
      :do_web,
      :smtp_ssl,
      :web_ssl
  ]

  #
  # Associations - sorted by association name
  #

  has_one :attachment, :dependent => :delete
  has_many :clients, :dependent => :delete_all, :class_name => 'Mdm::Client'
  has_many :email_addresses, :dependent => :delete_all
  has_one :email_template, :dependent => :delete
  has_one :web_template, :dependent => :delete
  belongs_to :workspace, :class_name => 'Mdm::Workspace'

  #
  # Callbacks - sorted by callback order
  #

  after_initialize :setup_or_update
  before_save :strip_attrs
  before_save :fix_bools
  before_destroy :stop_task

  #
  # Mass Assignment Security
  #

  attr_accessible :display_from
  attr_accessible :do_email
  attr_accessible :do_exe_gen
  attr_accessible :do_web
  attr_accessible :exe_lport
  attr_accessible :exe_name
  attr_accessible :listener_lhost
  attr_accessible :macro_name
  attr_accessible :mailfrom
  attr_accessible :name
  attr_accessible :payload_lhost
  attr_accessible :payload_type
  attr_accessible :smtp_port
  attr_accessible :smtp_server
  attr_accessible :smtp_ssl
  attr_accessible :smtp_user
  attr_accessible :web_srvhost
  attr_accessible :web_srvport
  attr_accessible :web_ssl
  attr_accessible :web_urihost
  attr_accessible :web_uripath
  attr_accessible :smtp_pass

  #
  # Nested Attributes
  #

  accepts_nested_attributes_for :email_addresses,
                                :reject_if => lambda { |a| a[:address].blank? },
                                :allow_destroy => true

  #
  # Scopes
  #

  scope :with_emails_by_id, lambda{ |the_id|
    where(:id => the_id).includes(:email_addresses)
  }

  #
  # Serializations
  #

  serialize :prefs, MetasploitDataModels::Base64Serializer.new

  #
  # Serialized Prefs
  #

    # General settings
  serialized_prefs_attr_accessor :payload_lhost, :listener_lhost, :payload_type

  # Email settings
  serialized_prefs_attr_accessor :do_email # bool
  serialized_prefs_attr_accessor :smtp_server, :smtp_port, :smtp_ssl
  serialized_prefs_attr_accessor :smtp_user, :smtp_pass
  serialized_prefs_attr_accessor :mailfrom, :display_from

  # Web settings
  serialized_prefs_attr_accessor :do_web # bool
  serialized_prefs_attr_accessor :web_uripath, :web_urihost, :web_srvport, :web_srvhost
  serialized_prefs_attr_accessor :web_ssl # bool

  # Executable settings
  serialized_prefs_attr_accessor :do_exe_gen # bool
  serialized_prefs_attr_accessor :exe_lport
  serialized_prefs_attr_accessor :exe_name
  serialized_prefs_attr_accessor :macro_name

  # State
  serialized_prefs_attr_accessor :error

  #
  # Validations
  #

  validates_presence_of :name
  validates_uniqueness_of :name, :scope => :workspace_id

  validates_each :web_uripath do |record, attr, value|
    if attr == :web_uripath
      record.errors.add(attr, "must start with '/'") unless value =~ /^\//
    end
  end

  validates_each :payload_lhost, :listener_lhost do |record, attr, value|
    record.errors.add(attr, "must be an IP address") unless value.blank? or value =~ /^(:?\d+\.){3}\d+/
  end

  validates_each :smtp_port, :web_srvport, :exe_lport do |record, attr, value|
    unless value.to_i > 0 and value.to_i < 65536
      record.errors.add(attr, "must be a number between 1 and 65535")
    end
  end

  def client_stats
    stats = {}

    self.clients.count(:all,
                       :group => "clients.ua_name",
                       :order => "count_all DESC",
                       :conditions => ['clients.ua_name is not NULL'],
                       :limit => 5
    ).each_pair do |k,v|
      stats[k.upcase] = v
    end

    stats
  end

  # FIXME: Workaround for #5450. Note we don't actually do anything with this,
  # but we should. 
  attr_accessor :file

  def os_stats
    stats = {}
    # XXX: Not going to work, refactor
=begin
    clients.where("clients.ua_name is not NULL")
    .includes(:host)
    .group("hosts.os_name")
    .limit(5)
    .count
    .each_pair do |k,v|
      stats[k.upcase] = v
    end
=end
    clients.where("clients.ua_name is not NULL")
    .includes(:host)
    .map{|x| x.host.os_name }
    .each { |n|
      stats[n] ||= 0
      stats[n]  += 1
    }

    top5 = {}
    stats.keys.sort.sort{|a,b| stats[b] <=> stats[a] }[0,5].each do |n|
      top5[n.upcase] = stats[n]
    end

    top5
  end

  def running?
    [ Statuses[:started], Statuses[:email_sent] ].include? status
  end

  def stop_task
    if prefs[:task]
      task = Task.find(prefs[:task]) rescue nil
      if task
        task.stop
      end
    end
  end

  def web_link
    return "" if web_template.nil?

    proto = "http#{(web_ssl) ? "s" : ""}"

    if web_srvhost.nil? or web_srvhost.empty?
      server = Rex::Socket.source_address
    else
      server = "#{web_srvhost}"
    end

    if not web_urihost.nil? and not web_urihost.empty?
      server = "#{web_urihost}"
    end

    if web_ssl
      server << ":#{web_srvport}" if (web_srvport != "443")
    else
      server << ":#{web_srvport}" if (web_srvport != "80")
    end
    path = web_uripath
    path = path[1,path.length] if path[0,1] == "/"

    "#{proto}://#{server}/#{path}"
  end

  protected

  def fix_bools
    BOOLEAN_SERIALIZED_PREFS.each do |boolean|
      current_value = self.send(boolean)
      fixed_value = true

      if current_value == '0' or current_value.blank?
        fixed_value = false
      end

      setter_name = "#{boolean}="
      self.send(setter_name, fixed_value)
    end

    true
  end

  def setup_or_update
    unless self.new_record?
      # This is an existing record, update our status to handle the case
      # where our task was stopped or prosvc died without updating it.
      update_status!
      return
    end

    # This is a new record, set defaults appropriately

    self.prefs ||= {}
    self.prefs[:tasks] ||= {}

    self.payload_lhost = Rex::Socket.source_address unless self.payload_lhost
    self.listener_lhost = "0.0.0.0" unless self.listener_lhost
    self.payload_type = "reverse_tcp" unless self.payload_type

    # Don't set a default smtp server.  Gmail is finicky and will refuse to
    # send our emails if they contain an exe attachment or trip their
    # maliciousness-detector.  See #3117
    #self.smtp_server = "smtp.gmail.com" unless self.smtp_server
    #self.smtp_user = "@gmail.com" unless self.smtp_user

    # Make these strings so they match values from forms
    self.smtp_port = "465" unless self.smtp_port
    self.web_srvport = "80" unless self.web_srvport
    self.web_uripath = "/" unless self.web_uripath

    self.exe_lport = "4444" unless self.exe_lport

    # Enable the EXE generator by default
    self.exe_name ||= "ClickMe.exe"

    fix_bools
  end

  def strip_attrs
    self.smtp_server.strip! if self.smtp_server
    self.smtp_user.strip! if self.smtp_user
  end
end

