# Explicit require for engine/prosvc.rb
#
# Gems
#
require 'state_machine'

module SocialEngineering
  class Campaign < ActiveRecord::Base
    # Server config interfaces provide getters and setters for
    # virtual (serialized) attrs like host, port, SSL, etc.
    include SocialEngineering::SmtpServerConfigInterface
    include SocialEngineering::WebServerConfigInterface

    class NotExecutable < RuntimeError; end

    self.table_name = :se_campaigns

    #
    # CONSTANTS
    #
    
    CAMPAIGN_CONFIG_TYPES = ['wizard', 'custom']
    CAMPAIGN_CONFIG_TYPES_FORMATTED = [['Phishing Campaign', 'wizard'], ['Custom Campaign', 'custom']]
    # Default port to server web pages for the {SocialEngineering::Campaign}
    DEFAULT_WEB_PORT = 8080
    # Default port used by Browser Auto-Pwn (BAP)
    DEFAULT_WEB_BAP_PORT = 8081
    VALID_STATES = %w(unconfigured launchable preparing running cancelled finished interrupted)

    #
    # Associations - sorted by name
    #

    has_many :emails, :class_name => 'SocialEngineering::Email', :dependent => :destroy
    has_many :portable_files, :class_name => 'SocialEngineering::PortableFile', :dependent => :destroy
    belongs_to :started_by_user, :class_name => 'Mdm::User'
    belongs_to :user, :class_name => 'Mdm::User'
    has_many :web_pages, :class_name => 'SocialEngineering::WebPage', :dependent => :destroy
    belongs_to :workspace, :class_name => 'Mdm::Workspace', :touch => true

    #
    # Callbacks - in call order
    #

    before_validation :ports_to_i

    #
    # Mass Assignment Security
    #

    # User is allowed to set config_type when selecting between the wizard and custom campaign
    attr_accessible :config_type
    # User is NOT allowed to set timestamps: created_at, started_at, and updated_at
    # User is allowed to set name so they can name the campaign
    attr_accessible :name
    # User is allowed to modify notification settings to configure a notification email
    attr_accessible :notification_email_address
    attr_accessible :notification_email_message
    attr_accessible :notification_email_subject
    attr_accessible :notification_enabled
    # FIXME remove port column as ports are now in prefs as smtp_port, web_bap_port, and web_port
    # User is NOT allowed to set prefs as it must set using the serialized pref accessor
    # User is NOT allowed to set started_by_user_id as they could fake the paper trail then
    # User is NOT allowed to set state as it is managed by state machine.
    # User is NOT allowed to set user_id as it is derived from the current user
    # User is NOT allowed to set workspace_id as it is derived from the current workspace
    
    #
    # Scopes
    #

    scope :running, where(:state => 'running')

    #
    # Serializations
    #

    serialize :prefs # TODO: JSON backend

    #
    # State Machines
    #

    state_machine :state, :initial => :unconfigured do
      #
      # Ordered by event name
      # States in transitions ordered by name
      #

      event :cancel do
        transition :preparing => :cancelled
      end

      event :finish do
        transition [:running, :preparing] => :finished
      end

      event :interrupt do
        transition [:running, :preparing] => :interrupted
      end

      event :launch do
        transition [:finished, :interrupted, :launchable] => :preparing
      end

      event :run do
        transition :preparing => :running
      end

      event :set_launchable do
        transition :unconfigured => :launchable
      end

      event :unconfigure do
        transition [:cancelled, :finished, :interrupted, :launchable] => :unconfigured
      end
    end

    #
    # Validations - sorted by attribute name
    #

    validates :config_type,
              :inclusion => {
                  :in => CAMPAIGN_CONFIG_TYPES
              }
    validates :name,
              :presence => true,
              :uniqueness => {
                  :scope => :workspace_id
              },
              :length => {
                  :maximum => 100
              }
    validates :state,
              :presence => true,
              :inclusion => {
                  :in => VALID_STATES
              }
    validates :user, :presence => true
    validates :workspace, :presence => true
    validate  :validate_notification_email

    #
    # Methods - ordered by name
    #

    # Returns an array of all components, sorted by creation_date
    #
    # @return [Array<ActiveRecord::Base>]
    def all_components
      # refactor this into sql!!
      (self.emails + self.web_pages + self.portable_files).sort_by(&:created_at)
    end

    def bap_campaign?
      self.web_pages.each do |page|
        return true if page.attack_type == 'bap'
      end
      return false
    end

    def configure!
      if configurable?
        set_launchable if unconfigured?
      else
        unconfigure if !unconfigured?
      end
    end

    def email_campaign?
      emails.present?
    end

    def executable?
      !unconfigured?
    end

    def execute!(current_user)
      if executable?
        if is_running?
          CampaignRunner.stop(self)
        else
          CampaignRunner.start(self, current_user)
        end
      else
        raise NotExecutable
      end
    end

    def send_notification_email
      return unless self.notification_enabled
      message = MailMessage.new
      notification_email_recipients.each { |addr| message.add_recipient addr }
      message.subject = self.notification_email_subject
      message.from_address = 'notifications@pro.metasploit.com'
      message.body = self.notification_email_message
      ActionMailerSender.new.send(message)
    end

    def startable?
      executable? and (!is_running?)
    end

    def usb_campaign?
      portable_files.present?
    end

    def uses_wizard?
      config_type == 'wizard'
    end

    def visits_with_targets
      visits = emails.collect{|e| e.visits.includes(:human_target)}
      visits.flatten
    end

    def web_campaign?
      web_pages.present?
    end

    def web_page_for_path(path)
      potential_pages = web_pages.where(:path => path)
      potential_pages.present? ? potential_pages.first : nil
    end

    # @return [":8080"] by default
    # @return [""] if web_port is 80
    def web_port_with_colon
      if self.web_port == 80
        ''
      else
        ":#{self.web_port}"
      end
    end

    def is_running?
      preparing? || running?
    end

    private

    def configurable?
      ConfigurableCampaignChecker.configured?(self)
    end

    def notification_email_recipients
      if self.notification_enabled and self.notification_email_address.present?
        self.notification_email_address.split(',').map(&:strip)
      else
        []
      end
    end

    def ports_to_i
      # If these ports are about to be saved, AND they are numeric,
      # coerce them to integers.
      # Rails's numericality will not attempt to cast them to their
      # db type (which should be 'integer') since they are serialized attrs,
      # so we check that they are integers and cast them here
      integer_regex = /^\d+$/

      if self.web_bap_port.present? && self.web_bap_port =~ integer_regex
        self.web_bap_port = self.web_bap_port.to_i
      end

      if self.web_port.present? && self.web_port =~ integer_regex
        self.web_port = self.web_port.to_i
      end

      if self.smtp_port.present? && self.smtp_port =~ integer_regex
        self.smtp_port = self.smtp_port.to_i
      end
    end

    def validate_notification_email
      return unless self.notification_enabled
      # validate message is present
      if notification_email_message.blank?
        errors.add("Notification message", 'cannot be blank')
      end
      # validate recipients present and correctly formatted
      recipients = notification_email_recipients
      if recipients.empty?
        errors.add("Notification ", 'must contain at least one email address')
      else
        # validate all email addresses
        email_pat = /^[\S]+@[\S]+\.[\S]+$/
        failed = recipients.select { |r| r.strip.match(email_pat).nil? }
        unless failed.empty?
          failed.each do |failed|
            errors.add("Email", "'#{failed}' is an invalid address")
          end
        end
      end
    end
  end
end
