class SocialEngineering::Email < ActiveRecord::Base
  include LiquidTemplating::Powers
  include SocialEngineering::EmailAttackConfigInterface

  self.table_name = :se_emails

  #
  # CONSTANTS
  #

  ATTACK_TYPES = ['none', 'file']
  # we favor the rich text editor as the default UI choice
  EDITOR_TYPES = ['rich_text', 'plain_text', 'preview']
  ORIGIN_TYPES = ['custom', 'template']
  PHISHING_DEFAULTS = {
    :name => 'E-mail',
    :subject => 'URGENT: Reset your intranet password',
    :from_address => 'security@yourcompany.com',
    :from_name => 'IT Security',
    :content => <<-eos
<p>{{first_name}},</p>
<p>We have increased our password requirements for security purposes. 
Please use the link below within 24 hours to maintain access to your account.</p>
<p><a href="{% campaign_href 'Landing Page' %}">Click here</a></p>
<p>Thank you,</p>
<p>The IT Support Staff</p>
<p><small>This email was intended for {{first_name}} {{last_name}}.</small></p>
eos
  }
  VALID_FILE_GENERATION_TYPES = ['exe_agent', 'file_format']

  #
  # Associations - sorted by association name
  #

  belongs_to :campaign, :class_name => 'SocialEngineering::Campaign', :touch => true
  has_many :email_openings, :class_name => 'SocialEngineering::EmailOpening'
  has_many :email_sends, :class_name => 'SocialEngineering::EmailSend'
  has_many :files, :as => :attachable, :class_name => 'SocialEngineering::CampaignFile'
  belongs_to :target_list, :class_name => 'SocialEngineering::TargetList'
  belongs_to :template, :class_name => 'SocialEngineering::EmailTemplate', :foreign_key => :email_template_id
  belongs_to :user, :class_name => 'Mdm::User'
  has_many :visits, :class_name => 'SocialEngineering::Visit'

  #
  # Callbacks
  #

  before_save :autoset_origin_type

  #
  # Liquid
  #

  liquid_template :content
  liquid_drops SocialEngineering::HumanTarget, :first_name, :last_name, :email_address

  #
  # Mass Assignment Security
  #

  # User is allowed to set attack_type so they can choose between 'none' and 'file'
  attr_accessible :attack_type
  # User is NOT allowed to set campaign_id since it should be derived from the path.
  # User is allowed to set content of email.
  attr_accessible :content
  # User is NOT allowed to set timestamp fields, created_at or updated_at.
  # User is allowed to set editor_type so they can toggle back and forth between rich-text and plain-text.
  attr_accessible :editor_type
  # User is allowed to set email_template_id so they can choose the template for this email.
  attr_accessible :email_template_id
  # User is allowed to set from_address and from_name to fake From for email.
  attr_accessible :from_address
  attr_accessible :from_name
  # Don't care what you set insert to be, as its not a DB field
  attr_accessible :insert
  # Don't care what you set phishing_redirect to be, as it is not a DB field
  attr_accessible :phishing_redirect
  # User is NOT allowed to set msg_sent_count since it is used to track the number of emails sent using this email.
  # User is allowed to set name so that the user can refer to the email by name
  attr_accessible :name
  # User is allowed to set an origin_type, as this affects the immediate UI render when creating
  #  a new campaign. On save, it will simply be overwritten by autoset_origin_type.
  attr_accessible :origin_type
  # User is NOT allowed to set prefs as it a complex serialization that should only be set through its subattributes
  # User is NOT allowed to set sent_at as set by the pro/modules/auxiliary/pro/social_engineering/send_email.rb#notify_send_complete.
  # FIXME status is an obsolete column and should be removed.
  # User is allowed to set subject for the email.
  attr_accessible :subject
  # User is allowed to set target_list_id to select the targets to which to send the email
  attr_accessible :target_list_id
  # FIXME template_id is an obsolete column and should be removed.
  # User is NOT allowed to set user_id since it should be inferred from the current user.

  #
  # Serializations
  #

  serialize :prefs # TODO: use JSON to serialize after upgrading to Rails 3.2.5

  #
  # Validations
  #

  validates :attack_type, :inclusion => { :in => ATTACK_TYPES }
  validates :campaign, :presence => true
  validates :content, :presence => true
  validates :name, :presence => true
  validates :origin_type, :inclusion => { :in => ORIGIN_TYPES }
  validates :target_list, :presence => true

  #
  # Scopes
  #

  scope :retrieve_for_process,
    lambda{ |email_id|
      where(:id => email_id).includes(:target_list => :human_targets)
    }

  #
  # Custom accessors. Used as placeholders for the UI, since formastic requires
  #   an attribute to build a field.
  #
  attr_accessor :insert
  attr_accessor :phishing_redirect

  def custom_attributes
    unless @custom_attributes.present?
      @custom_attributes = SocialEngineering::Email.liquid_drops_select_list_pairs
      if other_web_pages.present? && !campaign.uses_wizard?
        @custom_attributes << ['Link to Web Page', 'campaign_link']
      end
      if campaign && campaign.uses_wizard?
        @custom_attributes << ['Link to Landing Page', 'campaign_landing_link']
      end
    end
    @custom_attributes
  end

  def other_web_pages
    @other_web_pages ||= SocialEngineering::WebPage.find_all_by_campaign_id(self.campaign.id)
  end

  # Render an email with template, using human_target as the "drop
  # object" that carries data for placeholders in Email#content
  def render_body_for_send(human_target, tracking=true)
    if uses_template? # pass human_target & campaign back up to template
      template.render_with_email_content liquid_render_with_drop_object(human_target, tracking), human_target
    else
      liquid_render_with_drop_object(human_target, tracking)
    end
  end

  def target_list_name
    if target_list.present?
      target_list.name
    else
      ''
    end
  end

  def target_list_size
    if target_list.present?
      target_list.human_targets.count
    else
      0
    end
  end

  def to_hash
    {
      name: name,
      type: 'email',
      attack_type: attack_type.humanize,
      status: 'N/A',
      target_list_name: target_list_name,
      id: id
    }
  end

  def uses_template?
    origin_type == 'template'
  end

  private

  def autoset_origin_type
    self.origin_type = if template.present? then 'template' else 'custom' end
  end
end
