# Explict requires for engine/prosvc.rb
require 'state_machine'

class TaskChain < ActiveRecord::Base
  #
  # Associations - sorted by assocition name
  #

  has_many :scheduled_tasks, :dependent => :destroy, :order => 'position'
  belongs_to :user, :class_name => 'Mdm::User'
  belongs_to :workspace, :class_name => 'Mdm::Workspace'

  #
  # Mass Assignment Security
  #

  # User is allowed to set clear_workspace_before_run since it's part of the configuration
  attr_accessible :clear_workspace_before_run
  # User is NOT allowed to set created_at or updated_at because they are timestamps for the model
  # User is NOT allowed to set hidden because that is up to the controller to decide
  # User is allowed to set the name of a TaskChain because its how the user refers to the TaskChain
  attr_accessible :name
  # User is NOT allowed to set schedule because it needs to be in ice_cube format, so it is derived for the user by the
  #   controller
  # User is NOT allowed to set last_run_at or next_run_at since it's a book-keeping field for when the run the tasks
  # User is NOT allowed to set state since it is maintained by StateMachine
  # User is NOT allowed to set user_id since it would allow users to steal over users' TaskChains
  # User is NOT allowed to set workspace_id since it should be derived from the current workspace


  #
  # Serializations
  #

  serialize :schedule

  #
  # Scopes
  #

  scope :for_workspace, lambda { |workspace| where(:workspace_id => workspace.id)}
  scope :visible, lambda { where(:hidden => false) }

  #
  # State Machines
  #

  state_machine :state, :initial => :ready do
    event :start do
      transition :ready => :running
    end

    event :finish do
      transition :running => :ready
    end
  end

  #
  # Validations
  #

  validates :name, :presence => true, :uniqueness => {:scope => :workspace_id}
  validates :user, :presence => true
  validates :workspace, :presence => true

  # Flag to tell the task runner to wipe out existing
  # Workspace data before starting first task in chain.
  def clear_workspace_before_run?
    clear_workspace_before_run.present?
  end

  def print_scheduled_kinds
    str = ''
    scheduled_tasks.each_with_index do |task, i|
      str << ', ' if i > 0
      str << task.kind.humanize
    end
    str
  end

  def scheduled_to_run?
    self.next_run_at.present?
  end

  def schedule_to_run_at(time)
    return if time.nil?
    self.next_run_at = time
  end
end
