# Creates hosts, services and loot for load testing.
#
# With no options, a single host is created with a small number of
# services. Each host also has several sample loot files attached,
# including screenshot images and text files.
#
# Options:
#  --host-count: Number of hosts to create (default 1)
#  --service-count: Number of services to add per host (default random 0-20)
#  --loot-count: Number of loots to add per host (default random 0-5)
#  --workspace-id: ID of workspace to create hosts in (default makes new one)
#
# Assumptions: one User exists. Since /tmp is currently used, assumes
# *nix/OS X (TODO Use MS tmp?)
#
# TODO: Error handling (IPs might conflict); consider adding sessions,
# services, vulndetail

require 'base64'
require 'fileutils'
require 'optparse'


def create_host(workspace_id)
  address = "%d.%d.%d.%d" % [rand(256), rand(256), rand(256), rand(256)]
  mac = "00:0c:29:8d:ec:99"
  name = "HostName-#{rand(100000)}"
  state = "Alive"
  os_list = ["Linux", "Windows", "Mac OS X"]
  os_name = os_list[rand(3)]
  flavors = { "Linux" => ["Debian", "Ubuntu", "CentOS"],
              "Windows" => ["NT", "7", "2008"],
              "Mac OS X" => ["", "", ""]
            }
  os_flavor = flavors[os_name][rand(3)]
  os_lang = "English"
  purpose = "Server"

  #puts "Creating: #{name}, os: #{os_name}, os flav: #{os_flavor}, addy: #{address}, mac: #{mac}"
  host = Mdm::Host.new(:address => address,
                       :mac => mac,
                       :name => name,
                       :state => state,
                       :os_name => os_name,
                       :os_flavor => os_flavor,
                       :os_lang => os_lang,
                       :workspace_id => workspace_id,
                       :purpose => purpose
                      )
  host.save
  host.id
end


def create_services(host_id, service_count)
  services = [ "21|ftp|tcp",
               "22|ssh|tcp",
               "23|telnet|tcp",
               "25|smtp|tcp",
               "53|dns|tcp",
               "53|dns|udp",
               "80|http|tcp",
               "110|pop|tcp",
               "111|portmapper|udp",
               "111|portmapper|tcp",
               "123|ntp|udp",
               "135|dcerpc|tcp",
               "137|netbios|udp",
               "139|smb|tcp",
               "389|ldap|tcp",
               "443|https|tcp",
               "445|smb|tcp",
               "512|rexec|tcp",
               "513|login|tcp",
               "514|shell|tcp"]
  states = ['open', 'closed', 'filtered', 'unknown']

  service_count.times do |s|
    serv_arr = services[rand(20)].split('|')
    #puts "Creating service, port: #{serv_arr[0]}; proto: #{serv_arr[2]}; state: #{states[rand(4)]}; name: #{serv_arr[1]}"
    
    service = Mdm::Service.new(:host_id => host_id,
                               :port => serv_arr[0],
                               :proto => serv_arr[2],
                               :state => states[rand(4)],
                               :name => serv_arr[1])
    service.save
  end
end

def group_file_data
  %Q|root:x:0:
  daemon:x:1:
  bin:x:2:
  sys:x:3:
  adm:x:4:msfadmin
  tty:x:5:
  disk:x:6:
  lp:x:7:
  mail:x:8:
  news:x:9:
  uucp:x:10:
  man:x:12:
  proxy:x:13:
  kmem:x:15:
  dialout:x:20:msfadmin
  fax:x:21:
  voice:x:22:
  cdrom:x:24:msfadmin
  floppy:x:25:msfadmin
  tape:x:26:
  sudo:x:27:
  audio:x:29:msfadmin
  dip:x:30:msfadmin
  www-data:x:33:
  backup:x:34:
  operator:x:37:
  list:x:38:
  irc:x:39:
  src:x:40:
  gnats:x:41:
  shadow:x:42:
  utmp:x:43:telnetd
  video:x:44:msfadmin
  sasl:x:45:
  plugdev:x:46:msfadmin
  staff:x:50:
  games:x:60:
  users:x:100:
  nogroup:x:65534:
  libuuid:x:101:
  dhcp:x:102:
  syslog:x:103:
  klog:x:104:
  scanner:x:105:
  nvram:x:106:
  fuse:x:107:msfadmin
  crontab:x:108:
  mlocate:x:109:
  ssh:x:110:
  msfadmin:x:1000:
  lpadmin:x:111:msfadmin
  admin:x:112:msfadmin
  bind:x:113:
  ssl-cert:x:114:postgres
  postfix:x:115:
  postdrop:x:116:
  postgres:x:117:
  mysql:x:118:
  sambashare:x:119:msfadmin
  user:x:1001:
  service:x:1002:
  telnetd:x:120:|
end

def passwd_file_data
  %Q|root:x:0:0:root:/root:/bin/bash
  daemon:x:1:1:daemon:/usr/sbin:/bin/sh
  bin:x:2:2:bin:/bin:/bin/sh
  sys:x:3:3:sys:/dev:/bin/sh
  sync:x:4:65534:sync:/bin:/bin/sync
  games:x:5:60:games:/usr/games:/bin/sh
  man:x:6:12:man:/var/cache/man:/bin/sh
  lp:x:7:7:lp:/var/spool/lpd:/bin/sh
  mail:x:8:8:mail:/var/mail:/bin/sh
  news:x:9:9:news:/var/spool/news:/bin/sh
  uucp:x:10:10:uucp:/var/spool/uucp:/bin/sh
  proxy:x:13:13:proxy:/bin:/bin/sh
  www-data:x:33:33:www-data:/var/www:/bin/sh
  libuuid:x:100:101::/var/lib/libuuid:/bin/sh
  dhcp:x:101:102::/nonexistent:/bin/false
  syslog:x:102:103::/home/syslog:/bin/false
  klog:x:103:104::/home/klog:/bin/false
  sshd:x:104:65534::/var/run/sshd:/usr/sbin/nologin
  msfadmin:x:1000:1000:msfadmin,,,:/home/msfadmin:/bin/bash
  bind:x:105:113::/var/cache/bind:/bin/false
  postfix:x:106:115::/var/spool/postfix:/bin/false
  ftp:x:107:65534::/home/ftp:/bin/false
  postgres:x:108:117:PostgreSQL
  administrator,,,:/var/lib/postgresql:/bin/bash
  mysql:x:109:118:MySQL
  Server,,,:/var/lib/mysql:/bin/false
  tomcat55:x:110:65534::/usr/share/tomcat5.5:/bin/false
  distccd:x:111:65534::/:/bin/false
  service:x:1002:1002:,,,:/home/service:/bin/bash
  telnetd:x:112:120::/nonexistent:/bin/false
  proftpd:x:113:65534::/var/run/proftpd:/bin/false
  statd:x:114:65534::/var/lib/nfs:/bin/false|
end

def save_text_file(path,data)
  unless File.exists? path
    file = File.new(path, 'w+')
    file.write(data)
    file.close
  end
end

def copy_image_file(orig_file, path)
  unless File.exists? path
    FileUtils.cp(orig_file, path)
  end
end

# Loot files are removed from FS when host is deleted.
# Reports and web UI uses path to load file, not data. Including that as
# well in case somewhere actually uses it.
# Note that the file path the files are copied to is the same for all
# hosts. This still works in the UI and has the added benefit of not
# wasting FS space!
def add_loot(loot_count, host_id, workspace_id)
  loots = ["host_screenshot|image/png|An Image|image.png",
           "host.unix.group|text/plain|/etc/group|group",
           "host.unix.passwd|text/plain|/etc/passwd|passwd"]

  # For each loot to add, pick a random type:
  loot_count.times do |l|
    loot_arr = loots[rand(3)].split('|')
    ltype, content_type = loot_arr[0], loot_arr[1]
    info, name = loot_arr[2], loot_arr[3]

    case ltype
    when 'host_screenshot'
      images = {:small => 'small_image.png',
                :large => 'large_image.png'}
      name = images[images.keys[rand(images.size)]]
      orig_file = File.dirname(__FILE__) + '/' + name
      # Leaving out for now to save DB space; not needed?
      # data = Base64.encode64(File.open(orig_file).read)
      path = '/tmp/' + name
      copy_image_file(orig_file, path)
    when /host.unix\.*/
      # Leaving out for now to save DB space; not needed?
      data = group_file_data
      case ltype 
      when /.*group/
        path = '/tmp/group_file.txt'
      when /.*passwd/
        path = '/tmp/passwd_file.txt'
      end
      save_text_file(path,data)
    end

    #puts  "Creating loot, ltype: #{ltype}; path: #{path}; content_type: #{content_type}; info: #{info}; name: #{name}"
    loot = Mdm::Loot.new(:workspace_id => workspace_id,
                         :host_id => host_id,
                         :ltype => ltype,
                         # :data => data,
                         :path => path,
                         :content_type => content_type,
                         :name => name,
                         :info => info)
    loot.save
  end
end

def add_ref
  ref = Mdm::Ref.create(:name => 'URL-http://www.metasploit.com')
  ref.id
end

# TODO: Real vulns are usually connected to services,
# can have attempts.
def add_vulns(vuln_count, host_id, workspace_id)
  host = Mdm::Host.where("workspace_id = #{workspace_id} and id = #{host_id}")
  vuln_count.times do |v|
    vuln = Mdm::Vuln.create(:host_id => host_id, :name => "Stochastic M-brane instability", :info => "Your flux capacitor is subject to a time traveller in the middle attack.")
    #
    # TODO Mdm::VulnDetail
    # Random reference:
    ref_id = Mdm::Ref.count > 0 ? Mdm::Ref.first.id : add_ref
    Mdm::VulnRef.create(:ref_id => ref_id, :vuln_id => vuln.id)
  end
end

def get_opts
  options = {}
  optparse = OptionParser.new do |opt|
    opt.on("--host-count=COUNT", "How many hosts to create") do |host_count|
      options[:host_count] = host_count
    end
    opt.on("--service-count=COUNT", "How many services to add on each host") do |service_count|
      options[:service_count] = service_count
    end
    opt.on("--loot-count=COUNT", "How many loot files to add to each host") do |loot_count|
      options[:loot_count] = loot_count
    end
    opt.on("--vuln-count=COUNT", "How many vulnerabilities to add to each host") do |vuln_count|
      options[:vuln_count] = vuln_count
    end
    opt.on("--workspace-id=ID", "ID of workspace to use (default new is created)") do |workspace_id|
      options[:workspace_id] = workspace_id
    end
  end
  optparse.parse!
  options
end

def generate_dataset
  opts = get_opts
  start_time = Time.now
  host_count = opts[:host_count] ? opts[:host_count].to_i : 1

  unless opts[:workspace_id]
    ws = Mdm::Workspace.find_or_create_by_name("LoadTesting", :owner_id => Mdm::User.first.id)
    workspace_id = ws.id
    puts "Using workspace: #{Mdm::Workspace.find(workspace_id).name}(#{workspace_id})"
  else
    workspace_id = opts[:workspace_id].to_i
  end
  workspace_name = Mdm::Workspace.find(workspace_id).name

  puts "Creating #{host_count} hosts in workspace #{workspace_name}...\n"
  host_count.times do |h|
    service_count = opts[:service_count] ? opts[:service_count].to_i : rand(20)
    loot_count = opts[:loot_count] ? opts[:loot_count].to_i : rand(5)
    vuln_count = opts[:vuln_count] ? opts[:vuln_count].to_i : rand(10)

    host_id = create_host(workspace_id)
    create_services(host_id, service_count)
    add_loot(loot_count, host_id, workspace_id)
    add_vulns(vuln_count, host_id, workspace_id)
  end
  duration = Time.now - start_time
  puts "#{host_count} hosts created with services and loot in #{duration}s. View workspace #{workspace_name} (ID: #{workspace_id})!"
end

generate_dataset
