# DataTableQueryResponse: a formatted response for DataTable AJAX
# sources using a scoped collection of a model
#
# Usage: in your controller method,
#
# def evil_people_ajax_source
#   response = DataTableQueryResponse.new(UserFinder.new, params)
#   render :json => response.to_json
# end
#
#
# Where UserFinder responds to:
#     #datatable_columns, #datatable_search_columns,
#     #find, and (optionally) #datatable_virtual_columns


require 'csv'

class DataTableQueryResponse < Struct.new(:finder, :params)

  def columns
    finder.datatable_columns
  end

  def search_columns
    finder.datatable_search_columns
  end

  # use this method to describe columns that don't belong to the
  # actual table we are query (e.g. columns renamed with an AS)
  def virtual_columns
    begin
      finder.datatable_virtual_columns
    rescue NoMethodError
      []
    end
  end


  def current_page
    start = params[:iDisplayStart].to_i
    start / per_page + 1
  end

  def per_page
    display_len = params[:iDisplayLength].to_i
    if display_len > 0
      display_len
    else
      20
    end
  end

  # return and save the results of the finder
  def collection
    @collection ||= finder.find
  end

  # returns an of conditions checking every field
  # e.g. ["name ILIKE ? OR email ILIKE ?", '%joe%', '%joe%']
  def conditions
    return "" unless params[:sSearch].present?
    statements = search_columns.collect { |col| "#{col} ILIKE ?" }
    values = Array.new(statements.size) { "%#{params[:sSearch]}%" }
    [statements.join(' OR '), *values]
  end

  # returns 'ASC' or 'DESC'
  def order_direction
    dir = (params[:sSortDir_0] || 'ASC').upcase
    return 'ASC' unless ['ASC', 'DESC'].include? dir
    dir
  end

  # returns e.g. "email_address ASC"
  def order_string
    return "" unless params[:iSortCol_0].present?
    col_num = params[:iSortCol_0].to_i
    col_name = columns.at(col_num) || columns.first
    klass = begin
      collection.klass 
    rescue NoMethodError
      collection
    end
    col_attrs = klass.columns_hash[col_name.to_s]
    if col_attrs && col_attrs.type == :string
      # (LOWER() is required for case-insensitive compares in psql)
      "LOWER(#{klass.table_name}.#{col_name}) #{order_direction}"
    else
      # if we have specified that this is a renamed column, DONT include table name
      if virtual_columns.include? col_name
        "#{col_name} #{order_direction}"
      else
        if col_attrs.present?
          # column matches a column in the current table
          "#{klass.table_name}.#{col_name} #{order_direction}"
        else
          # no matching columns in the current table
          "#{col_name} #{order_direction}"
        end
      end
    end
  end

  # size of the entire collection we're paginating
  def total_count
    collection.length
  end

  # size of the *filtered* collection we're paginating
  def scoped_count
    collection.where(conditions).length
  end

  # return the scoped AR query
  def find
    collection.where(conditions)
              .order(order_string)
              .page(current_page)
              .per(per_page)
  end

  # format the JSON data output correctly
  def as_json(options={})
    {
      :iTotalRecords => total_count,
      :iTotalDisplayRecords => scoped_count,
      :sEcho => params[:sEcho].to_i,
      :sColumns => columns.join(','),
      :aaData => find.as_json(:only => columns)
    }
  end

  def to_csv
    CSV.generate do |csv|
       csv << columns
       collection.each do |result|
         csv << columns.collect { |col| result[col] }
       end
    end
  end
end
