module Mdm::Host::Decorator
  extend ActiveSupport::Concern

  #
  # CONSTANTS
  #

  # Defines how recent "recent" is
  RECENTLY_UPDATED = 8.hours

  def os
    str = os_name
    if str
      str = str + " (#{os_flavor})" if (os_flavor and not os_flavor.empty?)
    else
      str = os_flavor
    end
    (str.nil? or str.empty?) ? 'Unknown OS' : str
  end

  def title
    name ? "#{address} - #{name}" : address
  end

  def is_vm_guest?
    !self.virtual_host.blank?
  end

  def tag_frequency
    freq = {}
    tags.each do |t|
      freq[t.to_s] = Mdm::Tag.find_by_name(t.to_s).hosts.size
    end
    freq
  end

  def tag_names
    tags.map {|t| t.name}.join(", ")
  end

  def tags_count
    tags.uniq.size
  end

  def tag_count
    tags_count
  end

  # add_tag_by_name accepts a tag_name and either finds a previous Tag object
  # or creates a new one, and assigns it to the current host
  def add_tag_by_name(tag_name)
    tag = nil
    if tag_name.to_i != 0 # if name is an int, look tag up by id (not sure why we do this)
      tag ||= Mdm::Tag.find(tag_name.to_i)
    end
    # otherwise grab tag from database
    tag ||= Mdm::Tag.where("tags.name = ? AND hosts.workspace_id = ?", tag_name, workspace.id).includes(:hosts).first
    # otherwise create a new tag
    unless tag.present?
      tag = Mdm::Tag.new(:name => tag_name) 
      tag.save
    end
    add_tag tag
  end

  # add_tag examines a host's existing tags and, 
  #   if necessary, adds the current tag
  def add_tag(tag)
    tags << tag if tag.present? && tag.valid? && !tag_names.include?(tag.name)
    tag
  end

  def flagged?
    notes.flagged.size > 0
  end

  def recently_updated?
    updated_at >= RECENTLY_UPDATED.ago
  end

  def clear_flags
    notes.flagged.each { |n| n.seen = true; n.save! }
  end

  def to_s
    address
  end

  def name_and_address
    if name.nil? || name.empty?
      address
    else
      "%s (%s)" % [name, address]
    end
  end

  def address_int
    if address
      Rex::Socket.addr_atoi(address)
    else
      nil
    end
  end

  def status(workspace_stats=workspace.stats)
    return 4 if workspace_stats[:loot_hosts][address]
    return 3 if workspace_stats[:sess_hosts][address]
    return 2 if workspace_stats[:pass_hosts][address]
    return 1
  end

  def screenshot
    loots.find_last_by_ltype("host.windows.screenshot")
  end

  def smb_shares
    notes.find_all_by_ntype("smb.shares")
  end

  def nfs_exports
    notes.find_all_by_ntype("nfs.exports")
  end

  def all_vulns
    ( vulns + web_sites.map{|w| w.web_vulns} ).flatten
  end

  def web_vulns
    web_sites.map{|w| w.web_vulns}.flatten
  end

  def attribute_locked?(attr)
    n = notes.find_by_ntype("host.updated.#{attr}")
    n && n.data[:locked]
  end

  attr_accessor :lock_attrs
end

