# contains the logic for inputting a hash of a few different
#   task configurations, and rolling them up into a QuickPentest
#   model, and then into a TaskChain for scheduling.
# in retrospect, I should have made QuickPentest subclass TaskChain,
#   and built a more reusable TaskChain builder.
class QuickPentestBuilder
  # we need some of the TaskController process_task_* helpers
  include TasksSharedControllerMethods

  attr_accessor :params
  attr_accessor :quick_pentest
  attr_accessor :current_user

  def initialize(params, user)
    self.params = params
    self.current_user = user
    self.quick_pentest = QuickPentest.new(params[:quick_pentest] || {})
  end

  # returns an initialized QuickPentest instance
  def build
    # build all of the inner models
    build_workspace_from_params
    build_scan_task_from_params
    build_exploit_task_from_params
    build_report_task_from_params
    # return our new model
    quick_pentest
  end

  # returns an initialized TaskChain, ready fo' savin
  def to_task_chain
    build_task_chain
  end

  # set general options for the user that never drills down
  #   into the "Advanced" fieldsets.
  def set_defaults!
    quick_pentest.collect_evidence = true
    quick_pentest.scan_type = QuickPentest::QUICK_PENTEST_SCAN_TYPES_FORMATTED[0][1]
    quick_pentest.report_enabled = true
    quick_pentest.exploit_enabled = true
    quick_pentest.workspace = Mdm::Workspace.new({
      :boundary => default_range
    })
    quick_pentest.scan_task = ::ScanTask.new({
      :workspace => quick_pentest.workspace,
      :address_string => quick_pentest.workspace.boundary
    })
    quick_pentest.import_task = ImportTask.new({
      :workspace => quick_pentest.workspace
    })
    quick_pentest.exploit_task = ExploitTask.new({
      :workspace => quick_pentest.workspace
    })
    quick_pentest.report_task = ReportTask.new({
      :workspace => quick_pentest.workspace,
      :username => current_user.username,
      :whitelist_string => quick_pentest.workspace.boundary
    })
    quick_pentest.nexpose_task = NexposeTask.new({
      :workspace => quick_pentest.workspace
    })
    self
  end

  private
  # WETly copied from WorkspacesController:20
  # returns local intranet/8 range
  def default_range
    default_range = ""
    mine = Rex::Socket.source_address("50.50.50.50")
    if mine != "127.0.0.1"
      default_range = mine.sub(/\.\d+$/, ".1-254")
    end
    default_range
  end

  def build_workspace_from_params
    params[:workspace] ||= {}
    quick_pentest.workspace = Mdm::Workspace.new(params[:workspace].merge({
      :owner => current_user  
    }))
  end

  def build_scan_task_from_params
    params[:scan_task] ||= {}

    case safe_pentest_scan_type
    when :regular_scan
      quick_pentest.scan_task = ::ScanTask.new(params[:scan_task].merge({
        :address_string => quick_pentest.workspace.boundary || '',
        :workspace => quick_pentest.workspace,
        :username => current_user.username
      }))
    when :nexpose_scan
      params[:nexpose_task] ||= {}
      quick_pentest.nexpose_task = NexposeTask.new(params[:nexpose_task].merge({
        :whitelist_string => quick_pentest.workspace.boundary || '',
        :blacklist_string => params[:scan_task][:blacklist_string] || '',
        :workspace => quick_pentest.workspace,
        :username => current_user.username
      }))
    when :import
      params[:import_task] ||= {}
      quick_pentest.import_task = ImportTask.new(params[:import_task].merge({
        :workspace => quick_pentest.workspace,
        :blacklist_string => params[:scan_task][:blacklist_string] || '',
        :username => current_user.username,
        :no_files => params[:no_files],
        :path => write_file_param_to_quickfile(params[:import_task][:file])
      }))
    end
  end

  def build_exploit_task_from_params
    params[:exploit_task] ||= {}
    quick_pentest.exploit_task = ExploitTask.new(params[:exploit_task].merge({
      :workspace => quick_pentest.workspace,
      :whitelist_string => quick_pentest.workspace.boundary,
      :username => current_user.username
    }))
  end

  def build_report_task_from_params
    if quick_pentest.report_enabled
      params[:report_task] ||= {}
      # these are some gross hacks to get the process_task_config(:report, ...)
      #   method to work on this model.
      @workspace = quick_pentest.workspace
      @users = {current_user.username => 0} # so we don't gather_users
      @workspace_created_at = Time.now.utc  # so we don't gather_workspaces
      quick_pentest.report_task = process_task_config(:report, params[:report_task].merge({
        :workspace => quick_pentest.workspace,
        :username => current_user.username,
        :whitelist_string => quick_pentest.workspace.boundary,
        :skip_host_existence_check => true,
        :dont_populate_mod_refs => true
      }))
    end
  end

  # todo: this should probably be punted into a reusable class,
  #   e.g. TaskChainBuilder
  def build_task_chain
    # for the current QuickPentest, set up a new TaskChain
    task_chain = TaskChain.new(
      :name => generate_task_chain_name
    )
    task_chain.user = current_user
    task_chain.workspace = quick_pentest.workspace
    # add the Scan or Import task.
    case safe_pentest_scan_type
    when :regular_scan
      task = task_chain.scheduled_tasks.build(
        :config_hash => quick_pentest.scan_task.config_to_hash,
      )
      task.kind = 'scan'
    when :nexpose_scan
      task = task_chain.scheduled_tasks.build(
        :config_hash => quick_pentest.nexpose_task.config_to_hash
      )
      task.kind = 'nexpose'
    when :import
      task = task_chain.scheduled_tasks.build(
        :config_hash => quick_pentest.import_task.config_to_hash,
      )
      task.kind = 'import'
    end
    # add the Exploit task :)
    if quick_pentest.exploit_enabled
      task = task_chain.scheduled_tasks.build(
        :config_hash => quick_pentest.exploit_task.config_to_hash,
      )
      task.kind = 'exploit'
      # add the collect evidence task, if necessary
      # todo: refactor into a build_collect_evidence.
      #   there aren't any validatable opts.
      if quick_pentest.collect_evidence
        task = task_chain.scheduled_tasks.build(
          :config_hash => CollectEvidenceTask.new(
            :run_on_all_sessions => true,
            :workspace => quick_pentest.workspace
          ).config_to_hash
        )
        task.kind = 'collect_evidence'
      end
    end
    # add the Report task, if enabled
    if quick_pentest.report_enabled
      task = task_chain.scheduled_tasks.build(
        :config_hash => quick_pentest.report_task.config_to_hash,
      )
      task.kind = 'report'
    end
    # add the position attribute to each task
    task_chain.scheduled_tasks.each_with_index { |task, idx| task.position = idx+1 }
    # we dont want the TaskChain to show up in our list of Chains
    task_chain.hidden = true
    # return our initialized TaskChain, ready for saving
    task_chain.schedule_to_run_at(Time.now.utc)
    task_chain
  end

  def generate_task_chain_name
    # making the TC name unique with the workspace name will sorta work.
    # need a more unique solution (why doesnt postgres autoincrement ID?)
    "Quick Pentest: #{quick_pentest.workspace.name}"
  end

  def safe_pentest_scan_type
    # helper for checking a nil
    if quick_pentest.scan_type.present?
      quick_pentest.scan_type.to_sym
    else
      # default to :regular_scan
      QuickPentest::QUICK_PENTEST_SCAN_TYPES_FORMATTED[0][1].to_sym
    end
  end

  # mostly yanked from tasks_shared_controller_methods#process_import
  # todo: move to a better, shared place
  def write_file_param_to_quickfile(file_param)
    temp = nil
    path = nil
    if file_param.present?
      # Write the uploaded file to a tempfile
      temp = ::Rex::Quickfile.new('import')
      uploaded_io = file_param
      begin
        while (buff = uploaded_io.read(1024*64))
          temp.write(buff)
        end
      rescue ::EOFError
      end
      temp.flush
    else
      temp = nil
    end
    temp ? temp.path : nil
  end
end