class QuickPentestValidator < Struct.new(:quick_pentest)
  # Running actual scans require an address range. Importing does not.
  SCAN_TYPES_THAT_REQUIRE_ADDRESSES = %w(regular_scan nexpose_scan)

  # some metaprogramming niceness to make it easier to validate for a given step
  def step_is_valid?(step)
    self.send("#{step.to_s}_valid?")
  end

  # pass in an array of steps and it will check them all.
  # useful for running all validations at the end
  def steps_are_valid?(steps)
    steps.inject(true) {|snowball, step| snowball &= step_is_valid?(step) }
  end

  #
  #  Validators: one per tab of the wizard
  #
  def create_project_valid?
    # When doing regular_scan or nexpose_scan, we HAVE to ensure
    #   that the IP boundary is set on workspace
    is_valid = quick_pentest.workspace.valid?
    scan_type = quick_pentest.scan_type.to_s
    if QuickPentestValidator::SCAN_TYPES_THAT_REQUIRE_ADDRESSES.include? scan_type
      if quick_pentest.workspace.boundary.blank?
        quick_pentest.workspace.errors.add(:boundary, 'must be defined')
        is_valid = false
      end
    end
    is_valid
  end

  def configure_scan_valid?
    case safe_pentest_scan_type
    when :regular_scan
      quick_pentest.scan_task.valid?
    when :nexpose_scan
      quick_pentest.nexpose_task.valid?
    when :import
      quick_pentest.import_task.valid?
    end
  end

  def configure_exploits_valid?
    quick_pentest.exploit_task.valid?
  end

  def generate_report_valid?
    if quick_pentest.report_enabled
      quick_pentest.report_task.valid?
    else 
      true 
    end
  end

  # returns a JSON object containing inner models and names of errors
  def errors_hash
    qp = quick_pentest
    {
      :workspace => qp.workspace.present? ? quick_pentest.workspace.errors.messages : {},
      :scan_task => qp.scan_task.present? ? qp.scan_task.specific_error : {},
      :nexpose_task => qp.nexpose_task.present? ? qp.nexpose_task.specific_error : {},
      :import => qp.import_task.present? ? qp.import_task.specific_error : {},
      :report_task => qp.report_task.present? ? qp.report_task.specific_error : {},
      :exploit_task => qp.exploit_task.present? ? qp.exploit_task.specific_error : {}
    }
  end

  def errors_hash_for_step(step)
    case step.to_sym
    when :create_project
      errors_hash.select { |k, v| k == :workspace }
    when :configure_scan
      errors_hash.select { |k, v| [:scan_task, :nexpose_task, :import].include? k }
    when :configure_report
      errors_hash.select { |k, v| k == :report_task }
    end
  end

  private
  def safe_pentest_scan_type
    # helper for checking a nil
    if quick_pentest.scan_type.present?
      quick_pentest.scan_type.to_sym
    else
      # default to :regular_scan
      QuickPentest::QUICK_PENTEST_SCAN_TYPES_FORMATTED[0][1].to_sym
    end
  end

end