// This [jQuery](http://jquery.com/) plugin implements an `<iframe>`
// [transport](http://api.jquery.com/extending-ajax/#Transports) so that
// `$.ajax()` calls support the uploading of files using standard HTML file
// input fields. This is done by switching the exchange from `XMLHttpRequest`
// to a hidden `iframe` element containing a form that is submitted.

// The [source for the plugin](http://github.com/cmlenz/jquery-iframe-transport)
// is available on [Github](http://github.com/) and dual licensed under the MIT
// or GPL Version 2 licenses.

// ## Usage

// To use this plugin, you simply add an `iframe` option with the value `true`
// to the Ajax settings an `$.ajax()` call, and specify the file fields to
// include in the submssion using the `files` option, which can be a selector,
// jQuery object, or a list of DOM elements containing one or more
// `<input type="file">` elements:

//     $("#myform").submit(function() {
//         $.ajax(this.action, {
//             files: $(":file", this),
//             iframe: true
//         }).complete(function(data) {
//             console.log(data);
//         });
//     });

// The plugin will construct hidden `<iframe>` and `<form>` elements, add the
// file field(s) to that form, submit the form, and process the response.

// If you want to include other form fields in the form submission, include
// them in the `data` option, and set the `processData` option to `false`:

//     $("#myform").submit(function() {
//         $.ajax(this.action, {
//             data: $(":text", this).serializeArray(),
//             files: $(":file", this),
//             iframe: true,
//             processData: false
//         }).complete(function(data) {
//             console.log(data);
//         });
//     });

// ### Response Data Types

// As the transport does not have access to the HTTP headers of the server
// response, it is not as simple to make use of the automatic content type
// detection provided by jQuery as with regular XHR. If you can't set the
// expected response data type (for example because it may vary depending on
// the outcome of processing by the server), you will need to employ a
// workaround on the server side: Send back an HTML document containing just a
// `<textarea>` element with a `data-type` attribute that specifies the MIME
// type, and put the actual payload in the textarea:

//     <textarea data-type="application/json">
//       {"ok": true, "message": "Thanks so much"}
//     </textarea>

// The iframe transport plugin will detect this and pass the value of the
// `data-type` attribute on to jQuery as if it was the "Content-Type" response
// header, thereby enabling the same kind of conversions that jQuery applies
// to regular responses. For the example above you should get a Javascript
// object as the `data` parameter of the `complete` callback, with the
// properties `ok: true` and `message: "Thanks so much"`.

// ### Handling Server Errors

// Another problem with using an `iframe` for file uploads is that it is
// impossible for the javascript code to determine the HTTP status code of the
// servers response. Effectively, all of the calls you make will look like they
// are getting successful responses, and thus invoke the `done()` or
// `complete()` callbacks. You can only determine communicate problems using
// the content of the response payload. For example, consider using a JSON
// response such as the following to indicate a problem with an uploaded file:

//     <textarea data-type="application/json">
//       {"ok": false, "message": "Please only upload reasonably sized files."}
//     </textarea>

// ### Compatibility

// This plugin has primarily been tested on Safari 5 (or later), Firefox 4 (or
// later), and Internet Explorer (all the way back to version 6). While I
// haven't found any issues with it so far, I'm fairly sure it still doesn't
// work around all the quirks in all different browsers. But the code is still
// pretty simple overall, so you should be able to fix it and contribute a
// patch :)

// ## Annotated Source

(function ($, undefined) {
  "use strict";

  // Register a prefilter that checks whether the `iframe` option is set, and
  // switches to the "iframe" data type if it is `true`.
  $.ajaxPrefilter(function (options, origOptions, jqXHR) {
    if (options.iframe) {
      return "iframe";
    }
  });

  // Register a transport for the "iframe" data type. It will only activate
  // when the "files" option has been set to a non-empty list of enabled file
  // inputs.
  $.ajaxTransport("iframe", function (options, origOptions, jqXHR) {
    var form = null,
        iframe = null,
        name = "iframe-" + $.now(),
        files = $(options.files).filter(":file:enabled"),
        markers = null;

    // This function gets called after a successful submission or an abortion
    // and should revert all changes made to the page to enable the
    // submission via this transport.
    function cleanUp() {
      markers.replaceWith(function (idx) {
        return files.get(idx);
      });
      form.remove();
      iframe.attr("src", "javascript:false;").remove();
    }

    // Remove "iframe" from the data types list so that further processing is
    // based on the content type returned by the server, without attempting an
    // (unsupported) conversion from "iframe" to the actual type.
    options.dataTypes.shift();

    form = $("<form enctype='multipart/form-data' method='post'></form>")
      .hide().attr({action: options.url, target: name});

    // If there is any additional data specified via the `data` option,
    // we add it as hidden fields to the form. This (currently) requires
    // the `processData` option to be set to false so that the data doesn't
    // get serialized to a string.
    if (typeof(options.data) === "string" && options.data.length > 0) {
      $.error("data must not be serialized");
    } else {
      $.each($.param(options.data || {}).split('&'), function (index, el) {
        el = el.split('=');
        $("<input type='hidden' />").attr({
          name: decodeURIComponent(el[0])
        , value: decodeURIComponent(el[1].replace(/\+/g, '%20'))
        }).appendTo(form);
      });
    }


    // Add a hidden `X-Requested-With` field with the value `IFrame` to the
    // field, to help server-side code to determine that the upload happened
    // through this transport.
    $("<input type='hidden' value='IFrame' name='X-Requested-With' />")
      .appendTo(form);

    // Move the file fields into the hidden form, but first remember their
    // original locations in the document by replacing them with disabled
    // clones. This should also avoid introducing unwanted changes to the
    // page layout during submission.
    markers = files.after(function (idx) {
      return $(this).clone().attr("disabled", true);
    }).next();
    files.appendTo(form);

    return {

      // The `send` function is called by jQuery when the request should be
      // sent.
      send: function (headers, completeCallback) {
        iframe = $("<iframe src='javascript:false;' name='" + name +
          "' id='" + name + "' style='display:none'></iframe>");

        // The first load event gets fired after the iframe has been injected
        // into the DOM, and is used to prepare the actual submission.
        iframe.bind("load", function () {

          // The second load event gets fired when the response to the form
          // submission is received. The implementation detects whether the
          // actual payload is embedded in a `<textarea>` element, and
          // prepares the required conversions to be made in that case.
          iframe.unbind("load").bind("load", function () {
            var doc = this.contentWindow ? this.contentWindow.document :
              (this.contentDocument ? this.contentDocument : this.document),
              root = doc.body,
              textarea = root.getElementsByTagName("textarea")[0],
              type = textarea ? textarea.getAttribute("data-type") : null,
              status = textarea ? textarea.getAttribute("data-status") : 200,
              statusText = textarea ? textarea.getAttribute("data-statusText") : "OK",
              content = {
                html: root.innerHTML,
                text: type ?
                  textarea.value :
                  $('<div/>').html(root.innerHTML).html()
              };

            cleanUp();
            completeCallback(status, statusText, content, type ?
              ("Content-Type: " + type) :
              null);
          });

          // Now that the load handler has been set up, submit the form.
          form[0].submit();
        });

        // After everything has been set up correctly, the form and iframe
        // get injected into the DOM so that the submission can be
        // initiated.
        $("body").append(form, iframe);
      },

      // The `abort` function is called by jQuery when the request should be
      // aborted.
      abort: function () {
        if (iframe !== null) {
          iframe.unbind("load").attr("src", "javascript:false;");
          cleanUp();
        }
      }
    };
  });

}(jQuery));
(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  jQueryInWindow(function($) {
    return this.TabbedModalView = (function(_super) {

      __extends(TabbedModalView, _super);

      function TabbedModalView() {
        this.tabAt = __bind(this.tabAt, this);

        this.tabClicked = __bind(this.tabClicked, this);

        this.index = __bind(this.index, this);

        this.close = __bind(this.close, this);

        this.open = __bind(this.open, this);

        this.layout = __bind(this.layout, this);

        this.setMaxWidth = __bind(this.setMaxWidth, this);

        this.setTitle = __bind(this.setTitle, this);

        this.setDescription = __bind(this.setDescription, this);

        this.setButtons = __bind(this.setButtons, this);

        this.setTabs = __bind(this.setTabs, this);
        return TabbedModalView.__super__.constructor.apply(this, arguments);
      }

      TabbedModalView.prototype._tabIdx = -1;

      TabbedModalView.prototype._opened = false;

      TabbedModalView.prototype.initialize = function() {
        this.setMaxWidth(1000);
        return window.origConfirm || (window.origConfirm = window.confirm);
      };

      TabbedModalView.prototype.events = {
        'click a.close': 'close',
        'click ul.tabs>li': 'tabClicked'
      };

      TabbedModalView.prototype.setTabs = function(_tabs) {
        this._tabs = _tabs;
        return _.each(this._tabs, function(tab) {
          tab['class'] || (tab['class'] = '');
          return tab['checkbox'] || (tab['checkbox'] = false);
        });
      };

      TabbedModalView.prototype.setButtons = function(_btns) {
        this._btns = _btns;
        return _.each(this._btns, function(btn) {
          return btn['class'] || (btn['class'] = '');
        });
      };

      TabbedModalView.prototype.setDescription = function(description) {
        this.description = description;
      };

      TabbedModalView.prototype.setTitle = function(title) {
        this.title = title;
      };

      TabbedModalView.prototype.setMaxWidth = function(maxWidth) {
        this.maxWidth = maxWidth;
      };

      TabbedModalView.prototype.layout = function() {
        var modalWidth, screenWidth;
        if (this.$modal.width() > this.maxWidth) {
          this.$modal.width(this.maxWidth);
        }
        modalWidth = this.$modal.width();
        screenWidth = $(window).width();
        this.$modal.css('left', parseInt(($(window).width() - this.$modal.width()) / 2) + 'px');
        this.$modal.css('top', parseInt(($(window).height() - this.$modal.height()) / 2) + 'px');
        return $('ul.tabs>li:first-child', this.el).addClass('first-child');
      };

      TabbedModalView.prototype.template = _.template($('#tabbed-modal').html());

      TabbedModalView.prototype.open = function() {
        this.render();
        this._opened = true;
        this.escEnabled = true;
        $('body').css({
          overflow: 'hidden'
        }).height($(window).height());
        $(window).unbind('resize.tabbedModal');
        $(window).unbind('keyup.tabbedModal');
        $(window).bind('resize.tabbedModal', this.layout);
        $(window).bind('keyup.tabbedModal', this.escKeyHandler);
        return this.stubCustomConfirm();
      };

      TabbedModalView.escKeyHandler = function(e) {
        if (e.keyCode === 27 && TabbedModalView.escEnabled) {
          return TabbedModalView.close();
        }
      };

      TabbedModalView.prototype.close = function(e) {
        if (e && e.preventHandler) {
          e.preventHandler();
        }
        this.$el.html('');
        this._opened = false;
        this.escEnabled = false;
        $('body').css({
          overflow: 'auto'
        }).css({
          height: 'auto',
          overflow: 'inherit'
        });
        $(window).unbind('resize.tabbedModal');
        $(window).unbind('keyup.tabbedModal');
        window.confirm = window.origConfirm;
        return this.undelegateEvents();
      };

      TabbedModalView.prototype.stubCustomConfirm = function() {
        var _this = this;
        return window.confirm = function() {
          if (_this.opened) {
            _this.escEnabled = false;
            _.delay((function() {
              return _this.escEnabled = true;
            }), 300);
            return origConfirm.apply(window, arguments);
          } else {
            return origConfirm.apply(window, arguments);
          }
        };
      };

      TabbedModalView.prototype.index = function(idx) {
        var $li, $ul;
        if (typeof idx === 'undefined') {
          return this._tabIdx;
        }
        if (idx === this._tabIdx || idx < 0 || idx >= this._tabs.length) {
          return this._tabIdx;
        }
        $ul = $('ul.tabs', this.el);
        $li = $('li', $ul).removeClass('selected').eq(idx).addClass('selected');
        $('div.content', this.el).toggleClass('top-left-square', idx === 0);
        this._tabIdx = idx;
        return this._tabIdx;
      };

      TabbedModalView.prototype.tabClicked = function(e) {
        var $li;
        $li = $(e.currentTarget);
        return this.index($li.index());
      };

      TabbedModalView.prototype.tabAt = function(idx) {
        return $('ul.tabs li', this.$modal).eq(idx);
      };

      TabbedModalView.prototype.render = function() {
        TabbedModalView.__super__.render.apply(this, arguments);
        this.$el.html(this.template(this));
        this.$modal = $('.tabbed-modal', this.$el);
        this.layout();
        if (this._tabIdx < 0) {
          return this.index(0);
        }
      };

      TabbedModalView.prototype.tabs = function() {
        return $('ul.tabs>li', this.$modal);
      };

      TabbedModalView.prototype.content = function() {
        return $('.padding>.content', this.$modal);
      };

      return TabbedModalView;

    }).call(this, Backbone.View);
  });

}).call(this);
(function() {
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  jQueryInWindow(function($) {
    var QUICK_PENTEST_IS_FIRST_TASK_READY_URL, QUICK_PENTEST_REDIRECT_AFTER_SUCCESS_URL, QUICK_PENTEST_URL;
    QUICK_PENTEST_URL = '/quick_pentest';
    QUICK_PENTEST_IS_FIRST_TASK_READY_URL = '/quick_pentest/first_task_ready';
    QUICK_PENTEST_REDIRECT_AFTER_SUCCESS_URL = '/quick_pentest/redirect_after_success';
    return this.QuickPentestModal = (function(_super) {

      __extends(QuickPentestModal, _super);

      function QuickPentestModal() {
        this.formLoadedSuccessfully = __bind(this.formLoadedSuccessfully, this);

        this.pageAt = __bind(this.pageAt, this);

        this.handleErrors = __bind(this.handleErrors, this);

        this.someInputChanged = __bind(this.someInputChanged, this);

        this.resetErrorsOnTab = __bind(this.resetErrorsOnTab, this);

        this.resetErrors = __bind(this.resetErrors, this);

        this.startScanClicked = __bind(this.startScanClicked, this);

        this.reportTypeChanged = __bind(this.reportTypeChanged, this);

        this.scanTypeChanged = __bind(this.scanTypeChanged, this);

        this.toggleExcludeIPs = __bind(this.toggleExcludeIPs, this);

        this.toggleGenerateReport = __bind(this.toggleGenerateReport, this);

        this.toggleExploits = __bind(this.toggleExploits, this);

        this.toggleAdvanced = __bind(this.toggleAdvanced, this);
        return QuickPentestModal.__super__.constructor.apply(this, arguments);
      }

      QuickPentestModal.prototype.initialize = function() {
        QuickPentestModal.__super__.initialize.apply(this, arguments);
        this.setTitle('Quick Penetration Test');
        this.setDescription('Quickly set up a test that automatically gathers ' + 'information about the target network, launches attacks ' + 'against the targets, and builds a report of test findings.');
        this.setTabs([
          {
            name: 'Target Settings'
          }, {
            name: 'Configure Scan'
          }, {
            name: 'Run Exploits',
            checkbox: true
          }, {
            name: 'Generate Report',
            checkbox: true
          }
        ]);
        this.setButtons([
          {
            name: 'Cancel',
            "class": 'close'
          }, {
            name: 'Start Test',
            "class": 'btn primary'
          }
        ]);
        this.loadForm();
        return $("#modals").appendTo($('body'));
      };

      QuickPentestModal.prototype.showLoadingDialog = function() {
        this._loaderDialog = $('<div class="loading">').dialog({
          modal: true,
          closeOnEscape: false,
          title: 'Loading...'
        });
        this._loaderDialog.parents('.ui-dialog').addClass('white');
        return this._loaderDialog.append($('<p class="dialog-msg center">').text('Loading...'));
      };

      QuickPentestModal.prototype.hideLoadingDialog = function() {
        return this._loaderDialog.dialog('close').remove();
      };

      QuickPentestModal.prototype.setDialogTitle = function(title) {
        return $('.dialog-msg', this._loaderDialog).text(title);
      };

      QuickPentestModal.prototype.events = _.extend({
        'click a.advanced': 'toggleAdvanced',
        'click input#tab_run_exploits': 'toggleExploits',
        'click input#tab_generate_report': 'toggleGenerateReport',
        'click a.exclude_toggle': 'toggleExcludeIPs',
        'change input[name="quick_pentest[scan_type]"]': 'scanTypeChanged',
        'change select#quick_pentest_report_type': 'reportTypeChanged',
        'click .modal-actions .btn.primary': 'startScanClicked',
        'change select': 'someInputChanged',
        'change input, textarea': 'someInputChanged',
        'keydown input, textarea': 'someInputChanged'
      }, TabbedModalView.prototype.events);

      QuickPentestModal.prototype._steps = [];

      QuickPentestModal.prototype.loadForm = function() {
        _.each(this.tabs(), function(t) {
          return $(t).addClass('ui_disabled');
        });
        this.content().addClass('loading').html('');
        return $.ajax({
          url: QUICK_PENTEST_URL,
          method: 'get',
          success: this.formLoadedSuccessfully,
          error: function() {
            var _this = this;
            this.content().removeClass('loading').addClass('error').text("Unable to reach Metasploit Pro. Please ensure you" + " are connected and the Metasploit Pro service is running.");
            return _.delay((function() {
              return _this.content().is(':visible') && _this.loadForm();
            }), 5000);
          }
        });
      };

      QuickPentestModal.prototype.toggleAdvanced = function(e) {
        e.preventDefault();
        $(e.currentTarget).toggleClass('not_advanced');
        return $('~div.advanced', e.currentTarget).toggle();
      };

      QuickPentestModal.prototype.toggleExploits = function(e) {
        var checked;
        checked = $(e.currentTarget).is(':checked');
        $('h3.enabled>span', this.$modal).removeClass('disabled enabled');
        $('.configure_exploits h3.enabled>span', this.$modal).removeClass('disabled enabled');
        if (checked) {
          $('.configure_exploits h3.enabled>span', this.$modal).text("enabled").addClass('enabled');
          return $("[name='quick_pentest[exploit_enabled]']", this.$modal).attr('value', '1');
        } else {
          $('.configure_exploits h3.enabled>span', this.$modal).text("disabled").addClass('disabled');
          return $("[name='quick_pentest[exploit_enabled]']", this.$modal).removeAttr('value');
        }
      };

      QuickPentestModal.prototype.toggleGenerateReport = function(e) {
        var checked;
        checked = $(e.currentTarget).is(':checked');
        $('.generate_report h3.enabled>span', this.$modal).removeClass('disabled enabled');
        if (checked) {
          $('.generate_report h3.enabled>span', this.$modal).text("enabled").addClass('enabled');
          return $("[name='quick_pentest[report_enabled]']", this.$modal).attr('value', '1');
        } else {
          $('.generate_report h3.enabled>span', this.$modal).text("disabled").addClass('disabled');
          return $("[name='quick_pentest[report_enabled]']", this.$modal).removeAttr('value');
        }
      };

      QuickPentestModal.prototype.toggleExcludeIPs = function(e) {
        e.preventDefault();
        $('.excluded_addresses', this.$modal).show();
        $('.exclude_toggle_wrap', this.$modal).hide();
        return $('.create_project .advanced', this.$modal).css({
          top: '182px'
        });
      };

      QuickPentestModal.prototype.scanTypeChanged = function(e) {
        var toggle_divs;
        toggle_divs = ['toggle_regular_scan', 'toggle_nexpose_scan', 'toggle_import'];
        _.each(toggle_divs, function(div) {
          return $("." + div, this.$modal).hide();
        });
        return $(".toggle_" + ($(e.target).val()), this.$modal).show();
      };

      QuickPentestModal.prototype.reportTypeChanged = function(e) {
        var format;
        format = $('select#quick_pentest_report_type option:selected', this.$modal).val();
        $(".report_formats>div", this.$modal).hide().removeClass('active_report_option');
        $(".report_sections>div", this.$modal).hide().removeClass('active_report_option');
        $(".report_options>div", this.$modal).hide().removeClass('active_report_option');
        $(".report_formats #report_format_for_" + format, this.$modal).show().addClass('active_report_option');
        $(".report_sections #report_section_for_" + format, this.$modal).show().addClass('active_report_option');
        return $(".report_options #report_option_for_" + format, this.$modal).show().addClass('active_report_option');
      };

      QuickPentestModal.prototype.startScanClicked = function(e) {
        var $closeBtn, $form, $page, $primaryBtn,
          _this = this;
        e.preventDefault();
        this.resetErrors();
        $form = $('form', this.$modal).first();
        $('.hasErrors', this.$modal).hide();
        $page = $('div.content form .page').eq(this._tabIdx);
        this.showLoadingDialog();
        $primaryBtn = $('a.btn.primary', this.$modal).addClass('disabled');
        $closeBtn = $('.modal-actions a.close', this.$modal).addClass('ui-disabled');
        return this.submitFormAjax($form, {
          url: QUICK_PENTEST_URL,
          method: 'POST',
          error: function(e) {
            _this.hideLoadingDialog();
            $page.removeClass('page-loading');
            $primaryBtn.addClass('disabled');
            $closeBtn.removeClass('ui-disabled');
            return _this.handleErrors.apply(_this, arguments);
          },
          success: function(data) {
            var cancelMe, taskChainQuery;
            _this.setDialogTitle('Waiting for first task to start...');
            taskChainQuery = "?task_chain_id=" + data.task_chain_id;
            return cancelMe = window.setInterval((function() {
              return $.ajax({
                url: QUICK_PENTEST_IS_FIRST_TASK_READY_URL + taskChainQuery,
                dataType: 'json',
                success: function(data) {
                  if (data.success) {
                    window.clearInterval(cancelMe);
                    return window.location = QUICK_PENTEST_REDIRECT_AFTER_SUCCESS_URL + taskChainQuery;
                  }
                }
              });
            }), 3000);
          }
        });
      };

      QuickPentestModal.prototype.resetErrors = function(scope) {
        if (scope == null) {
          scope = this.$modal;
        }
        $('p.inline-error', scope).remove();
        return $('li.error', scope).removeClass('error');
      };

      QuickPentestModal.prototype.resetErrorsOnTab = function(tabIdx) {
        if (tabIdx == null) {
          tabIdx = this._tabIdx;
        }
        this.resetErrors($('form.quick_pentest>div.page', this.$el).eq(tabIdx));
        return this.tabAt(tabIdx).find('.hasErrors').hide();
      };

      QuickPentestModal.prototype.someInputChanged = function(e) {
        $('a.btn.primary', this.$modal).removeClass('disabled');
        return this._somethingHasChanged = true;
      };

      QuickPentestModal.prototype.handleErrors = function(errorsHash) {
        var modelsToTabs,
          _this = this;
        window.eh = errorsHash;
        $('.hasError', this.$modal).hide();
        modelsToTabs = {
          workspace: 0,
          "import": 1,
          scan_task: 1,
          nexpose_task: 1,
          exploit_task: 2,
          report_task: 3
        };
        return _.each(errorsHash.errors, function(modelErrors, modelName) {
          var $page, $tab, tabIdx;
          tabIdx = modelsToTabs[modelName];
          $page = _this.pageAt(tabIdx);
          if (typeof modelErrors === 'string' && $.trim(modelErrors).length > 0) {
            $tab = _this.tabAt(tabIdx);
            $tab.find('.hasErrors').show();
            $page.find('p.error-desc').remove();
            return $page.prepend($('<p>').addClass('error-desc').text(modelErrors));
          } else if (typeof modelErrors === 'object' && _.keys(modelErrors).length > 0) {
            $page.find('p.error-desc').remove();
            _this.tabAt(tabIdx).find('.hasErrors').show();
            return _.each(modelErrors, function(attrErrors, attrName) {
              var $li, inputName;
              inputName = "" + modelName + "[" + attrName + "]";
              $li = $("[name='" + inputName + "']", this.$modal).parents('li').first().addClass('error');
              return _.each(attrErrors, function(errorMsg) {
                return $("<p>").addClass('inline-error').appendTo($li).text(errorMsg);
              });
            });
          }
        });
      };

      QuickPentestModal.prototype.pageAt = function(idx) {
        return $('>form>.page', this.content()).eq(idx);
      };

      QuickPentestModal.prototype.formLoadedSuccessfully = function(html) {
        var _this = this;
        if (!this.content().is(':visible')) {
          return;
        }
        this.content().removeClass('loading').html(html);
        _.each(this.tabs(), function(t) {
          return $(t).removeClass('ui_disabled');
        });
        if ($("[name='quick_pentest[report_enabled]']", this.$modal).val()) {
          $("#tab_generate_report", this.$modal).attr('checked', true);
        }
        if ($("[name='quick_pentest[exploit_enabled]']", this.$modal).val()) {
          $("#tab_run_exploits", this.$modal).attr('checked', true);
        }
        $('form.quick_pentest>div.page', this.$el).hide().first().show();
        this.reportTypeChanged();
        $('#report_format_for_audit ol li label', this.$modal).removeAttr('for');
        this._steps = $.parseJSON($('meta[name=steps]', this.$modal).attr('content'));
        $('div.content form .page', this.$modal).each(function() {
          return $(this).append($('<div>').addClass('page-overlay'));
        });
        $('input:file', this.$modal).each(function() {
          var $label, $p, $span;
          $label = $(this).prev();
          $(this).attr('size', '50').css({
            overflow: 'hidden'
          });
          $p = $('<p>').text('Choose a file');
          $span = $('<span>').text($label.text());
          $label.html('').append($p).append($span);
          return $(this).change(function() {
            var path;
            path = $(this).val() || '';
            path = path.replace(/.*(\\|\/)/g, '');
            return $p.text(path);
          });
        });
        _.defer(this.layout);
        return _.defer(function() {
          return $('form :text:visible:first', _this.content()).focus();
        });
      };

      QuickPentestModal.prototype.index = function(idx) {
        var $allTabs, $currTab, $form, $li, $page, oldTabIdx,
          _this = this;
        $form = $('form.quick_pentest', this.$el);
        $allTabs = $('form.quick_pentest>div.page', this.$el);
        $currTab = $allTabs.eq(idx);
        this._somethingHasChanged = false;
        if (!(this._tabIdx < 0 || idx === this._tabIdx)) {
          oldTabIdx = this._tabIdx;
          this.resetErrorsOnTab(oldTabIdx);
          $li = $('ul.tabs>li', this.$modal).eq(this._tabIdx).addClass('tab-loading');
          $page = $('div.content form .page').eq(this._tabIdx).addClass('page-loading');
          $page.find('p.error-desc').remove();
          this.submitFormAjax($form, {
            noFiles: true,
            success: function() {
              $li.removeClass('tab-loading ui-disabled');
              $page.removeClass('page-loading');
              if (!($('.hasErrors:visible', _this.$modal).length > 0)) {
                return $('a.btn.primary', _this.$modal).removeClass('disabled');
              }
            },
            error: function() {
              $li.removeClass('tab-loading ui-disabled');
              $page.removeClass('page-loading');
              $('a.btn.primary', _this.$modal).addClass('disabled');
              return _this.handleErrors.apply(_this, arguments);
            }
          });
        }
        $allTabs.hide().eq(idx).show();
        return QuickPentestModal.__super__.index.apply(this, arguments);
      };

      QuickPentestModal.prototype.submitFormAjax = function($form, opts) {
        var data,
          _this = this;
        if (opts == null) {
          opts = {};
        }
        $('input[name=_method]', $form).not(':first').remove();
        $('input[name=_method]', $form).val(opts.method || 'PUT');
        $('input[name=no_files]', $form).val((opts.noFiles && '1') || '');
        data = $(':input', $form).not(':file').not('[type=checkbox]:not(:checked)').not(".report_formats>div:not(.active_report_option) *").not(".report_sections>div:not(.active_report_option) *").not(".report_options>div:not(.active_report_option) *");
        return $.ajax({
          url: opts['url'] || this._steps[this._tabIdx][1],
          type: 'POST',
          data: data.serializeArray(),
          processData: false,
          iframe: true,
          files: opts['noFiles'] ? $() : $(':file', $form),
          dataType: 'json',
          complete: function(resp) {
            try {
              data = $.parseJSON(resp.responseText);
            } catch (e) {
              alert('Error processing response from server. Please ensure Metasploit Pro is running.');
              return;
            }
            if (!data || !data['success']) {
              if (opts['error']) {
                return opts['error'].call(_this, data);
              }
            } else {
              if (opts['success']) {
                return opts['success'].call(_this, data);
              }
            }
          }
        });
      };

      return QuickPentestModal;

    })(this.TabbedModalView);
  });

}).call(this);



